@@ -1,4 +1,4 @@
-;;; tango-dark-theme.el --- Tango-based custom theme for faces
+;;; tango-dark-theme.el --- Tango-based custom theme for faces  -*- lexical-binding:t -*-
 
 ;; Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
@@ -20,7 +20,7 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
-;;; Commentary
+;;; Commentary:
 
 ;; The colors in this theme come from the Tango palette, which is in
 ;; the public domain: http://tango.freedesktop.org/
@@ -45,7 +45,9 @@ Semantic, and Ansi-Color faces are included.")
       (alum-4 "#888a85") (alum-5 "#555753") (alum-6 "#2e3436")
       ;; Not in Tango palette; used for better contrast.
       (cham-0 "#b4fa70") (blue-0 "#8cc4ff") (plum-0 "#e9b2e3")
-      (red-0 "#ff4b4b")  (alum-5.5 "#41423f") (alum-7 "#212526"))
+      (red-0 "#ff4b4b")  (alum-5.5 "#41423f") (alum-7 "#212526")
+      ;; Not in Tango palette; used for ANSI cyan.
+      (cyan-1 "#34e2e2") (cyan-2 "#06989a"))
 
   (custom-theme-set-faces
    'tango-dark
@@ -162,12 +164,31 @@ Semantic, and Ansi-Color faces are included.")
    `(semantic-decoration-on-unparsed-includes
      ((,class (:background ,alum-5.5))))
    `(semantic-tag-boundary-face ((,class (:overline ,blue-1))))
-   `(semantic-unmatched-syntax-face ((,class (:underline ,red-1)))))
-
-  (custom-theme-set-variables
-   'tango-dark
-   `(ansi-color-names-vector [,alum-7 ,red-0 ,cham-0 ,butter-1
-			      ,blue-1 ,plum-1 ,blue-0 ,alum-1])))
+   `(semantic-unmatched-syntax-face ((,class (:underline ,red-1))))
+   ;; ANSI colors
+   `(ansi-color-black ((,class (:background ,alum-7 :foreground ,alum-7))))
+   `(ansi-color-red ((,class (:background ,red-1 :foreground ,red-1))))
+   `(ansi-color-green ((,class (:background ,cham-2 :foreground ,cham-2))))
+   `(ansi-color-yellow ((,class (:background ,butter-2 :foreground ,butter-2))))
+   `(ansi-color-blue ((,class (:background ,blue-2 :foreground ,blue-2))))
+   `(ansi-color-magenta ((,class (:background ,plum-1 :foreground ,plum-1))))
+   `(ansi-color-cyan ((,class (:background ,cyan-2 :foreground ,cyan-2))))
+   `(ansi-color-white ((,class (:background ,alum-2 :foreground ,alum-2))))
+   `(ansi-color-bright-black ((,class (:background ,alum-5
+				       :foreground ,alum-5))))
+   `(ansi-color-bright-red ((,class (:background ,red-0 :foreground ,red-0))))
+   `(ansi-color-bright-green ((,class (:background ,cham-1
+				       :foreground ,cham-1))))
+   `(ansi-color-bright-yellow ((,class (:background ,butter-1
+					:foreground ,butter-1))))
+   `(ansi-color-bright-blue ((,class (:background ,blue-0
+				      :foreground ,blue-0))))
+   `(ansi-color-bright-magenta ((,class (:background ,plum-0
+					 :foreground ,plum-0))))
+   `(ansi-color-bright-cyan ((,class (:background ,cyan-1
+				      :foreground ,cyan-1))))
+   `(ansi-color-bright-white ((,class (:background ,alum-1
+				       :foreground ,alum-1))))))
 
 (provide-theme 'tango-dark)
 
