@@ -1,17 +1,25 @@
-;;; loaddefs.el --- automatically extracted autoloads
+;;; loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "srecode/compile" "compile.el" "acfe9477fc67d2685419372e0379975f")
+;;;### (autoloads nil "srecode/compile" "compile.el" "9aba38103472d3718e0f9ce914284cd9")
 ;;; Generated autoloads from compile.el
 
 (autoload 'srecode-compile-templates "srecode/compile" "\
 Compile a semantic recode template file into a mode-local variable." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/compile"
+;;;;;;  "compile.el" (0 0 0 0))
+;;; Generated autoloads from compile.el
+
+(register-definition-prefixes "srecode/compile" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/cpp" "cpp.el" "c4be246160d15c6e195753239ea23ccf")
+;;;### (autoloads nil "srecode/cpp" "cpp.el" "37737e35d96bc069a48baed794dca8ea")
 ;;; Generated autoloads from cpp.el
 
 (autoload 'srecode-semantic-handle-:c "srecode/cpp" "\
@@ -29,10 +37,18 @@ Also adds the following:
  - nothing -
 
 \(fn DICT)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/cpp"
+;;;;;;  "cpp.el" (0 0 0 0))
+;;; Generated autoloads from cpp.el
+
+(register-definition-prefixes "srecode/cpp" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/document" "document.el" "b9505a6389e0ac0eb6b5547aad67469a")
+;;;### (autoloads nil "srecode/document" "document.el" "0cf9345a4c644dadb54e62b9145ffe41")
 ;;; Generated autoloads from document.el
 
 (autoload 'srecode-document-insert-comment "srecode/document" "\
@@ -67,10 +83,18 @@ to the beginning and end of the tags on the region.
 If there is only one tag in the region, complain.
 
 \(fn BEG END)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/document"
+;;;;;;  "document.el" (0 0 0 0))
+;;; Generated autoloads from document.el
+
+(register-definition-prefixes "srecode/document" '("srecode-document-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/el" "el.el" "fe573232e53c303746b59252c4f8d278")
+;;;### (autoloads nil "srecode/el" "el.el" "d88b618531efd08ad5fb7fbef2cae73a")
 ;;; Generated autoloads from el.el
 
 (autoload 'srecode-semantic-handle-:el "srecode/el" "\
@@ -87,18 +111,34 @@ Adds the following:
   FACEGROUP - The `defgroup' name you might want for faces.
 
 \(fn DICT)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/el" "el.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from el.el
+
+(register-definition-prefixes "srecode/el" '("srecode-semantic-apply-tag-to-dict"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/expandproto" "expandproto.el" "4c423ff2122b7fd570323a02b62a3f86")
+;;;### (autoloads nil "srecode/expandproto" "expandproto.el" "da99a52e5f662d558ac706e22d02cf64")
 ;;; Generated autoloads from expandproto.el
 
 (autoload 'srecode-insert-prototype-expansion "srecode/expandproto" "\
 Insert get/set methods for the current class." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/expandproto"
+;;;;;;  "expandproto.el" (0 0 0 0))
+;;; Generated autoloads from expandproto.el
+
+(register-definition-prefixes "srecode/expandproto" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/getset" "getset.el" "71f8f03a3b169611a734b6f6b30b40f4")
+;;;### (autoloads nil "srecode/getset" "getset.el" "b327536a49d496afa16dce9c62d5810e")
 ;;; Generated autoloads from getset.el
 
 (autoload 'srecode-insert-getset "srecode/getset" "\
@@ -109,10 +149,18 @@ If you do not specify CLASS-IN or FIELD-IN then a class and field
 will be derived.
 
 \(fn &optional CLASS-IN FIELD-IN)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/getset"
+;;;;;;  "getset.el" (0 0 0 0))
+;;; Generated autoloads from getset.el
+
+(register-definition-prefixes "srecode/getset" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/insert" "insert.el" "8e63c3e8ea1ba8c7f4b7ee654b237f56")
+;;;### (autoloads nil "srecode/insert" "insert.el" "82b06594b6d336e43878d6f3f6770676")
 ;;; Generated autoloads from insert.el
 
 (autoload 'srecode-insert "srecode/insert" "\
@@ -120,10 +168,18 @@ Insert the template TEMPLATE-NAME into the current buffer at point.
 DICT-ENTRIES are additional dictionary values to add.
 
 \(fn TEMPLATE-NAME &rest DICT-ENTRIES)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/insert"
+;;;;;;  "insert.el" (0 0 0 0))
+;;; Generated autoloads from insert.el
+
+(register-definition-prefixes "srecode/insert" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/java" "java.el" "df96f0ed4ee77c125c883e8ecfc8112e")
+;;;### (autoloads nil "srecode/java" "java.el" "e1285e2b56c4c9e64592da08635d91da")
 ;;; Generated autoloads from java.el
 
 (autoload 'srecode-semantic-handle-:java "srecode/java" "\
@@ -136,7 +192,7 @@ FILENAME_AS_CLASS - file converted to a Java class name.
 
 ;;;***
 
-;;;### (autoloads nil "srecode/map" "map.el" "5a06a260a31e329f81a62580fec53d4e")
+;;;### (autoloads nil "srecode/map" "map.el" "7377f16fa776290bdb6d3556078eee69")
 ;;; Generated autoloads from map.el
 
 (autoload 'srecode-get-maps "srecode/map" "\
@@ -144,19 +200,36 @@ Get a list of maps relevant to the current buffer.
 Optional argument RESET forces a reset of the current map.
 
 \(fn &optional RESET)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/map"
+;;;;;;  "map.el" (0 0 0 0))
+;;; Generated autoloads from map.el
+
+(register-definition-prefixes "srecode/map" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/mode" "mode.el" "92839df0bdfeadecc49843802024ab78")
+;;;### (autoloads nil "srecode/mode" "mode.el" "548503a8382761f646e1975cbddfa678")
 ;;; Generated autoloads from mode.el
 
 (autoload 'srecode-minor-mode "srecode/mode" "\
 Toggle srecode minor mode.
 
-If called interactively, enable Srecode minor mode if ARG is positive,
-and disable it if ARG is zero or negative.  If called from Lisp, also
-enable the mode if ARG is omitted or nil, and toggle it if ARG is
-`toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the `Srecode
+minor mode' mode.  If the prefix argument is positive, enable the
+mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable the
+mode if ARG is nil, omitted, or is a positive number.  Disable the
+mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `srecode-minor-mode'.
+
+The mode's hook is called both when the mode is enabled and when it is
+disabled.
 
 The minor mode can be turned on only if semantic feature is
 available and the current buffer was set up for parsing.  Return
@@ -179,16 +252,33 @@ or call the function `global-srecode-minor-mode'.")
 (autoload 'global-srecode-minor-mode "srecode/mode" "\
 Toggle global use of srecode minor mode.
 
-If called interactively, enable Global Srecode minor mode if ARG is
-positive, and disable it if ARG is zero or negative.  If called from
-Lisp, also enable the mode if ARG is omitted or nil, and toggle it if
-ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the `Global
+Srecode minor mode' mode.  If the prefix argument is positive, enable
+the mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable the
+mode if ARG is nil, omitted, or is a positive number.  Disable the
+mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `(default-value \\='global-srecode-minor-mode)'.
+
+The mode's hook is called both when the mode is enabled and when it is
+disabled.
 
 \(fn &optional ARG)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/mode"
+;;;;;;  "mode.el" (0 0 0 0))
+;;; Generated autoloads from mode.el
+
+(register-definition-prefixes "srecode/mode" '("srecode-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/srt" "srt.el" "73efd15bab1769a126c57ce3354b484a")
+;;;### (autoloads nil "srecode/srt" "srt.el" "cae9e2dda010152bc0aa6b64d3e05352")
 ;;; Generated autoloads from srt.el
 
 (autoload 'srecode-semantic-handle-:srt "srecode/srt" "\
@@ -199,18 +289,34 @@ ESCAPE_END - This files value of escape_end
 MODE - The mode of this buffer.  If not declared yet, guess.
 
 \(fn DICT)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/srt"
+;;;;;;  "srt.el" (0 0 0 0))
+;;; Generated autoloads from srt.el
+
+(register-definition-prefixes "srecode/srt" '("srecode-read-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/template" "template.el" "aca79ccd6134a23439bfd9739031bcaa")
+;;;### (autoloads nil "srecode/template" "template.el" "091ceedcfda7f99cf941dc9121fd05c4")
 ;;; Generated autoloads from template.el
 
 (autoload 'srecode-template-setup-parser "srecode/template" "\
 Setup buffer for parse." nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/template"
+;;;;;;  "template.el" (0 0 0 0))
+;;; Generated autoloads from template.el
+
+(register-definition-prefixes "srecode/template" '("semantic-tag-components"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "srecode/texi" "texi.el" "d5593a42d6e9bae1fc490ba533856d82")
+;;;### (autoloads nil "srecode/texi" "texi.el" "4d73e7f7f00747b4ce2a1c3389e38518")
 ;;; Generated autoloads from texi.el
 
 (autoload 'srecode-semantic-handle-:texi "srecode/texi" "\
@@ -227,6 +333,14 @@ Adds the following:
   TAGDOC - Texinfo formatted doc string for :tag.
 
 \(fn DICT)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "srecode/texi"
+;;;;;;  "texi.el" (0 0 0 0))
+;;; Generated autoloads from texi.el
+
+(register-definition-prefixes "srecode/texi" '("semantic-insert-foreign-tag" "srecode-texi-"))
+
+;;;***
 
 ;;;***
 
