@@ -1,7 +1,6 @@
-;;; completion.el --- dynamic word-completion code
+;;; completion.el --- dynamic word-completion code  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1990, 1993, 1995, 1997, 2001-2021 Free Software
-;; Foundation, Inc.
+;; Copyright (C) 1990-2021 Free Software Foundation, Inc.
 
 ;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: abbrev convenience
@@ -179,7 +178,7 @@
 ;;    Inserts a completion at point
 ;;
 ;;  completion-initialize
-;;    Loads the completions file and sets up so that exiting emacs will
+;;    Loads the completions file and sets up so that exiting Emacs will
 ;;  save them.
 ;;
 ;;  save-completions-to-file &optional filename
@@ -208,7 +207,7 @@
 ;;   Add package prefix smarts (for Common Lisp)
 ;;   Add autoprompting of possible completions after every keystroke (fast
 ;;      terminals only !)
-;;   Add doc. to texinfo
+;;   Add documentation to texinfo
 ;;
 ;;
 ;;-----------------------------------------------
@@ -286,62 +285,52 @@
 (defcustom enable-completion t
   "Non-nil means enable recording and saving of completions.
 If nil, no new words are added to the database or saved to the init file."
-  :type 'boolean
-  :group 'completion)
+  :type 'boolean)
 
 (defcustom save-completions-flag t
   "Non-nil means save most-used completions when exiting Emacs.
 See also `save-completions-retention-time'."
-  :type 'boolean
-  :group 'completion)
+  :type 'boolean)
 
 (defcustom save-completions-file-name
   (locate-user-emacs-file "completions" ".completions")
   "The filename to save completions to."
-  :type 'file
-  :group 'completion)
+  :type 'file)
 
 (defcustom save-completions-retention-time 336
   "Discard a completion if unused for this many hours.
 \(1 day = 24, 1 week = 168).  If this is 0, non-permanent completions
 will not be saved unless these are used.  Default is two weeks."
-  :type 'integer
-  :group 'completion)
+  :type 'integer)
 
 (defcustom completion-on-separator-character nil
   "Non-nil means separator characters mark previous word as used.
 This means the word will be saved as a completion."
-  :type 'boolean
-  :group 'completion)
+  :type 'boolean)
 
 (defcustom completions-file-versions-kept kept-new-versions
   "Number of versions to keep for the saved completions file."
-  :type 'integer
-  :group 'completion)
+  :type 'integer)
 
 (defcustom completion-prompt-speed-threshold 4800
   "Minimum output speed at which to display next potential completion."
-  :type 'integer
-  :group 'completion)
+  :type 'integer)
 
 (defcustom completion-cdabbrev-prompt-flag nil
   "If non-nil, the next completion prompt does a cdabbrev search.
 This can be time consuming."
-  :type 'boolean
-  :group 'completion)
+  :type 'boolean)
 
 (defcustom completion-search-distance 15000
   "How far to search in the buffer when looking for completions.
 In number of characters.  If nil, search the whole buffer."
-  :type 'integer
-  :group 'completion)
+  :type 'integer)
 
 (defcustom completions-merging-modes '(lisp c)
   "List of modes {`c' or `lisp'} for automatic completions merging.
 Definitions from visited files which have these modes
 are automatically added to the completion database."
-  :type '(set (const lisp) (const c))
-  :group 'completion)
+  :type '(set (const lisp) (const c)))
 
 ;;(defvar *completion-auto-save-period* 1800
 ;;  "The period in seconds to wait for emacs to be idle before autosaving
@@ -399,13 +388,6 @@ Used to decide whether to save completions.")
 		  :up)
 		 (t :neither))))))
 
-;; Tests -
-;; (cmpl-string-case-type "123ABCDEF456") --> :up
-;; (cmpl-string-case-type "123abcdef456") --> :down
-;; (cmpl-string-case-type "123aBcDeF456") --> :mixed
-;; (cmpl-string-case-type "123456")       --> :neither
-;; (cmpl-string-case-type "Abcde123")     --> :capitalized
-
 (defun cmpl-coerce-string-case (string case-type)
   (cond ((eq case-type :down) (downcase string))
 	((eq case-type :up) (upcase string))
@@ -424,12 +406,6 @@ Used to decide whether to save completions.")
 	   ;; as is
 	   string-to-coerce))))
 
-;; Tests -
-;; (cmpl-merge-string-cases "AbCdEf456" "abc")     --> AbCdEf456
-;; (cmpl-merge-string-cases "abcdef456" "ABC")     --> ABCDEF456
-;; (cmpl-merge-string-cases "ABCDEF456" "Abc")     --> Abcdef456
-;; (cmpl-merge-string-cases "ABCDEF456" "abc")     --> abcdef456
-
 
 (defun cmpl-hours-since-origin ()
   (floor (time-convert nil 'integer) 3600))
@@ -518,9 +494,8 @@ Used to decide whether to save completions.")
 ;; Old name, non-namespace-clean.
 (defvaralias 'cmpl-syntax-table 'completion-syntax-table)
 
-(defvar completion-syntax-table completion-standard-syntax-table
+(defvar-local completion-syntax-table completion-standard-syntax-table
   "This variable holds the current completion syntax table.")
-(make-variable-buffer-local 'completion-syntax-table)
 
 ;;-----------------------------------------------
 ;; Symbol functions
@@ -964,9 +939,9 @@ Each symbol is bound to a single completion entry.")
 
 ;; READER Macros
 
-(defalias 'cmpl-prefix-entry-head 'car)
+(defalias 'cmpl-prefix-entry-head #'car)
 
-(defalias 'cmpl-prefix-entry-tail 'cdr)
+(defalias 'cmpl-prefix-entry-tail #'cdr)
 
 ;; WRITER Macros
 
@@ -992,31 +967,27 @@ Each symbol is bound to a single completion entry.")
   (setq cmpl-prefix-obarray (make-vector cmpl-obarray-length 0))
   (setq cmpl-obarray (make-vector cmpl-obarray-length 0)))
 
-(defvar completions-list-return-value)
-
 (defun list-all-completions ()
   "Return a list of all the known completion entries."
-  (let ((completions-list-return-value nil))
-    (mapatoms 'list-all-completions-1 cmpl-prefix-obarray)
-    completions-list-return-value))
+  (let ((return-value nil))
+    (mapatoms (lambda (prefix-symbol)
+                (if (boundp prefix-symbol)
+                    (setq return-value
+	                  (append (cmpl-prefix-entry-head
+		                   (symbol-value prefix-symbol))
+		                  return-value))))
+	      cmpl-prefix-obarray)
+    return-value))
 
-(defun list-all-completions-1 (prefix-symbol)
-  (if (boundp prefix-symbol)
-      (setq completions-list-return-value
-	    (append (cmpl-prefix-entry-head (symbol-value prefix-symbol))
-		    completions-list-return-value))))
-
-(defun list-all-completions-by-hash-bucket ()
+(defun list-all-completions-by-hash-bucket () ;FIXME: Unused!
   "Return list of lists of known completion entries, organized by hash bucket."
-  (let ((completions-list-return-value nil))
-    (mapatoms 'list-all-completions-by-hash-bucket-1 cmpl-prefix-obarray)
-    completions-list-return-value))
-
-(defun list-all-completions-by-hash-bucket-1 (prefix-symbol)
-  (if (boundp prefix-symbol)
-      (setq completions-list-return-value
-	    (cons (cmpl-prefix-entry-head (symbol-value prefix-symbol))
-		  completions-list-return-value))))
+  (let ((return-value nil))
+    (mapatoms (lambda (prefix-symbol)
+                (if (boundp prefix-symbol)
+                    (push (cmpl-prefix-entry-head (symbol-value prefix-symbol))
+		          return-value)))
+	      cmpl-prefix-obarray)
+    return-value))
 
 
 ;;-----------------------------------------------
@@ -1117,7 +1088,8 @@ Must be called after `find-exact-completion'."
   #'completion-locate-db-error "27.1")
 (defun completion-locate-db-error ()
   ;; recursive error: really scrod
-  (error "Completion database corrupted.  Try M-x clear-all-completions.  Send bug report"))
+  (error (substitute-command-keys
+          "Completion database corrupted.  Try \\[clear-all-completions].  Send bug report")))
 
 ;; WRITES
 (defun add-completion-to-tail-if-new (string)
@@ -1226,45 +1198,6 @@ String must be longer than `completion-prefix-min-length'."
                      (set cmpl-db-prefix-symbol nil)))))
       (error "Unknown completion `%s'" completion-string))))
 
-;; Tests --
-;;  - Add and Find -
-;; (add-completion-to-head "banana")     --> ("banana" 0 nil 0)
-;; (find-exact-completion "banana")      --> ("banana" 0 nil 0)
-;; (find-exact-completion "bana")        --> nil
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banana" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))  --> (("banana" ...))
-;; (add-completion-to-head "banish")     --> ("banish" 0 nil 0)
-;; (find-exact-completion "banish")      --> ("banish" 0 nil 0)
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banish" ...) ("banana" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))  --> (("banana" ...))
-;; (add-completion-to-head "banana")     --> ("banana" 0 nil 0)
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banana" ...) ("banish" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))  --> (("banish" ...))
-;;
-;;  - Deleting -
-;; (add-completion-to-head "banner")     --> ("banner" 0 nil 0)
-;; (delete-completion "banner")
-;; (find-exact-completion "banner")      --> nil
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banana" ...) ("banish" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))  --> (("banish" ...))
-;; (add-completion-to-head "banner")     --> ("banner" 0 nil 0)
-;; (delete-completion "banana")
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banner" ...) ("banish" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))  --> (("banish" ...))
-;; (delete-completion "banner")
-;; (delete-completion "banish")
-;; (find-cmpl-prefix-entry "ban")        --> nil
-;; (delete-completion "banner")          --> error
-;;
-;; - Tail -
-;; (add-completion-to-tail-if-new "banana") --> ("banana" 0 nil 0)
-;; (car (find-cmpl-prefix-entry "ban"))     --> (("banana" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))     --> (("banana" ...))
-;; (add-completion-to-tail-if-new "banish") --> ("banish" 0 nil 0)
-;; (car (find-cmpl-prefix-entry "ban"))     -->(("banana" ...) ("banish" ...))
-;; (cdr (find-cmpl-prefix-entry "ban"))     -->(("banish" ...))
-;;
-
 
 ;;---------------------------------------------------------------------------
 ;; Database Update :: Interface level routines
@@ -1276,11 +1209,7 @@ String must be longer than `completion-prefix-min-length'."
 
 (defun interactive-completion-string-reader (prompt)
   (let* ((default (symbol-under-or-before-point))
-	 (new-prompt
-	  (if default
-	      (format "%s (default %s): " prompt default)
-	      (format "%s: " prompt)))
-	 (read (completing-read new-prompt cmpl-obarray)))
+	 (read (completing-read (format-prompt prompt default) cmpl-obarray)))
     (if (zerop (length read)) (setq read (or default "")))
     (list read)))
 
@@ -1365,29 +1294,6 @@ Completions added this way will automatically be saved if
 		 (set-completion-num-uses entry 1)
 		 (setq cmpl-completions-accepted-p t)))))))
 
-;; Tests --
-;;  - Add and Find -
-;; (add-completion "banana" 5 10)
-;; (find-exact-completion "banana")  --> ("banana" 5 10 0)
-;; (add-completion "banana" 6)
-;; (find-exact-completion "banana")  --> ("banana" 6 10 0)
-;; (add-completion "banish")
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banish" ...) ("banana" ...))
-;;
-;;  - Accepting -
-;; (setq completion-to-accept "banana")
-;; (accept-completion)
-;; (find-exact-completion "banana")      --> ("banana" 7 10)
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banana" ...) ("banish" ...))
-;; (setq completion-to-accept "banish")
-;; (add-completion "banner")
-;; (car (find-cmpl-prefix-entry "ban"))
-;;        --> (("banner" ...) ("banish" 1 ...) ("banana" 7 ...))
-;;
-;;  - Deleting -
-;; (kill-completion "banish")
-;; (car (find-cmpl-prefix-entry "ban"))  --> (("banner" ...) ("banana" ...))
-
 
 ;;---------------------------------------------------------------------------
 ;; Searching the database
@@ -1509,46 +1415,6 @@ If there are no more entries, try cdabbrev and then return only a string."
     ;; Completely unsuccessful, return nil
     ))
 
-;; Tests --
-;;  - Add and Find -
-;; (add-completion "banana")
-;; (completion-search-reset "ban")
-;; (completion-search-next 0)        --> "banana"
-;;
-;;  - Discrimination -
-;; (add-completion "cumberland")
-;; (add-completion "cumberbund")
-;; cumbering
-;; (completion-search-reset "cumb")
-;; (completion-search-peek t)        --> "cumberbund"
-;; (completion-search-next 0)        --> "cumberbund"
-;; (completion-search-peek t)        --> "cumberland"
-;; (completion-search-next 1)        --> "cumberland"
-;; (completion-search-peek nil)      --> nil
-;; (completion-search-next 2)        --> "cumbering"  {cdabbrev}
-;; (completion-search-next 3)        -->  nil or "cumming"{depends on context}
-;; (completion-search-next 1)        --> "cumberland"
-;; (completion-search-peek t)        --> "cumbering"  {cdabbrev}
-;;
-;;  - Accepting -
-;; (completion-search-next 1)        --> "cumberland"
-;; (setq completion-to-accept "cumberland")
-;; (completion-search-reset "foo")
-;; (completion-search-reset "cum")
-;; (completion-search-next 0)        --> "cumberland"
-;;
-;;  - Deleting -
-;; (kill-completion "cumberland")
-;; cummings
-;; (completion-search-reset "cum")
-;; (completion-search-next 0)        --> "cumberbund"
-;; (completion-search-next 1)        --> "cummings"
-;;
-;;  - Ignoring Capitalization -
-;; (completion-search-reset "CuMb")
-;; (completion-search-next 0)            --> "cumberbund"
-
-
 
 ;;-----------------------------------------------
 ;; COMPLETE
@@ -1737,12 +1603,6 @@ Prefix args ::
   "\n(\\(\\w*:\\)?def\\(\\w\\|\\s_\\)*\\s +(*"
   "A regexp that searches for Lisp definition form.")
 
-;; Tests -
-;;  (and (string-match *lisp-def-regexp* "\n(defun foo") (match-end 0)) -> 8
-;;  (and (string-match *lisp-def-regexp* "\n(si:def foo") (match-end 0)) -> 9
-;;  (and (string-match *lisp-def-regexp* "\n(def-bar foo")(match-end 0)) -> 10
-;;  (and (string-match *lisp-def-regexp* "\n(defun (foo") (match-end 0)) -> 9
-
 ;; Parses all the definition names from a Lisp mode buffer and adds them to
 ;; the completion database.
 (defun add-completions-from-lisp-buffer ()
@@ -2058,68 +1918,64 @@ If file is not specified, then use `save-completions-file-name'."
 	    (clear-visited-file-modtime)
 	    (erase-buffer)
 
-	    (let ((insert-okay-p nil)
-		  (buffer (current-buffer))
+	    (let ((buffer (current-buffer))
 		  string entry last-use-time
 		  cmpl-entry cmpl-last-use-time
 		  (current-completion-source cmpl-source-init-file)
 		  (total-in-file 0) (total-perm 0))
 	      ;; insert the file into a buffer
 	      (condition-case nil
-		  (progn (insert-file-contents filename t)
-			 (setq insert-okay-p t))
-
+		  (insert-file-contents filename t)
 		(file-error
 		 (message "File error trying to load completion file %s."
-			  filename)))
-	      ;; parse it
-	      (if insert-okay-p
-		  (progn
-		    (goto-char (point-min))
+			  filename))
+                (:success
+	         ;; parse it
+		 (goto-char (point-min))
 
-		    (condition-case nil
-			(while t
-			  (setq entry (read buffer))
-			  (setq total-in-file (1+ total-in-file))
-			  (cond
-			   ((and (consp entry)
-				 (stringp (setq string (car entry)))
-				 (cond
-				  ((eq (setq last-use-time (cdr entry)) 'T)
-				   ;; handle case sensitivity
-				   (setq total-perm (1+ total-perm))
-				   (setq last-use-time t))
-				  ((eq last-use-time t)
-				   (setq total-perm (1+ total-perm)))
-				  ((integerp last-use-time))))
-			    ;; Valid entry
-			    ;; add it in
-			    (setq cmpl-last-use-time
-				  (completion-last-use-time
-				   (setq cmpl-entry
-					 (add-completion-to-tail-if-new string))))
-			    (if (or (eq last-use-time t)
-				    (and (> last-use-time 1000);;backcompatibility
-					 (not (eq cmpl-last-use-time t))
-					 (or (not cmpl-last-use-time)
-					     ;; more recent
-					     (> last-use-time  cmpl-last-use-time))))
-				;; update last-use-time
-				(set-completion-last-use-time cmpl-entry last-use-time)))
-			   (t
-			    ;; Bad format
-			    (message "Error: invalid saved completion - %s"
-				     (prin1-to-string entry))
-			    ;; try to get back in sync
-			    (search-forward "\n("))))
-		      (search-failed
-		       (message "End of file while reading completions."))
-		      (end-of-file
-		       (if (= (point) (point-max))
-			   (if (not no-message-p)
-			       (message "Loading completions from file %s . . . Done."
-					filename))
-			 (message "End of file while reading completions."))))))
+		 (condition-case nil
+		     (while t
+		       (setq entry (read buffer))
+		       (setq total-in-file (1+ total-in-file))
+		       (cond
+			((and (consp entry)
+			      (stringp (setq string (car entry)))
+			      (cond
+			       ((eq (setq last-use-time (cdr entry)) 'T)
+				;; handle case sensitivity
+				(setq total-perm (1+ total-perm))
+				(setq last-use-time t))
+			       ((eq last-use-time t)
+				(setq total-perm (1+ total-perm)))
+			       ((integerp last-use-time))))
+			 ;; Valid entry
+			 ;; add it in
+			 (setq cmpl-last-use-time
+			       (completion-last-use-time
+				(setq cmpl-entry
+				      (add-completion-to-tail-if-new string))))
+			 (if (or (eq last-use-time t)
+				 (and (> last-use-time 1000);;backcompatibility
+				      (not (eq cmpl-last-use-time t))
+				      (or (not cmpl-last-use-time)
+					  ;; more recent
+					  (> last-use-time  cmpl-last-use-time))))
+			     ;; update last-use-time
+			     (set-completion-last-use-time cmpl-entry last-use-time)))
+			(t
+			 ;; Bad format
+			 (message "Error: invalid saved completion - %s"
+				  (prin1-to-string entry))
+			 ;; try to get back in sync
+			 (search-forward "\n("))))
+		   (search-failed
+		    (message "End of file while reading completions."))
+		   (end-of-file
+		    (if (= (point) (point-max))
+			(if (not no-message-p)
+			    (message "Loading completions from file %s . . . Done."
+				     filename))
+		      (message "End of file while reading completions."))))))
 ))))))
 
 (defun completion-initialize ()
@@ -2166,7 +2022,7 @@ Patched to remove the most recent completion."
 
 ;; All common separators (eg. space "(" ")" """) characters go through a
 ;; function to add new words to the list of words to complete from.
-;; If the character before this was an alpha-numeric then this adds the
+;; If the character before this was an alphanumeric then this adds the
 ;; symbol before point to the completion list (using ADD-COMPLETION).
 
 (defvar completion-separator-chars
@@ -2281,7 +2137,6 @@ TYPE is the type of the wrapper to be added.  Can be :before or :under."
 (define-minor-mode dynamic-completion-mode
   "Toggle dynamic word-completion on or off."
   :global t
-  :group 'completion
   ;; This is always good, not specific to dynamic-completion-mode.
   (define-key function-key-map [C-return] [?\C-\r])
 
@@ -2365,7 +2220,7 @@ TYPE is the type of the wrapper to be added.  Can be :before or :under."
 (completion-def-wrapper 'delete-backward-char-untabify :backward)
 
 ;; Old name, non-namespace-clean.
-(defalias 'initialize-completions 'completion-initialize)
+(defalias 'initialize-completions #'completion-initialize)
 
 (provide 'completion)
 
