@@ -1,9 +1,9 @@
-;;; dired-loaddefs.el --- automatically extracted autoloads
+;;; dired-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "dired-aux" "dired-aux.el" "9615913c4310a73aca254ec4f179d3be")
+;;;### (autoloads nil "dired-aux" "dired-aux.el" "2c1c12306a7969464e928c947383d9b3")
 ;;; Generated autoloads from dired-aux.el
 
 (autoload 'dired-diff "dired-aux" "\
@@ -65,6 +65,9 @@ Examples of PREDICATE:
               (= (file-attribute-group-id fa1) - and GID.
                  (file-attribute-group-id fa2))))
 
+If the region is active in Transient Mark mode, mark files
+only in the active region if `dired-mark-region' is non-nil.
+
 \(fn DIR2 PREDICATE)" t nil)
 
 (autoload 'dired-do-chmod "dired-aux" "\
@@ -74,8 +77,7 @@ are supported.  Type M-n to pull the file attributes of the file
 at point into the minibuffer.
 
 See Info node `(coreutils)File permissions' for more information.
-Alternatively, see the man page for \"chmod\", using the command
-\\[man] in Emacs.
+Alternatively, see the man page for \"chmod(1)\".
 
 Note that on MS-Windows only the `w' (write) bit is meaningful:
 resetting it makes the file read-only.  Changing any other bit
@@ -137,7 +139,7 @@ are executed in the background on each file sequentially waiting
 for each command to terminate before running the next command.
 In shell syntax this means separating the individual commands with `;'.
 
-The output appears in the buffer `*Async Shell Command*'.
+The output appears in the buffer named by `shell-command-buffer-name-async'.
 
 \(fn COMMAND &optional ARG FILE-LIST)" t nil)
 
@@ -166,16 +168,16 @@ it, write `*\"\"' in place of just `*'.  This is equivalent to just
 `*' in the shell, but avoids Dired's special handling.
 
 If COMMAND ends in `&', `;', or `;&', it is executed in the
-background asynchronously, and the output appears in the buffer
-`*Async Shell Command*'.  When operating on multiple files and COMMAND
-ends in `&', the shell command is executed on each file in parallel.
-However, when COMMAND ends in `;' or `;&' then commands are executed
-in the background on each file sequentially waiting for each command
-to terminate before running the next command.  You can also use
-`dired-do-async-shell-command' that automatically adds `&'.
+background asynchronously, and the output appears in the buffer named
+by `shell-command-buffer-name-async'.  When operating on multiple files
+and COMMAND ends in `&', the shell command is executed on each file
+in parallel.  However, when COMMAND ends in `;' or `;&', then commands
+are executed in the background on each file sequentially waiting for
+each command to terminate before running the next command.  You can
+also use `dired-do-async-shell-command' that automatically adds `&'.
 
 Otherwise, COMMAND is executed synchronously, and the output
-appears in the buffer `*Shell Command Output*'.
+appears in the buffer named by `shell-command-buffer-name'.
 
 This feature does not try to redisplay Dired buffers afterward, as
 there's no telling what files COMMAND may have changed.
@@ -204,14 +206,24 @@ prompted for the shell command to use interactively.
 Kill all marked lines (not the files).
 With a prefix argument, kill that many lines starting with the current line.
 \(A negative argument kills backward.)
+
 If you use this command with a prefix argument to kill the line
 for a file that is a directory, which you have inserted in the
 Dired buffer as a subdirectory, then it deletes that subdirectory
 from the buffer as well.
+
 To kill an entire subdirectory (without killing its line in the
 parent directory), go to its directory header line and use this
 command with a prefix argument (the value does not matter).
 
+To undo the killing, the undo command can be used as normally.
+
+This function returns the number of killed lines.
+
+FMT is a format string used for messaging the user about the
+killed lines, and defaults to \"Killed %d line%s.\" if not
+present.  A FMT of \"\" will suppress the messaging.
+
 \(fn &optional ARG FMT)" t nil)
 
 (autoload 'dired-do-compress-to "dired-aux" "\
@@ -282,7 +294,9 @@ See Info node `(emacs)Subdir switches' for more details.
 \(fn FILENAME &optional MARKER-CHAR)" nil nil)
 
 (autoload 'dired-remove-file "dired-aux" "\
-
+Remove entry FILE on each dired buffer.
+Note this doesn't delete FILE in the file system.
+See `dired-delete-file' in case you wish that.
 
 \(fn FILE)" nil nil)
 
@@ -297,7 +311,9 @@ Create or update the line for FILE in all Dired buffers it would belong in.
 \(fn FROM TO OK-FLAG)" nil nil)
 
 (autoload 'dired-rename-file "dired-aux" "\
-
+Rename FILE to NEWNAME.
+Signal a `file-already-exists' error if a file NEWNAME already exists
+unless OK-IF-ALREADY-EXISTS is non-nil.
 
 \(fn FILE NEWNAME OK-IF-ALREADY-EXISTS)" nil nil)
 
@@ -310,14 +326,17 @@ If DIRECTORY already exists, signal an error.
 
 (autoload 'dired-create-empty-file "dired-aux" "\
 Create an empty file called FILE.
- Add a new entry for the new file in the Dired buffer.
- Parent directories of FILE are created as needed.
- If FILE already exists, signal an error.
+Add a new entry for the new file in the Dired buffer.
+Parent directories of FILE are created as needed.
+If FILE already exists, signal an error.
 
 \(fn FILE)" t nil)
 
 (autoload 'dired-do-copy "dired-aux" "\
 Copy all marked (or next ARG) files, or copy the current file.
+ARG has to be numeric for above functionality.  See
+`dired-get-marked-files' for more details.
+
 When operating on just the current file, prompt for the new name.
 
 When operating on multiple or marked files, prompt for a target
@@ -331,8 +350,15 @@ If `dired-copy-preserve-time' is non-nil, this command preserves
 the modification time of each old file in the copy, similar to
 the \"-p\" option for the \"cp\" shell command.
 
-This command copies symbolic links by creating new ones, similar
-to the \"-d\" option for the \"cp\" shell command.
+This command copies symbolic links by creating new ones,
+similar to the \"-d\" option for the \"cp\" shell command.
+But if `dired-copy-dereference' is non-nil, the symbolic
+links are dereferenced and then copied, similar to the \"-L\"
+option for the \"cp\" shell command.  If ARG is a cons with
+element 4 (`\\[universal-argument]'), the inverted value of
+`dired-copy-dereference' will be used.
+
+Also see `dired-do-revert-buffer'.
 
 \(fn &optional ARG)" t nil)
 
@@ -347,6 +373,8 @@ suggested for the target directory depends on the value of
 
 For relative symlinks, use \\[dired-do-relsymlink].
 
+Also see `dired-do-revert-buffer'.
+
 \(fn &optional ARG)" t nil)
 
 (autoload 'dired-do-hardlink "dired-aux" "\
@@ -358,6 +386,8 @@ with the same names that the files currently have.  The default
 suggested for the target directory depends on the value of
 `dired-dwim-target', which see.
 
+Also see `dired-do-revert-buffer'.
+
 \(fn &optional ARG)" t nil)
 
 (autoload 'dired-do-rename "dired-aux" "\
@@ -368,6 +398,8 @@ This command also renames any buffers that are visiting the files.
 The default suggested for the target directory depends on the value
 of `dired-dwim-target', which see.
 
+Also see `dired-do-revert-buffer'.
+
 \(fn &optional ARG)" t nil)
 
 (autoload 'dired-do-rename-regexp "dired-aux" "\
@@ -562,27 +594,64 @@ instead.
 
 \(fn FILE &optional DEREF-SYMLINKS)" t nil)
 
+(autoload 'dired-vc-next-action "dired-aux" "\
+Do the next version control operation on marked files/directories.
+When only files are marked then call `vc-next-action' with the
+same value of the VERBOSE argument.
+When also directories are marked then call `vc-dir' and mark
+the same files/directories in the VC-Dir buffer that were marked
+in the Dired buffer.
+
+\(fn VERBOSE)" t nil)
+
+(autoload 'dired-vc-deduce-fileset "dired-aux" "\
+
+
+\(fn &optional STATE-MODEL-ONLY-FILES NOT-STATE-CHANGING)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "dired-aux" "dired-aux.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from dired-aux.el
+
+(register-definition-prefixes "dired-aux" '("dired-" "minibuffer-default-add-dired-shell-commands"))
+
+;;;***
+
 ;;;***
 
-;;;### (autoloads nil "dired-x" "dired-x.el" "447ae37c65bb1e1ced3e92b7111af0f2")
+;;;### (autoloads nil "dired-x" "dired-x.el" "b330b9830bbdddfd49549c7a64b62a53")
 ;;; Generated autoloads from dired-x.el
 
-(autoload 'dired-jump "dired-x" "\
-Jump to Dired buffer corresponding to current buffer.
-If in a file, Dired the current directory and move to file's line.
-If in Dired already, pop up a level and goto old directory's line.
-In case the proper Dired file line cannot be found, refresh the dired
-buffer and try again.
-When OTHER-WINDOW is non-nil, jump to Dired buffer in other window.
-When FILE-NAME is non-nil, jump to its line in Dired.
-Interactively with prefix argument, read FILE-NAME.
+(autoload 'dired-omit-mode "dired-x" "\
+Toggle omission of uninteresting files in Dired (Dired-Omit mode).
 
-\(fn &optional OTHER-WINDOW FILE-NAME)" t nil)
+This is a minor mode.  If called interactively, toggle the `Dired-Omit
+mode' mode.  If the prefix argument is positive, enable the mode, and
+if it is zero or negative, disable the mode.
 
-(autoload 'dired-jump-other-window "dired-x" "\
-Like \\[dired-jump] (`dired-jump') but in other window.
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable the
+mode if ARG is nil, omitted, or is a positive number.  Disable the
+mode if ARG is a negative number.
 
-\(fn &optional FILE-NAME)" t nil)
+To check whether the minor mode is enabled in the current buffer,
+evaluate `dired-omit-mode'.
+
+The mode's hook is called both when the mode is enabled and when it is
+disabled.
+
+Dired-Omit mode is a buffer-local minor mode.  When enabled in a
+Dired buffer, Dired does not list files whose filenames match
+regexp `dired-omit-files', nor files ending with extensions in
+`dired-omit-extensions'.
+
+To enable omitting in every Dired buffer, you can put this in
+your init file:
+
+  (add-hook \\='dired-mode-hook (lambda () (dired-omit-mode)))
+
+See Info node `(dired-x) Omitting Variables' for more information.
+
+\(fn &optional ARG)" t nil)
 
 (autoload 'dired-do-relsymlink "dired-x" "\
 Relative symlink all marked (or next ARG) files into a directory.
@@ -598,6 +667,14 @@ not absolute ones like
 For absolute symlinks, use \\[dired-do-symlink].
 
 \(fn &optional ARG)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "dired-x" "dired-x.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from dired-x.el
+
+(register-definition-prefixes "dired-x" '("dired-" "virtual-dired"))
+
+;;;***
 
 ;;;***
 
