@@ -1,9 +1,8 @@
-;;; edmacro.el --- keyboard macro editor
+;;; edmacro.el --- keyboard macro editor  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1993-1994, 2001-2020 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1994, 2001-2021 Free Software Foundation, Inc.
 
 ;; Author: Dave Gillespie <daveg@synaptics.com>
-;; Version: 2.01
 ;; Keywords: abbrev
 
 ;; This file is part of GNU Emacs.
@@ -75,8 +74,8 @@ Default nil means to write characters above \\177 in octal notation."
 
 (defvar edmacro-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-c" 'edmacro-finish-edit)
-    (define-key map "\C-c\C-q" 'edmacro-insert-key)
+    (define-key map "\C-c\C-c" #'edmacro-finish-edit)
+    (define-key map "\C-c\C-q" #'edmacro-insert-key)
     map))
 
 (defvar edmacro-store-hook)
@@ -109,7 +108,7 @@ With a prefix argument, format the macro in a more concise way."
                  (memq cmd-noremap '(call-last-kbd-macro kmacro-call-macro kmacro-end-or-call-macro kmacro-end-and-call-macro))
 		 (member keys '("\r" [return])))
 	     (or last-kbd-macro
-		 (y-or-n-p "No keyboard macro defined.  Create one? ")
+                 (y-or-n-p "No keyboard macro defined.  Create one?")
 		 (keyboard-quit))
 	     (setq mac (or last-kbd-macro ""))
 	     (setq keys nil)
@@ -152,9 +151,9 @@ With a prefix argument, format the macro in a more concise way."
 	(setq buffer-read-only nil)
 	(setq major-mode 'edmacro-mode)
 	(setq mode-name "Edit Macro")
-	(set (make-local-variable 'edmacro-original-buffer) oldbuf)
-	(set (make-local-variable 'edmacro-finish-hook) finish-hook)
-	(set (make-local-variable 'edmacro-store-hook) store-hook)
+        (setq-local edmacro-original-buffer oldbuf)
+        (setq-local edmacro-finish-hook finish-hook)
+        (setq-local edmacro-store-hook store-hook)
 	(erase-buffer)
 	(insert ";; Keyboard Macro Editor.  Press C-c C-c to finish; "
 		"press C-x k RET to cancel.\n")
@@ -178,8 +177,8 @@ With a prefix argument, format the macro in a more concise way."
 	  (set-buffer-modified-p nil))
 	(run-hooks 'edmacro-format-hook)))))
 
-;;; The next two commands are provided for convenience and backward
-;;; compatibility.
+;; The next two commands are provided for convenience and backward
+;; compatibility.
 
 ;;;###autoload
 (defun edit-last-kbd-macro (&optional prefix)
@@ -238,24 +237,23 @@ or nil, use a compact 80-column format."
 		   ((looking-at "Command:[ \t]*\\([^ \t\n]*\\)[ \t]*$")
 		    (when edmacro-store-hook
 		      (error "\"Command\" line not allowed in this context"))
-		    (let ((str (buffer-substring (match-beginning 1)
-						 (match-end 1))))
+		    (let ((str (match-string 1)))
 		      (unless (equal str "")
 			(setq cmd (and (not (equal str "none"))
 				       (intern str)))
 			(and (fboundp cmd) (not (arrayp (symbol-function cmd)))
 			     (not (get cmd 'kmacro))
 			     (not (y-or-n-p
-				   (format "Command %s is already defined; %s"
-					   cmd "proceed? ")))
+                                   (format
+                                    "Command %s is already defined; proceed?"
+                                    cmd)))
 			     (keyboard-quit))))
 		    t)
 		   ((looking-at "Key:\\(.*\\)$")
 		    (when edmacro-store-hook
 		      (error "\"Key\" line not allowed in this context"))
 		    (let ((key (edmacro-parse-keys
-				(buffer-substring (match-beginning 1)
-						  (match-end 1)))))
+				(match-string 1))))
 		      (unless (equal key "")
 			(if (equal key "none")
 			    (setq no-keys t)
@@ -267,24 +265,22 @@ or nil, use a compact 80-column format."
 				     (not (or (arrayp (symbol-function b))
 					      (get b 'kmacro))))
 				 (not (y-or-n-p
-				       (format "Key %s is already defined; %s"
-					       (edmacro-format-keys key 1)
-					       "proceed? ")))
+                                       (format
+                                        "Key %s is already defined; proceed?"
+                                        (edmacro-format-keys key 1))))
 				 (keyboard-quit))))))
 		    t)
 		   ((looking-at "Counter:[ \t]*\\([^ \t\n]*\\)[ \t]*$")
 		    (when edmacro-store-hook
 		      (error "\"Counter\" line not allowed in this context"))
-		    (let ((str (buffer-substring (match-beginning 1)
-						 (match-end 1))))
+		    (let ((str (match-string 1)))
 		      (unless (equal str "")
 			(setq mac-counter (string-to-number str))))
 		    t)
 		   ((looking-at "Format:[ \t]*\"\\([^\n]*\\)\"[ \t]*$")
 		    (when edmacro-store-hook
 		      (error "\"Format\" line not allowed in this context"))
-		    (let ((str (buffer-substring (match-beginning 1)
-						 (match-end 1))))
+		    (let ((str (match-string 1)))
 		      (unless (equal str "")
 			(setq mac-format str)))
 		    t)
@@ -342,7 +338,7 @@ or nil, use a compact 80-column format."
 	(funcall finish-hook)))))
 
 (defun edmacro-insert-key (key)
-  "Insert the written name of a key in the buffer."
+  "Insert the written name of a KEY in the buffer."
   (interactive "kKey to insert: ")
   (if (bolp)
       (insert (edmacro-format-keys key t) "\n")
@@ -476,7 +472,7 @@ doubt, use whitespace."
 			 (and (not (memq (aref rest-mac i) pkeys))
 			      (prog1 (vconcat "C-u " (cl-subseq rest-mac 1 i) " ")
 				(cl-callf cl-subseq rest-mac i)))))))
-	     (bind-len (apply 'max 1
+	     (bind-len (apply #'max 1
 			      (cl-loop for map in maps
                                        for b = (lookup-key map rest-mac)
                                        when b collect b)))
@@ -507,7 +503,7 @@ doubt, use whitespace."
                        finally return i))
 	     desc)
 	(if (stringp bind) (setq bind nil))
-	(cond ((and (eq bind 'self-insert-command) (not prefix)
+	(cond ((and (eq bind #'self-insert-command) (not prefix)
 		    (> text 1) (integerp first)
 		    (> first 32) (<= first maxkey) (/= first 92)
 		    (progn
@@ -521,11 +517,11 @@ doubt, use whitespace."
 			    desc))))
 	       (when (or (string-match "^\\^.$" desc)
 			 (member desc res-words))
-		 (setq desc (mapconcat 'char-to-string desc " ")))
+		 (setq desc (mapconcat #'char-to-string desc " ")))
 	       (when verbose
 		 (setq bind (format "%s * %d" bind text)))
 	       (setq bind-len text))
-	      ((and (eq bind 'execute-extended-command)
+	      ((and (eq bind #'execute-extended-command)
 		    (> text bind-len)
 		    (memq (aref rest-mac text) '(return 13))
 		    (progn
@@ -536,36 +532,35 @@ doubt, use whitespace."
 	       (setq bind-len (1+ text)))
 	      (t
 	       (setq desc (mapconcat
-			   (function
-			    (lambda (ch)
-			      (cond
-			       ((integerp ch)
-				(concat
-				 (cl-loop for pf across "ACHMsS"
-                                          for bit in '(?\A-\^@ ?\C-\^@ ?\H-\^@
-                                                       ?\M-\^@ ?\s-\^@ ?\S-\^@)
-                                          when (/= (logand ch bit) 0)
-                                          concat (format "%c-" pf))
-				 (let ((ch2 (logand ch (1- (ash 1 18)))))
-				   (cond ((<= ch2 32)
-					  (pcase ch2
-					    (0 "NUL") (9 "TAB") (10 "LFD")
-					    (13 "RET") (27 "ESC") (32 "SPC")
-					    (_
-					     (format "C-%c"
-						     (+ (if (<= ch2 26) 96 64)
-							ch2)))))
-					 ((= ch2 127) "DEL")
-					 ((<= ch2 maxkey) (char-to-string ch2))
-					 (t (format "\\%o" ch2))))))
-			       ((symbolp ch)
-				(format "<%s>" ch))
-			       (t
-				(error "Unrecognized item in macro: %s" ch)))))
+                           (lambda (ch)
+                             (cond
+                              ((integerp ch)
+                               (concat
+                                (cl-loop for pf across "ACHMsS"
+                                         for bit in '(?\A-\^@ ?\C-\^@ ?\H-\^@
+                                                              ?\M-\^@ ?\s-\^@ ?\S-\^@)
+                                         when (/= (logand ch bit) 0)
+                                         concat (format "%c-" pf))
+                                (let ((ch2 (logand ch (1- (ash 1 18)))))
+                                  (cond ((<= ch2 32)
+                                         (pcase ch2
+                                           (0 "NUL") (9 "TAB") (10 "LFD")
+                                           (13 "RET") (27 "ESC") (32 "SPC")
+                                           (_
+                                            (format "C-%c"
+                                                    (+ (if (<= ch2 26) 96 64)
+                                                       ch2)))))
+                                        ((= ch2 127) "DEL")
+                                        ((<= ch2 maxkey) (char-to-string ch2))
+                                        (t (format "\\%o" ch2))))))
+                              ((symbolp ch)
+                               (format "<%s>" ch))
+                              (t
+                               (error "Unrecognized item in macro: %s" ch))))
 			   (or fkey key) " "))))
 	(if prefix
 	    (setq desc (concat (edmacro-sanitize-for-string prefix) desc)))
-	(unless (string-match " " desc)
+	(unless (string-search " " desc)
 	  (let ((times 1) (pos bind-len))
 	    (while (not (cl-mismatch rest-mac rest-mac
 				     :start1 0 :end1 bind-len
@@ -606,6 +601,12 @@ This function assumes that the events can be stored in a string."
              (setf (aref seq i) (logand (aref seq i) 127))))
   seq)
 
+;; These are needed in a --without-x build.
+(defvar mouse-wheel-down-event)
+(defvar mouse-wheel-up-event)
+(defvar mouse-wheel-right-event)
+(defvar mouse-wheel-left-event)
+
 (defun edmacro-fix-menu-commands (macro &optional noerror)
   (if (vectorp macro)
       (let (result)
@@ -618,7 +619,7 @@ This function assumes that the events can be stored in a string."
 		((eq (car ev) 'switch-frame))
 		((equal ev '(menu-bar))
 		 (push 'menu-bar result))
-		((equal (cl-cadadr ev) '(menu-bar))
+                ((equal (cadadr ev) '(menu-bar))
 		 (push (vector 'menu-bar (car ev)) result))
 		;; It would be nice to do pop-up menus, too, but not enough
 		;; info is recorded in macros to make this possible.
@@ -669,10 +670,8 @@ This function assumes that the events can be stored in a string."
 				  (substring word 2 -2) "\r")))
 	      ((and (string-match "^\\(\\([ACHMsS]-\\)*\\)<\\(.+\\)>$" word)
 		    (progn
-		      (setq word (concat (substring word (match-beginning 1)
-						    (match-end 1))
-					 (substring word (match-beginning 3)
-						    (match-end 3))))
+		      (setq word (concat (match-string 1 word)
+					 (match-string 3 word)))
 		      (not (string-match
 			    "\\<\\(NUL\\|RET\\|LFD\\|ESC\\|SPC\\|DEL\\)$"
 			    word))))
