@@ -1,9 +1,9 @@
-;;; cl-loaddefs.el --- automatically extracted autoloads
+;;; cl-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "cl-extra" "cl-extra.el" "5e253889c1b017ef55a1adcec23e9784")
+;;;### (autoloads nil "cl-extra" "cl-extra.el" "af2d06240c1d0fa0015e7a28872ad3cb")
 ;;; Generated autoloads from cl-extra.el
 
 (autoload 'cl-coerce "cl-extra" "\
@@ -59,8 +59,11 @@ Like `cl-maplist', but nconc's together the values returned by the function.
 \(fn FUNCTION LIST...)" nil nil)
 
 (autoload 'cl-some "cl-extra" "\
-Return true if PREDICATE is true of any element of SEQ or SEQs.
-If so, return the true (non-nil) value returned by PREDICATE.
+Say whether PREDICATE is true for any element in the SEQ sequences.
+More specifically, the return value of this function will be the
+same as the first return value of PREDICATE where PREDICATE has a
+non-nil value.
+
 
 \(fn PREDICATE SEQ...)" nil nil)
 
@@ -110,7 +113,7 @@ Return the least common multiple of the arguments.
 \(fn &rest ARGS)" nil nil)
 
 (autoload 'cl-isqrt "cl-extra" "\
-Return the integer square root of the (integer) argument.
+Return the integer square root of the (integer) argument X.
 
 \(fn X)" nil nil)
 
@@ -165,7 +168,7 @@ as an integer unless JUNK-ALLOWED is non-nil.
  (autoload 'cl-random-state-p "cl-extra")
 
 (autoload 'cl-random "cl-extra" "\
-Return a random nonnegative number less than LIM, an integer or float.
+Return a pseudo-random nonnegative number less than LIM, an integer or float.
 Optional second arg STATE is a random-state object.
 
 \(fn LIM &optional STATE)" nil nil)
@@ -192,10 +195,10 @@ too large if positive or too small if negative).
 
 \(fn SEQ START &optional END)" nil nil)
 
-(autoload 'cl-concatenate "cl-extra" "\
+(defalias 'cl-concatenate #'seq-concatenate "\
 Concatenate, into a sequence of type TYPE, the argument SEQUENCEs.
 
-\(fn TYPE SEQUENCE...)" nil nil)
+\(fn TYPE SEQUENCE...)")
 
 (autoload 'cl-revappend "cl-extra" "\
 Equivalent to (append (reverse X) Y).
@@ -266,10 +269,18 @@ Expand macros in FORM and insert the pretty-printed result.
 Display the documentation for type TYPE (a symbol).
 
 \(fn TYPE)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "cl-extra" "cl-extra.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from cl-extra.el
+
+(register-definition-prefixes "cl-extra" '("cl-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "cl-macs" "cl-macs.el" "d3ed447e604d9a472ac070bf060863c4")
+;;;### (autoloads nil "cl-macs" "cl-macs.el" "4bd52ccda9cdb15efa6b333a36740aef")
 ;;; Generated autoloads from cl-macs.el
 
 (autoload 'cl--compiler-macro-list* "cl-macs" "\
@@ -277,7 +288,7 @@ Display the documentation for type TYPE (a symbol).
 
 \(fn FORM ARG &rest OTHERS)" nil nil)
 
-(define-obsolete-function-alias 'cl--compiler-macro-cXXr 'internal--compiler-macro-cXXr "25.1")
+(define-obsolete-function-alias 'cl--compiler-macro-cXXr #'internal--compiler-macro-cXXr "25.1")
 
 (autoload 'cl-gensym "cl-macs" "\
 Generate a new uninterned symbol.
@@ -387,7 +398,7 @@ The result of the body appears to the compiler as a quoted constant.
 Eval EXPR and choose among clauses on that value.
 Each clause looks like (KEYLIST BODY...).  EXPR is evaluated and
 compared against each key in each KEYLIST; the corresponding BODY
-is evaluated.  If no clause succeeds, cl-case returns nil.  A
+is evaluated.  If no clause succeeds, this macro returns nil.  A
 single non-nil atom may be used in place of a KEYLIST of one
 atom.  A KEYLIST of t or `otherwise' is allowed only in the final
 clause, and matches if no other keys match.  Key values are
@@ -406,10 +417,10 @@ Like `cl-case', but error if no case fits.
 (function-put 'cl-ecase 'lisp-indent-function '1)
 
 (autoload 'cl-typecase "cl-macs" "\
-Evals EXPR, chooses among clauses on that value.
+Eval EXPR and choose among clauses on that value.
 Each clause looks like (TYPE BODY...).  EXPR is evaluated and, if it
 satisfies TYPE, the corresponding BODY is evaluated.  If no clause succeeds,
-cl-typecase returns nil.  A TYPE of t or `otherwise' is allowed only in the
+this macro returns nil.  A TYPE of t or `otherwise' is allowed only in the
 final clause, and matches if no other keys match.
 
 \(fn EXPR (TYPE BODY...)...)" nil t)
@@ -459,7 +470,8 @@ This is compatible with Common Lisp, but note that `defun' and
 The Common Lisp `loop' macro.
 Valid clauses include:
   For clauses:
-    for VAR from/upfrom/downfrom EXPR1 to/upto/downto/above/below EXPR2 [by EXPR3]
+    for VAR from/upfrom/downfrom EXPR1 to/upto/downto/above/below EXPR2
+        [by EXPR3]
     for VAR = EXPR1 then EXPR2
     for VAR in/on/in-ref LIST [by FUNC]
     for VAR across/across-ref ARRAY
@@ -501,7 +513,7 @@ Once the END-TEST becomes true, the RESULT forms are evaluated (with
 the VARs still bound to their values) to produce the result
 returned by `cl-do'.
 
-Note that the entire loop is enclosed in an implicit `nil' block, so
+Note that the entire loop is enclosed in an implicit nil block, so
 that you can use `cl-return' to exit at any time.
 
 Also note that END-TEST is checked before evaluating BODY.  If END-TEST
@@ -525,7 +537,7 @@ Once the END-TEST becomes true, the RESULT forms are evaluated (with
 the VARs still bound to their values) to produce the result
 returned by `cl-do*'.
 
-Note that the entire loop is enclosed in an implicit `nil' block, so
+Note that the entire loop is enclosed in an implicit nil block, so
 that you can use `cl-return' to exit at any time.
 
 Also note that END-TEST is checked before evaluating BODY.  If END-TEST
@@ -644,7 +656,7 @@ Like `cl-flet' but the definitions can refer to previous ones.
 
 (autoload 'cl-labels "cl-macs" "\
 Make local (recursive) function definitions.
-Each definition can take the form (FUNC ARGLIST BODY...) where
++BINDINGS is a list of definitions of the form (FUNC ARGLIST BODY...) where
 FUNC is the function name, ARGLIST its arguments, and BODY the
 forms of the function body.  FUNC is defined in any BODY, as well
 as FORM, so you can write recursive and mutually recursive
@@ -707,6 +719,8 @@ Return FORM.  If type-checking is enabled, assert that it is of TYPE.
 
 (function-put 'cl-the 'lisp-indent-function '1)
 
+(or (assq 'cl-optimize defun-declarations-alist) (let ((x (list 'cl-optimize #'cl--optimize))) (push x macro-declarations-alist) (push x defun-declarations-alist)))
+
 (autoload 'cl-declare "cl-macs" "\
 Declare SPECS about the current function while compiling.
 For instance
@@ -808,20 +822,28 @@ in SLOTs.  It defines a `make-NAME' constructor, a `copy-NAME'
 copier, a `NAME-p' predicate, and slot accessors named `NAME-SLOT'.
 You can use the accessors to set the corresponding slots, via `setf'.
 
-NAME may instead take the form (NAME OPTIONS...), where each
-OPTION is either a single keyword or (KEYWORD VALUE) where
-KEYWORD can be one of `:conc-name', `:constructor', `:copier',
-`:predicate', `:type', `:named', `:initial-offset',
-`:print-function', `:noinline', or `:include'.  See Info
-node `(cl)Structures' for the description of the options.
+NAME is usually a symbol, but may instead take the form (NAME
+OPTIONS...), where each OPTION is either a single keyword
+or (KEYWORD VALUE) where KEYWORD can be one of `:conc-name',
+`:constructor', `:copier', `:predicate', `:type', `:named',
+`:initial-offset', `:print-function', `:noinline', or `:include'.
+See Info node `(cl)Structures' for the description of the
+options.
+
+The first element in SLOTS can be a doc string.
+
+The rest of the elements in SLOTS is a list of SLOT elements,
+each of which should either be a symbol, or take the form (SNAME
+SDEFAULT SOPTIONS...), where SDEFAULT is the default value of
+that slot and SOPTIONS are keyword-value pairs for that slot.
 
-Each SLOT may instead take the form (SNAME SDEFAULT SOPTIONS...), where
-SDEFAULT is the default value of that slot and SOPTIONS are keyword-value
-pairs for that slot.
 Supported keywords for slots are:
-- `:read-only':  If this has a non-nil value, that slot cannot be set via `setf'.
+- `:read-only': If this has a non-nil value, that slot cannot be set via `setf'.
 - `:documentation': this is a docstring describing the slot.
-- `:type': the type of the field; currently unused.
+- `:type': the type of the field; currently only used for documentation.
+
+To see the documentation for a defined struct type, use
+\\[describe-symbol] or \\[cl-describe-type].
 
 \(fn NAME &optional DOCSTRING &rest SLOTS)" nil t)
 
@@ -829,18 +851,25 @@ Supported keywords for slots are:
 
 (function-put 'cl-defstruct 'lisp-indent-function '1)
 
-(autoload 'cl-struct--pcase-macroexpander "cl-macs" "\
-Pcase patterns that match cl-struct EXPVAL of type TYPE.
-Elements of FIELDS can be of the form (NAME PAT) in which case the
-contents of field NAME is matched against PAT, or they can be of
-the form NAME which is a shorthand for (NAME NAME).
-
-\(fn TYPE &rest FIELDS)" nil nil)
+(eval-and-compile (defun cl-struct--pcase-macroexpander (type &rest fields) "Pcase patterns that match cl-struct EXPVAL of type TYPE.\nElements of FIELDS can be of the form (NAME PAT) in which case the\ncontents of field NAME is matched against PAT, or they can be of\nthe form NAME which is a shorthand for (NAME NAME)." `(and (pred (pcase--flip cl-typep ',type)) ,@(mapcar (lambda (field) (let* ((name (if (consp field) (car field) field)) (pat (if (consp field) (cadr field) field))) `(app ,(if (eq (cl-struct-sequence-type type) 'list) `(nth ,(cl-struct-slot-offset type name)) `(pcase--flip aref ,(cl-struct-slot-offset type name))) ,pat))) fields))))
 
 (define-symbol-prop 'cl-struct--pcase-macroexpander 'edebug-form-spec '(sexp &rest [&or (sexp pcase-PAT) sexp]))
 
 (define-symbol-prop 'cl-struct 'pcase-macroexpander #'cl-struct--pcase-macroexpander)
 
+(autoload 'cl-struct-slot-info "cl-macs" "\
+Return a list of slot names of struct STRUCT-TYPE.
+Each entry is a list (SLOT-NAME . OPTS), where SLOT-NAME is a
+slot name symbol and OPTS is a list of slot options given to
+`cl-defstruct'.  Dummy slots that represent the struct name and
+slots skipped by :initial-offset may appear in the list.
+
+\(fn STRUCT-TYPE)" nil nil)
+
+(function-put 'cl-struct-slot-info 'side-effect-free 't)
+
+(function-put 'cl-struct-slot-info 'pure 't)
+
 (autoload 'cl-typep "cl-macs" "\
 
 
@@ -902,9 +931,23 @@ The type name can then be used in `cl-typecase', `cl-check-type', etc.
 
 (function-put 'cl-deftype 'lisp-indent-function '2)
 
+(eval-and-compile (defun cl-type--pcase-macroexpander (type) "Pcase pattern that matches objects of TYPE.\nTYPE is a type descriptor as accepted by `cl-typep', which see." `(pred (pcase--flip cl-typep ',type))))
+
+(define-symbol-prop 'cl-type--pcase-macroexpander 'edebug-form-spec 'nil)
+
+(define-symbol-prop 'cl-type 'pcase-macroexpander #'cl-type--pcase-macroexpander)
+
+;;;### (autoloads "actual autoloads are elsewhere" "cl-macs" "cl-macs.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from cl-macs.el
+
+(register-definition-prefixes "cl-macs" '("cl-" "function-form"))
+
+;;;***
+
 ;;;***
 
-;;;### (autoloads nil "cl-seq" "cl-seq.el" "d802a87ff9f142aa867a492c9a0a0767")
+;;;### (autoloads nil "cl-seq" "cl-seq.el" "95d0eb23ec50fd748e6eb0edf5e6c192")
 ;;; Generated autoloads from cl-seq.el
 
 (autoload 'cl-endp "cl-seq" "\
@@ -1407,6 +1450,14 @@ Atoms are compared by `eql'; cons cells are compared recursively.
 Keywords supported:  :test :test-not :key
 
 \(fn TREE1 TREE2 [KEYWORD VALUE]...)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "cl-seq" "cl-seq.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from cl-seq.el
+
+(register-definition-prefixes "cl-seq" '("cl--"))
+
+;;;***
 
 ;;;***
 
