@@ -1,33 +1,65 @@
-;;; erc-loaddefs.el --- automatically extracted autoloads
+;;; erc-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "erc-autoaway" "erc-autoaway.el" "ca668ea42f4deac871bfc9c2f92cab11")
+;;;### (autoloads nil "erc-autoaway" "erc-autoaway.el" "afeb242f8c0bd2b6052933f4082b330d")
 ;;; Generated autoloads from erc-autoaway.el
 (autoload 'erc-autoaway-mode "erc-autoaway")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-autoaway"
+;;;;;;  "erc-autoaway.el" (0 0 0 0))
+;;; Generated autoloads from erc-autoaway.el
+
+(register-definition-prefixes "erc-autoaway" '("erc-auto"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-button" "erc-button.el" "cdc986f12666c6240d8e541f30dc8b4d")
+;;;### (autoloads nil "erc-button" "erc-button.el" "33ec026c6796e8b91a879afc2d5276d5")
 ;;; Generated autoloads from erc-button.el
 (autoload 'erc-button-mode "erc-button" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-button" "erc-button.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-button.el
+
+(register-definition-prefixes "erc-button" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-capab" "erc-capab.el" "e1016ca6627c94c6f16d8e21e7b2c900")
+;;;### (autoloads nil "erc-capab" "erc-capab.el" "1566962e3085bac03fcdc3768f370e75")
 ;;; Generated autoloads from erc-capab.el
 (autoload 'erc-capab-identify-mode "erc-capab" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-capab" "erc-capab.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-capab.el
+
+(register-definition-prefixes "erc-capab" '("erc-capab-identify-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-compat" "erc-compat.el" "aab018d738484d60354572e400001ac2")
+;;;### (autoloads nil "erc-compat" "erc-compat.el" "439c48d3d9c19164c096a2015ad7185b")
 ;;; Generated autoloads from erc-compat.el
 (autoload 'erc-define-minor-mode "erc-compat")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-compat" "erc-compat.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-compat.el
+
+(register-definition-prefixes "erc-compat" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-dcc" "erc-dcc.el" "11bfbaa7dcbaa5f9185803d62c4742f9")
+;;;### (autoloads nil "erc-dcc" "erc-dcc.el" "dd8dd9a27a8978ae12c6b0fb4151196a")
 ;;; Generated autoloads from erc-dcc.el
 (autoload 'erc-dcc-mode "erc-dcc")
 
@@ -51,17 +83,33 @@ It examines the DCC subcommand, and calls the appropriate routine for
 that subcommand.
 
 \(fn PROC NICK LOGIN HOST TO QUERY)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-dcc" "erc-dcc.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-dcc.el
+
+(register-definition-prefixes "erc-dcc" '("erc-" "pcomplete/erc-mode/"))
+
+;;;***
 
 ;;;***
 
 ;;;### (autoloads nil "erc-desktop-notifications" "erc-desktop-notifications.el"
-;;;;;;  "2576908e6624c2c6c00707c91269148e")
+;;;;;;  "b8c4bc23d173b33ea3b7f77d4a4a73f0")
 ;;; Generated autoloads from erc-desktop-notifications.el
 (autoload 'erc-notifications-mode "erc-desktop-notifications" "" t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-desktop-notifications"
+;;;;;;  "erc-desktop-notifications.el" (0 0 0 0))
+;;; Generated autoloads from erc-desktop-notifications.el
+
+(register-definition-prefixes "erc-desktop-notifications" '("erc-notifications-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-ezbounce" "erc-ezbounce.el" "34f21e38aa5c98666c71a97b1d31e2d6")
+;;;### (autoloads nil "erc-ezbounce" "erc-ezbounce.el" "9d99fc2ddb11eb02daf56bed4063b871")
 ;;; Generated autoloads from erc-ezbounce.el
 
 (autoload 'erc-cmd-ezb "erc-ezbounce" "\
@@ -116,20 +164,36 @@ Select a detached EZBounce session." nil nil)
 
 (autoload 'erc-ezb-initialize "erc-ezbounce" "\
 Add EZBouncer convenience functions to ERC." nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-ezbounce"
+;;;;;;  "erc-ezbounce.el" (0 0 0 0))
+;;; Generated autoloads from erc-ezbounce.el
+
+(register-definition-prefixes "erc-ezbounce" '("erc-ezb-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-fill" "erc-fill.el" "731a19ef9a3b13b9ff98c756e5881d04")
+;;;### (autoloads nil "erc-fill" "erc-fill.el" "dd7564055e585b1e2348ccf231d62a96")
 ;;; Generated autoloads from erc-fill.el
 (autoload 'erc-fill-mode "erc-fill" nil t)
 
 (autoload 'erc-fill "erc-fill" "\
 Fill a region using the function referenced in `erc-fill-function'.
 You can put this on `erc-insert-modify-hook' and/or `erc-send-modify-hook'." nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-fill" "erc-fill.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-fill.el
+
+(register-definition-prefixes "erc-fill" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-identd" "erc-identd.el" "ab7b9860d67ffa89d3ab0544f7ccc805")
+;;;### (autoloads nil "erc-identd" "erc-identd.el" "5ce0ffafdfd3842af1d320a0a528b39f")
 ;;; Generated autoloads from erc-identd.el
 (autoload 'erc-identd-mode "erc-identd")
 
@@ -147,29 +211,61 @@ system.
 
 
 \(fn &rest IGNORE)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-identd" "erc-identd.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-identd.el
+
+(register-definition-prefixes "erc-identd" '("erc-identd-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-imenu" "erc-imenu.el" "f5c4224eb343e9204935f5e36dd2cd8d")
+;;;### (autoloads nil "erc-imenu" "erc-imenu.el" "0a061caa45c8feb7ddca531f5889cc24")
 ;;; Generated autoloads from erc-imenu.el
 
 (autoload 'erc-create-imenu-index "erc-imenu" nil nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-imenu" "erc-imenu.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-imenu.el
+
+(register-definition-prefixes "erc-imenu" '("erc-unfill-notice"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-join" "erc-join.el" "f7e4b71821e96b2bb5cedde311576280")
+;;;### (autoloads nil "erc-join" "erc-join.el" "6bdb94840c9d9bbb1aa73b2a4440af74")
 ;;; Generated autoloads from erc-join.el
 (autoload 'erc-autojoin-mode "erc-join" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-join" "erc-join.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-join.el
+
+(register-definition-prefixes "erc-join" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-list" "erc-list.el" "029e85ef7b385dd8ae26c8495dada2b7")
+;;;### (autoloads nil "erc-list" "erc-list.el" "92cdca5207728ab310b2d2639ee5a3e1")
 ;;; Generated autoloads from erc-list.el
 (autoload 'erc-list-mode "erc-list")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-list" "erc-list.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-list.el
+
+(register-definition-prefixes "erc-list" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-log" "erc-log.el" "a7aff2dc724e69ccd0b5470695b478ba")
+;;;### (autoloads nil "erc-log" "erc-log.el" "5bc49891005bc89a0430d07340463554")
 ;;; Generated autoloads from erc-log.el
 (autoload 'erc-log-mode "erc-log" nil t)
 
@@ -195,10 +291,18 @@ You can save every individual message by putting this function on
 `erc-insert-post-hook'.
 
 \(fn &optional BUFFER)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-log" "erc-log.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-log.el
+
+(register-definition-prefixes "erc-log" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-match" "erc-match.el" "94b1f11acd5a99478d58d017536b0bb7")
+;;;### (autoloads nil "erc-match" "erc-match.el" "739b8d04258f65a5ac6ca5b75278c3ae")
 ;;; Generated autoloads from erc-match.el
 (autoload 'erc-match-mode "erc-match")
 
@@ -225,25 +329,49 @@ Add dangerous-host interactively to `erc-dangerous-hosts'." t nil)
 
 (autoload 'erc-delete-dangerous-host "erc-match" "\
 Delete dangerous-host interactively to `erc-dangerous-hosts'." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-match" "erc-match.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-match.el
+
+(register-definition-prefixes "erc-match" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-menu" "erc-menu.el" "38c3a0663348cdfc93f035042eb6d1f3")
+;;;### (autoloads nil "erc-menu" "erc-menu.el" "197d5a35512da86a6190d408970dd2f3")
 ;;; Generated autoloads from erc-menu.el
 (autoload 'erc-menu-mode "erc-menu" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-menu" "erc-menu.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-menu.el
+
+(register-definition-prefixes "erc-menu" '("erc-menu-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-netsplit" "erc-netsplit.el" "91f7cfa5e118765b3baeb5a69d0af861")
+;;;### (autoloads nil "erc-netsplit" "erc-netsplit.el" "cc38dd587427a87e1fc7cf4b060f9ec8")
 ;;; Generated autoloads from erc-netsplit.el
 (autoload 'erc-netsplit-mode "erc-netsplit")
 
 (autoload 'erc-cmd-WHOLEFT "erc-netsplit" "\
 Show who's gone." nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-netsplit"
+;;;;;;  "erc-netsplit.el" (0 0 0 0))
+;;; Generated autoloads from erc-netsplit.el
+
+(register-definition-prefixes "erc-netsplit" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-notify" "erc-notify.el" "87dd5d737dbdbb6536ff859993f9d41a")
+;;;### (autoloads nil "erc-notify" "erc-notify.el" "fe3346d0a15a28c6cd0e11b7573e597e")
 ;;; Generated autoloads from erc-notify.el
 (autoload 'erc-notify-mode "erc-notify" nil t)
 
@@ -255,34 +383,74 @@ with args, toggle notify status of people.
 \(fn &rest ARGS)" nil nil)
 
 (autoload 'pcomplete/erc-mode/NOTIFY "erc-notify" nil nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-notify" "erc-notify.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-notify.el
+
+(register-definition-prefixes "erc-notify" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-page" "erc-page.el" "e610077a6d3425be885698de53e8616b")
+;;;### (autoloads nil "erc-page" "erc-page.el" "f8581852b5dbfcd19604530f8d134166")
 ;;; Generated autoloads from erc-page.el
 (autoload 'erc-page-mode "erc-page")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-page" "erc-page.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-page.el
+
+(register-definition-prefixes "erc-page" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-pcomplete" "erc-pcomplete.el" "62ee5e21e5161a0b280efc32f264e5fe")
+;;;### (autoloads nil "erc-pcomplete" "erc-pcomplete.el" "37103113ccaad9756f7da4249037ca0e")
 ;;; Generated autoloads from erc-pcomplete.el
 (autoload 'erc-completion-mode "erc-pcomplete" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-pcomplete"
+;;;;;;  "erc-pcomplete.el" (0 0 0 0))
+;;; Generated autoloads from erc-pcomplete.el
+
+(register-definition-prefixes "erc-pcomplete" '("erc-pcomplet" "pcomplete"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-replace" "erc-replace.el" "531f61bc0e3d9be12a68c7ca493975a8")
+;;;### (autoloads nil "erc-replace" "erc-replace.el" "115c42579003282bc0a2826c42654b0f")
 ;;; Generated autoloads from erc-replace.el
 (autoload 'erc-replace-mode "erc-replace")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-replace"
+;;;;;;  "erc-replace.el" (0 0 0 0))
+;;; Generated autoloads from erc-replace.el
+
+(register-definition-prefixes "erc-replace" '("erc-replace-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-ring" "erc-ring.el" "67a9a5d171237b0cbf376181acdca72e")
+;;;### (autoloads nil "erc-ring" "erc-ring.el" "fb580f4fe404614b3d1f7336e82e1a03")
 ;;; Generated autoloads from erc-ring.el
 (autoload 'erc-ring-mode "erc-ring" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-ring" "erc-ring.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-ring.el
+
+(register-definition-prefixes "erc-ring" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-services" "erc-services.el" "9dec03d381ea5a151a203478402275f7")
+;;;### (autoloads nil "erc-services" "erc-services.el" "e5460f88128567db6bd2608f2dea8f1c")
 ;;; Generated autoloads from erc-services.el
 (autoload 'erc-services-mode "erc-services" nil t)
 
@@ -292,41 +460,109 @@ Set up hooks according to which MODE the user has chosen.
 \(fn MODE)" t nil)
 
 (autoload 'erc-nickserv-identify "erc-services" "\
-Send an \"identify <PASSWORD>\" message to NickServ.
-When called interactively, read the password using `read-passwd'.
+Identify to NickServ immediately.
+Identification will either use NICK or the current nick if not
+provided, and some password obtained through
+`erc-nickserv-get-password' (which see).  If no password can be
+found, an error is reported through `erc-error'.
 
-\(fn PASSWORD)" t nil)
+Interactively, the user will be prompted for NICK, an empty
+string meaning to default to the current nick.
+
+Returns t if the identify message could be sent, nil otherwise.
+
+\(fn &optional PASSWORD NICK)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-services"
+;;;;;;  "erc-services.el" (0 0 0 0))
+;;; Generated autoloads from erc-services.el
+
+(register-definition-prefixes "erc-services" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-sound" "erc-sound.el" "d583b70db2faa6da6259f3d8ab1960e0")
+;;;### (autoloads nil "erc-sound" "erc-sound.el" "7cd495576247367ff538fcded9b98cb1")
 ;;; Generated autoloads from erc-sound.el
 (autoload 'erc-sound-mode "erc-sound")
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-sound" "erc-sound.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-sound.el
+
+(register-definition-prefixes "erc-sound" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-speedbar" "erc-speedbar.el" "fbe37562633113346fdaf0b1b040c424")
+;;;### (autoloads nil "erc-speedbar" "erc-speedbar.el" "c9d9a0bb9a8d990154610cb6b6b327d8")
 ;;; Generated autoloads from erc-speedbar.el
 
 (autoload 'erc-speedbar-browser "erc-speedbar" "\
 Initialize speedbar to display an ERC browser.
 This will add a speedbar major display mode." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-speedbar"
+;;;;;;  "erc-speedbar.el" (0 0 0 0))
+;;; Generated autoloads from erc-speedbar.el
+
+(register-definition-prefixes "erc-speedbar" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-spelling" "erc-spelling.el" "cbc826eb1503bc61f9262efd419530db")
+;;;### (autoloads nil "erc-spelling" "erc-spelling.el" "dbf3bbf520d08bce90484ea548026fce")
 ;;; Generated autoloads from erc-spelling.el
 (autoload 'erc-spelling-mode "erc-spelling" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-spelling"
+;;;;;;  "erc-spelling.el" (0 0 0 0))
+;;; Generated autoloads from erc-spelling.el
+
+(register-definition-prefixes "erc-spelling" '("erc-spelling-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-stamp" "erc-stamp.el" "8a33d68476c0ba5649a2390a709caa22")
+;;;### (autoloads nil "erc-stamp" "erc-stamp.el" "d1e63c5723226962509b9e402bb5675c")
 ;;; Generated autoloads from erc-stamp.el
 (autoload 'erc-timestamp-mode "erc-stamp" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-stamp" "erc-stamp.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-stamp.el
+
+(register-definition-prefixes "erc-stamp" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-track" "erc-track.el" "1ed40a808bb343c1c3510f6703c658b4")
+;;;### (autoloads nil "erc-status-sidebar" "erc-status-sidebar.el"
+;;;;;;  "7c94954b144ef5623550645214991d54")
+;;; Generated autoloads from erc-status-sidebar.el
+
+(autoload 'erc-status-sidebar-open "erc-status-sidebar" "\
+Open or create a sidebar." t nil)
+
+(autoload 'erc-status-sidebar-toggle "erc-status-sidebar" "\
+Toggle the sidebar open/closed on the current frame." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-status-sidebar"
+;;;;;;  "erc-status-sidebar.el" (0 0 0 0))
+;;; Generated autoloads from erc-status-sidebar.el
+
+(register-definition-prefixes "erc-status-sidebar" '("erc-status-sidebar-"))
+
+;;;***
+
+;;;***
+
+;;;### (autoloads nil "erc-track" "erc-track.el" "afd34032b88bc837bc1d9a6aba58d15b")
 ;;; Generated autoloads from erc-track.el
 
 (defvar erc-track-minor-mode nil "\
@@ -339,10 +575,19 @@ for a description of this minor mode.")
 (autoload 'erc-track-minor-mode "erc-track" "\
 Toggle mode line display of ERC activity (ERC Track minor mode).
 
-If called interactively, enable Erc-Track minor mode if ARG is
-positive, and disable it if ARG is zero or negative.  If called from
-Lisp, also enable the mode if ARG is omitted or nil, and toggle it if
-ARG is `toggle'; disable the mode otherwise.
+This is a minor mode.  If called interactively, toggle the `Erc-Track
+minor mode' mode.  If the prefix argument is positive, enable the
+mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable the
+mode if ARG is nil, omitted, or is a positive number.  Disable the
+mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `(default-value \\='erc-track-minor-mode)'.
+
+The mode's hook is called both when the mode is enabled and when it is
+disabled.
 
 ERC Track minor mode is a global minor mode.  It exists for the
 sole purpose of providing the C-c C-SPC and C-c C-@ keybindings.
@@ -351,10 +596,18 @@ keybindings will not do anything useful.
 
 \(fn &optional ARG)" t nil)
 (autoload 'erc-track-mode "erc-track" nil t)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-track" "erc-track.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-track.el
+
+(register-definition-prefixes "erc-track" '("erc-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-truncate" "erc-truncate.el" "b5827787102055435ead19aefa224250")
+;;;### (autoloads nil "erc-truncate" "erc-truncate.el" "ab0a13a87fb8032baf76ee703e90b876")
 ;;; Generated autoloads from erc-truncate.el
 (autoload 'erc-truncate-mode "erc-truncate" nil t)
 
@@ -368,17 +621,33 @@ region is logged if `erc-logging-enabled' returns non-nil.
 (autoload 'erc-truncate-buffer "erc-truncate" "\
 Truncates the current buffer to `erc-max-buffer-size'.
 Meant to be used in hooks, like `erc-insert-post-hook'." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-truncate"
+;;;;;;  "erc-truncate.el" (0 0 0 0))
+;;; Generated autoloads from erc-truncate.el
+
+(register-definition-prefixes "erc-truncate" '("erc-max-buffer-size"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "erc-xdcc" "erc-xdcc.el" "a561a16a265fef197cc8209360591ed2")
+;;;### (autoloads nil "erc-xdcc" "erc-xdcc.el" "3a584ca82ce15289da4034c372caca06")
 ;;; Generated autoloads from erc-xdcc.el
 (autoload 'erc-xdcc-mode "erc-xdcc")
 
 (autoload 'erc-xdcc-add-file "erc-xdcc" "\
-Add a file to `erc-xdcc-files'.
+Add FILE to `erc-xdcc-files'.
 
 \(fn FILE)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "erc-xdcc" "erc-xdcc.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from erc-xdcc.el
+
+(register-definition-prefixes "erc-xdcc" '("erc-"))
+
+;;;***
 
 ;;;***
 
