@@ -1,27 +1,43 @@
-;;; esh-groups.el --- automatically extracted autoloads
+;;; esh-groups.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "em-alias" "em-alias.el" "a9c92e4ece52f5887c98d16873f9d46d")
+;;;### (autoloads nil "em-alias" "em-alias.el" "62cfb134ea7f64e56df2a9c847469554")
 ;;; Generated autoloads from em-alias.el
 
 (defgroup eshell-alias nil "\
 Command aliases allow for easy definition of alternate commands." :tag "Command aliases" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-alias" "em-alias.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-alias.el
+
+(register-definition-prefixes "em-alias" '("eshell" "pcomplete/eshell-mode/alias"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-banner" "em-banner.el" "ea1a93a909c7fbfc26a897da2b1cc8c4")
+;;;### (autoloads nil "em-banner" "em-banner.el" "0c37f3181d1f0b7ce91fb08cd08d3fb5")
 ;;; Generated autoloads from em-banner.el
 
 (defgroup eshell-banner nil "\
 This sample module displays a welcome banner at login.
 It exists so that others wishing to create their own Eshell extension
 modules may have a simple template to begin with." :tag "Login banner" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-banner" "em-banner.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-banner.el
+
+(register-definition-prefixes "em-banner" '("eshell-banner-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-basic" "em-basic.el" "16045ad4ae7945a08a13ddc72f43891d")
+;;;### (autoloads nil "em-basic" "em-basic.el" "3daaa6ef7beaf517a576c60cb3829419")
 ;;; Generated autoloads from em-basic.el
 
 (defgroup eshell-basic nil "\
@@ -30,20 +46,36 @@ are traditionally considered shell builtins.  Since all of the
 functionality provided by them is accessible through Lisp, they are
 not really builtins at all, but offer a command-oriented way to do the
 same thing." :tag "Basic shell commands" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-basic" "em-basic.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-basic.el
+
+(register-definition-prefixes "em-basic" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-cmpl" "em-cmpl.el" "e295b900e443849032b930d5fca643ec")
+;;;### (autoloads nil "em-cmpl" "em-cmpl.el" "cc0ac9df9bc538004f180ce1dab31cf8")
 ;;; Generated autoloads from em-cmpl.el
 
 (defgroup eshell-cmpl nil "\
 This module provides a programmable completion function bound to
 the TAB key, which allows for completing command names, file names,
 variable names, arguments, etc." :tag "Argument completion" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-cmpl" "em-cmpl.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-cmpl.el
+
+(register-definition-prefixes "em-cmpl" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-dirs" "em-dirs.el" "d15eb1f93a9727049ada22b5acb2e32f")
+;;;### (autoloads nil "em-dirs" "em-dirs.el" "f797e0b1c81764ebabe674e487bb8014")
 ;;; Generated autoloads from em-dirs.el
 
 (defgroup eshell-dirs nil "\
@@ -52,39 +84,71 @@ current directory, maintaining a directory stack, and also keeping
 track of a history of the last directory locations the user was in.
 Emacs does provide standard Lisp definitions of `pwd' and `cd', but
 they lack somewhat in feel from the typical shell equivalents." :tag "Directory navigation" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-dirs" "em-dirs.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-dirs.el
+
+(register-definition-prefixes "em-dirs" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-glob" "em-glob.el" "50e0e1f8293f0330c13b099da448c57c")
+;;;### (autoloads nil "em-glob" "em-glob.el" "247b031b1a248ff16c82ca2c6c0eb4bb")
 ;;; Generated autoloads from em-glob.el
 
 (defgroup eshell-glob nil "\
 This module provides extended globbing syntax, similar what is used
 by zsh for filename generation." :tag "Extended filename globbing" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-glob" "em-glob.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-glob.el
+
+(register-definition-prefixes "em-glob" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-hist" "em-hist.el" "aaf4df032d688fa7656dd48aa4ceaca2")
+;;;### (autoloads nil "em-hist" "em-hist.el" "43010f01525c335a0c7a1a44d6adc4d1")
 ;;; Generated autoloads from em-hist.el
 
 (defgroup eshell-hist nil "\
 This module provides command history management." :tag "History list management" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-hist" "em-hist.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-hist.el
+
+(register-definition-prefixes "em-hist" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-ls" "em-ls.el" "880ddae8f15a7d785a6ff78303987d7e")
+;;;### (autoloads nil "em-ls" "em-ls.el" "fe4a6cdf1efbbafe9ecc045c2873cb85")
 ;;; Generated autoloads from em-ls.el
 
 (defgroup eshell-ls nil "\
-This module implements the \"ls\" utility fully in Lisp.  If it is
-passed any unrecognized command switches, it will revert to the
-operating system's version.  This version of \"ls\" uses text
-properties to colorize its output based on the setting of
+This module implements the \"ls\" utility fully in Lisp.
+If it is passed any unrecognized command switches, it will revert
+to the operating system's version.  This version of \"ls\" uses
+text properties to colorize its output based on the setting of
 `eshell-ls-use-colors'." :tag "Implementation of `ls' in Lisp" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-ls" "em-ls.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-ls.el
+
+(register-definition-prefixes "em-ls" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-pred" "em-pred.el" "8f2f1ce127b1e03ed3fa4717f33bd157")
+;;;### (autoloads nil "em-pred" "em-pred.el" "a39a87ecd7c86cbfd7a7497f44b558cc")
 ;;; Generated autoloads from em-pred.el
 
 (defgroup eshell-pred nil "\
@@ -92,19 +156,35 @@ This module allows for predicates to be applied to globbing
 patterns (similar to zsh), in addition to string modifiers which can
 be applied either to globbing results, variable references, or just
 ordinary strings." :tag "Value modifiers and predicates" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-pred" "em-pred.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-pred.el
+
+(register-definition-prefixes "em-pred" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-prompt" "em-prompt.el" "bce3ed0abe4436faf73d0ba7aa2fad66")
+;;;### (autoloads nil "em-prompt" "em-prompt.el" "84e9955df5aa65d1dbd02991e6af5962")
 ;;; Generated autoloads from em-prompt.el
 
 (defgroup eshell-prompt nil "\
 This module provides command prompts, and navigation between them,
 as is common with most shells." :tag "Command prompts" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-prompt" "em-prompt.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-prompt.el
+
+(register-definition-prefixes "em-prompt" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-rebind" "em-rebind.el" "b661555b406a014bf1937fff01d1024e")
+;;;### (autoloads nil "em-rebind" "em-rebind.el" "36cf7b7fe947595e85ac82987dd7077d")
 ;;; Generated autoloads from em-rebind.el
 
 (defgroup eshell-rebind nil "\
@@ -117,19 +197,35 @@ history, C-u kills the current input text, etc.  It also, if
 commands to cause the point to leave the input area, such as
 `backward-word', `previous-line', etc.  This module intends to mimic
 the behavior of normal shells while the user editing new input text." :tag "Rebind keys at input" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-rebind" "em-rebind.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-rebind.el
+
+(register-definition-prefixes "em-rebind" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-script" "em-script.el" "e86a06f546d09de73acc492ea22b2127")
+;;;### (autoloads nil "em-script" "em-script.el" "920f78f7404bd70d453dac65edea6afe")
 ;;; Generated autoloads from em-script.el
 
 (defgroup eshell-script nil "\
 This module allows for the execution of files containing Eshell
 commands, as a script file." :tag "Running script files." :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-script" "em-script.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-script.el
+
+(register-definition-prefixes "em-script" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-smart" "em-smart.el" "7cf786e1cb24f3a0d6f077d7381feae8")
+;;;### (autoloads nil "em-smart" "em-smart.el" "55ddbdf45bb1862b1df5c51db0ee85f4")
 ;;; Generated autoloads from em-smart.el
 
 (defgroup eshell-smart nil "\
@@ -139,10 +235,18 @@ some of the edit/review concepts inherent in the design of Plan 9's
 
 Most likely you will have to turn this option on and play around with
 it to get a real sense of how it works." :tag "Smart display of output" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-smart" "em-smart.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-smart.el
+
+(register-definition-prefixes "em-smart" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-term" "em-term.el" "e929a121c54be983ea46be80a3cd3d33")
+;;;### (autoloads nil "em-term" "em-term.el" "cfbf7a37b53fed174cf8f092746354a6")
 ;;; Generated autoloads from em-term.el
 
 (defgroup eshell-term nil "\
@@ -151,10 +255,18 @@ the `term' package, which comes with Emacs.  This package handles most
 of the ANSI control codes, allowing curses-based applications to run
 within an Emacs window.  The variable `eshell-visual-commands' defines
 which commands are considered visual in nature." :tag "Running visual commands" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-term" "em-term.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-term.el
+
+(register-definition-prefixes "em-term" '("eshell-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-tramp" "em-tramp.el" "6430aca2660e9c43e035a5ed096bb2a2")
+;;;### (autoloads nil "em-tramp" "em-tramp.el" "c07ad97f69958f5415390dd80f56316d")
 ;;; Generated autoloads from em-tramp.el
 
 (defgroup eshell-tramp nil "\
@@ -163,10 +275,18 @@ This module defines commands that use TRAMP in a way that is
   built-in su and sudo commands, which are not compatible with
   the full, external su and sudo commands, and require the user
   to understand how to use the TRAMP sudo method." :tag "TRAMP Eshell features" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-tramp" "em-tramp.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-tramp.el
+
+(register-definition-prefixes "em-tramp" '("eshell"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-unix" "em-unix.el" "9e6dca6fb3088f69c758a2b9fd763e15")
+;;;### (autoloads nil "em-unix" "em-unix.el" "143c379b5c60d22242b36a1d96095a4c")
 ;;; Generated autoloads from em-unix.el
 
 (defgroup eshell-unix nil "\
@@ -179,10 +299,18 @@ usually much slower.  But in several cases their tight integration
 with Eshell makes them more versatile than their traditional cousins
 \(such as being able to use `kill' to kill Eshell background processes
 by name)." :tag "UNIX commands in Lisp" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-unix" "em-unix.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-unix.el
+
+(register-definition-prefixes "em-unix" '("eshell" "nil-blank-string"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "em-xtra" "em-xtra.el" "19700e81e07e6c9aba5c6fece7eeb8ff")
+;;;### (autoloads nil "em-xtra" "em-xtra.el" "63d19498b91f2c629c0839ce67c460db")
 ;;; Generated autoloads from em-xtra.el
 
 (defgroup eshell-xtra nil "\
@@ -190,6 +318,14 @@ This module defines some extra alias functions which are entirely
 optional.  They can be viewed as samples for how to write Eshell alias
 functions, or as aliases which make some of Emacs's behavior more
 naturally accessible within Emacs." :tag "Extra alias functions" :group 'eshell-module)
+
+;;;### (autoloads "actual autoloads are elsewhere" "em-xtra" "em-xtra.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from em-xtra.el
+
+(register-definition-prefixes "em-xtra" '("eshell/"))
+
+;;;***
 
 ;;;***
 
