@@ -1,6 +1,6 @@
-;;; gnus-range.el --- range and sequence functions for Gnus
+;;; gnus-range.el --- range and sequence functions for Gnus  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1996-2018 Free Software Foundation, Inc.
+;; Copyright (C) 1996-2021 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
@@ -24,13 +24,11 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
-
 ;;; List and range functions
 
 (defsubst gnus-range-normalize (range)
   "Normalize RANGE.
-If RANGE is a single range, return (RANGE). Otherwise, return RANGE."
+If RANGE is a single range, return (RANGE).  Otherwise, return RANGE."
   (if (listp (cdr-safe range)) range (list range)))
 
 (defun gnus-last-element (list)
@@ -38,18 +36,12 @@ If RANGE is a single range, return (RANGE). Otherwise, return RANGE."
   (while (cdr list)
     (setq list (cdr list)))
   (car list))
+(make-obsolete 'gnus-last-element "use `car' of `last' instead." "27.1")
 
-(defun gnus-copy-sequence (list)
-  "Do a complete, total copy of a list."
-  (let (out)
-    (while (consp list)
-      (if (consp (car list))
-	  (push (gnus-copy-sequence (pop list)) out)
-	(push (pop list) out)))
-    (if list
-	(nconc (nreverse out) list)
-      (nreverse out))))
+(define-obsolete-function-alias 'gnus-copy-sequence 'copy-tree "27.1")
 
+;;; We could be using `seq-difference' here, but it's much slower
+;;; on these data sets.  See bug#50877.
 (defun gnus-set-difference (list1 list2)
   "Return a list of elements of LIST1 that do not appear in LIST2."
   (let ((hash2 (make-hash-table :test 'eq))
@@ -97,7 +89,7 @@ Both ranges must be in ascending order."
   (setq range2 (gnus-range-normalize range2))
   (let* ((new-range (cons nil (copy-sequence range1)))
          (r new-range)
-         (safe t))
+         ) ;; (safe t)
     (while (cdr r)
       (let* ((r1 (cadr r))
              (r2 (car range2))
@@ -189,12 +181,8 @@ Both lists have to be sorted over <."
 
 ;;;###autoload
 (defun gnus-intersection (list1 list2)
-  (let ((result nil))
-    (while list2
-      (when (memq (car list2) list1)
-	(setq result (cons (car list2) result)))
-      (setq list2 (cdr list2)))
-    result))
+  (declare (obsolete seq-intersection "28.1"))
+  (nreverse (seq-intersection list1 list2 #'eq)))
 
 ;;;###autoload
 (defun gnus-sorted-intersection (list1 list2)
@@ -455,7 +443,7 @@ modified."
   (if (or (null range1) (null range2))
       range1
     (let (out r1 r2 r1_min r1_max r2_min r2_max
-	      (range2 (gnus-copy-sequence range2)))
+	      (range2 (copy-tree range2)))
       (setq range1 (if (listp (cdr range1)) range1 (list range1))
 	    range2 (sort (if (listp (cdr range2)) range2 (list range2))
 			 (lambda (e1 e2)
