@@ -1,6 +1,6 @@
-;;; gnus-vm.el --- vm interface for Gnus
+;;; gnus-vm.el --- vm interface for Gnus  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1994-2017 Free Software Foundation, Inc.
+;; Copyright (C) 1994-2021 Free Software Foundation, Inc.
 
 ;; Author: Per Persson <pp@gnu.ai.mit.edu>
 ;; Keywords: news, mail
@@ -34,12 +34,6 @@
 (require 'gnus)
 (require 'gnus-msg)
 
-(eval-when-compile
-  (require 'cl))
-
-(autoload 'vm-mode "vm")
-(autoload 'vm-save-message "vm")
-
 (defvar gnus-vm-inhibit-window-system nil
   "Inhibit loading `win-vm' if using a window-system.
 Has to be set before gnus-vm is loaded.")
@@ -49,6 +43,8 @@ Has to be set before gnus-vm is loaded.")
     (when window-system
       (require 'win-vm))))
 
+(declare-function vm-mode "ext:vm" (&optional read-only))
+
 (defun gnus-vm-make-folder (&optional buffer)
   (require 'vm)
   (let ((article (or buffer (current-buffer)))
@@ -76,13 +72,15 @@ If N is a positive number, save the N next articles.
 If N is a negative number, save the N previous articles.
 If N is nil and any articles have been marked with the process mark,
 save those articles instead."
-  (interactive "P")
+  (interactive "P" gnus-article-mode gnus-summary-mode)
   (require 'gnus-art)
   (let ((gnus-default-article-saver 'gnus-summary-save-in-vm))
     (gnus-summary-save-article arg)))
 
+(declare-function vm-save-message "ext:vm-save" (folder &optional count))
+
 (defun gnus-summary-save-in-vm (&optional folder)
-  (interactive)
+  (interactive nil gnus-article-mode gnus-summary-mode)
   (require 'vm)
   (setq folder
 	(gnus-read-save-file-name
