@@ -1,6 +1,6 @@
-;;; mm-view.el --- functions for viewing MIME objects
+;;; mm-view.el --- functions for viewing MIME objects  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1998-2019 Free Software Foundation, Inc.
+;; Copyright (C) 1998-2021 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; This file is part of GNU Emacs.
@@ -22,7 +22,7 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
 (require 'mail-parse)
 (require 'mailcap)
 (require 'mm-bodies)
@@ -50,23 +50,27 @@
     (w3m . mm-inline-text-html-render-with-w3m)
     (w3m-standalone . mm-inline-text-html-render-with-w3m-standalone)
     (gnus-w3m . gnus-article-html)
-    (links mm-inline-render-with-file
-	   mm-links-remove-leading-blank
-	   "links" "-dump" file)
+    (links . mm-inline-render-with-links)
     (lynx mm-inline-render-with-stdin nil
 	  "lynx" "-dump" "-force_html" "-stdin" "-nolist")
     (html2text mm-inline-render-with-function html2text))
   "The attributes of renderer types for text/html.")
 
 (defcustom mm-fill-flowed t
-  "If non-nil a format=flowed article will be displayed flowed."
+  "If non-nil, format=flowed articles will be displayed flowed."
   :type 'boolean
   :version "22.1"
   :group 'mime-display)
 
+;; Not a defcustom, since it's usually overridden by the callers of
+;; the mm functions.
+(defvar mm-inline-font-lock t
+  "If non-nil, do font locking of inline media types that support it.")
+
 (defcustom mm-inline-large-images-proportion 0.9
-  "Maximum proportion of large image resized when
-`mm-inline-large-images' is set to resize."
+  "Maximum proportion large images can occupy in the buffer.
+This is only used if `mm-inline-large-images' is set to
+`resize'."
   :type 'float
   :version "24.1"
   :group 'mime-display)
@@ -98,11 +102,10 @@
     (insert "\n")
     (mm-handle-set-undisplayer
      handle
-     `(lambda ()
-	(let ((b ,b)
-	      (inhibit-read-only t))
-	  (remove-images b b)
-	  (delete-region b (1+ b)))))))
+     (lambda ()
+       (let ((inhibit-read-only t))
+	 (remove-images b b)
+	 (delete-region b (1+ b)))))))
 
 (defvar mm-w3m-setup nil
   "Whether gnus-article-mode has been setup to use emacs-w3m.")
@@ -131,7 +134,7 @@
 		 (equal "multipart" (mm-handle-media-supertype elem)))
 	(mm-w3m-cid-retrieve-1 url elem)))))
 
-(defun mm-w3m-cid-retrieve (url &rest args)
+(defun mm-w3m-cid-retrieve (url &rest _args)
   "Insert a content pointed by URL if it has the cid: scheme."
   (when (string-match "\\`cid:" url)
     (or (catch 'found-handle
@@ -143,6 +146,9 @@
 	    nil
 	  (message "Failed to find \"Content-ID: %s\"" url)))))
 
+(defvar w3m-force-redisplay)
+(defvar w3m-safe-url-regexp)
+
 (defun mm-inline-text-html-render-with-w3m (handle)
   "Render a text/html part using emacs-w3m."
   (mm-setup-w3m)
@@ -193,10 +199,11 @@
 			       'keymap w3m-minor-mode-map)))
 	(mm-handle-set-undisplayer
 	 handle
-	 `(lambda ()
-	    (let ((inhibit-read-only t))
-	      (delete-region ,(point-min-marker)
-			     ,(point-max-marker)))))))))
+	 (let ((beg (point-min-marker))
+	       (end (point-max-marker)))
+	   (lambda ()
+	     (let ((inhibit-read-only t))
+	       (delete-region beg end)))))))))
 
 (defcustom mm-w3m-standalone-supports-m17n-p 'undecided
   "T means the w3m command supports the m17n feature."
@@ -255,6 +262,7 @@
     (mm-inline-render-with-stdin handle nil "w3m" "-dump" "-T" "text/html")))
 
 (defun mm-links-remove-leading-blank ()
+  (declare (obsolete nil "28.1"))
   ;; Delete the annoying three spaces preceding each line of links
   ;; output.
   (goto-char (point-min))
@@ -262,29 +270,61 @@
     (delete-region (match-beginning 0) (match-end 0))))
 
 (defun mm-inline-wash-with-file (post-func cmd &rest args)
-  (let ((file (make-temp-file
-	       (expand-file-name "mm" mm-tmp-directory))))
-    (let ((coding-system-for-write 'binary))
-      (write-region (point-min) (point-max) file nil 'silent))
-    (delete-region (point-min) (point-max))
-    (unwind-protect
-	(apply 'call-process cmd nil t nil (mapcar 'eval args))
-      (delete-file file))
-    (and post-func (funcall post-func))))
+  (declare (obsolete nil "28.1"))
+  (with-suppressed-warnings ((lexical file))
+    (dlet ((file (make-temp-file
+	          (expand-file-name "mm" mm-tmp-directory))))
+      (let ((coding-system-for-write 'binary))
+        (write-region (point-min) (point-max) file nil 'silent))
+      (delete-region (point-min) (point-max))
+      (unwind-protect
+	  (apply #'call-process cmd nil t nil
+                 (mapcar (lambda (e) (eval e t)) args))
+        (delete-file file))
+      (and post-func (funcall post-func)))))
 
 (defun mm-inline-wash-with-stdin (post-func cmd &rest args)
   (let ((coding-system-for-write 'binary))
-    (apply 'call-process-region (point-min) (point-max)
+    (apply #'call-process-region (point-min) (point-max)
 	   cmd t t nil args))
   (and post-func (funcall post-func)))
 
 (defun mm-inline-render-with-file (handle post-func cmd &rest args)
+  (declare (obsolete nil "28.1"))
   (let ((source (mm-get-part handle)))
     (mm-insert-inline
      handle
      (mm-with-unibyte-buffer
        (insert source)
-       (apply 'mm-inline-wash-with-file post-func cmd args)
+       (with-suppressed-warnings ((obsolete mm-inline-wash-with-file))
+         (apply #'mm-inline-wash-with-file post-func cmd args))
+       (buffer-string)))))
+
+(defun mm-inline-render-with-links (handle)
+  (let ((source (mm-get-part handle))
+        file charset)
+    (mm-insert-inline
+     handle
+     (with-temp-buffer
+       (setq charset (mail-content-type-get (mm-handle-type handle) 'charset))
+       (insert source)
+       (unwind-protect
+           (progn
+             (setq file (make-temp-file (expand-file-name
+                                         "mm" mm-tmp-directory)))
+             (let ((coding-system-for-write 'binary))
+               (write-region (point-min) (point-max) file nil 'silent))
+             (delete-region (point-min) (point-max))
+             (if charset
+                 (with-environment-variables (("LANG" (format "en-US.%s"
+                                                              charset)))
+	           (call-process "links" nil t nil "-dump" file))
+               (call-process "links" nil t nil "-dump" file))
+             (goto-char (point-min))
+             (while (re-search-forward "^   " nil t)
+               (delete-region (match-beginning 0) (match-end 0))))
+         (when (and file (file-exists-p file))
+           (delete-file file)))
        (buffer-string)))))
 
 (defun mm-inline-render-with-stdin (handle post-func cmd &rest args)
@@ -293,7 +333,7 @@
      handle
      (mm-with-unibyte-buffer
        (insert source)
-       (apply 'mm-inline-wash-with-stdin post-func cmd args)
+       (apply #'mm-inline-wash-with-stdin post-func cmd args)
        (buffer-string)))))
 
 (defun mm-inline-render-with-function (handle func &rest args)
@@ -311,13 +351,15 @@
 
 (defun mm-inline-text-html (handle)
   (if (stringp (car handle))
-      (mapcar 'mm-inline-text-html (cdr handle))
+      (mapcar #'mm-inline-text-html (cdr handle))
     (let* ((func mm-text-html-renderer)
 	   (entry (assq func mm-text-html-renderer-alist))
 	   (inhibit-read-only t))
       (if entry
 	  (setq func (cdr entry)))
       (cond
+       ((null func)
+	(mm-insert-inline handle (mm-get-part handle)))
        ((functionp func)
 	(funcall func handle))
        (t
@@ -357,8 +399,8 @@
       (save-restriction
 	(narrow-to-region b (point))
 	(goto-char b)
-	(fill-flowed nil (equal (cdr (assoc 'delsp (mm-handle-type handle)))
-				"yes"))
+	(fill-flowed nil (cl-equalp (cdr (assoc 'delsp (mm-handle-type handle)))
+				    "yes"))
 	(goto-char (point-max))))
     (save-restriction
       (narrow-to-region b (point))
@@ -368,10 +410,13 @@
 	  (enriched-decode (point-min) (point-max))))
       (mm-handle-set-undisplayer
        handle
-       `(lambda ()
-          (let ((inhibit-read-only t))
-	    (delete-region ,(copy-marker (point-min) t)
-			   ,(point-max-marker))))))))
+       (if (= (point-min) (point-max))
+	   #'ignore
+	 (let ((beg (copy-marker (point-min) t))
+	       (end (point-max-marker)))
+	   (lambda ()
+	     (let ((inhibit-read-only t))
+	       (delete-region beg end)))))))))
 
 (defun mm-insert-inline (handle text)
   "Insert TEXT inline from HANDLE."
@@ -381,12 +426,13 @@
       (insert "\n"))
     (mm-handle-set-undisplayer
      handle
-     `(lambda ()
-	(let ((inhibit-read-only t))
-	  (delete-region ,(copy-marker b t)
-			 ,(point-marker)))))))
+     (let ((beg (copy-marker b t))
+           (end (point-marker)))
+       (lambda ()
+	 (let ((inhibit-read-only t))
+	   (delete-region beg end)))))))
 
-(defun mm-inline-audio (handle)
+(defun mm-inline-audio (_handle)
   (message "Not implemented"))
 
 (defun mm-view-message ()
@@ -403,12 +449,18 @@
   (fundamental-mode)
   (goto-char (point-min)))
 
+(defvar mm-inline-message-prepare-function nil
+  "Function called by `mm-inline-message' to do client specific setup.
+It is called with two parameters -- the MIME handle and the charset.")
+
 (defun mm-inline-message (handle)
+  "Insert HANDLE (a message/rfc822 part) into the current buffer.
+This function will call `mm-inline-message-prepare-function'
+after inserting the part."
   (let ((b (point))
 	(bolp (bolp))
 	(charset (mail-content-type-get
-		  (mm-handle-type handle) 'charset))
-	gnus-displaying-mime handles)
+		  (mm-handle-type handle) 'charset)))
     (when (and charset
 	       (stringp charset))
       (setq charset (intern (downcase charset)))
@@ -418,16 +470,8 @@
       (save-restriction
 	(narrow-to-region b b)
 	(mm-insert-part handle)
-	(let (gnus-article-mime-handles
-	      ;; disable prepare hook
-	      gnus-article-prepare-hook
-	      (gnus-newsgroup-charset
-	       (unless (eq charset 'gnus-decoded) ;; mm-uu might set it.
-		 (or charset gnus-newsgroup-charset))))
-	  (let ((gnus-original-article-buffer (mm-handle-buffer handle)))
-	    (run-hooks 'gnus-article-decode-hook))
-	  (gnus-article-prepare-display)
-	  (setq handles gnus-article-mime-handles))
+        (when mm-inline-message-prepare-function
+	  (funcall mm-inline-message-prepare-function handle charset))
 	(goto-char (point-min))
 	(unless bolp
 	  (insert "\n"))
@@ -435,14 +479,13 @@
 	(unless (bolp)
 	  (insert "\n"))
 	(insert "----------\n\n")
-	(when handles
-	  (setq gnus-article-mime-handles
-		(mm-merge-handles gnus-article-mime-handles handles)))
 	(mm-handle-set-undisplayer
 	 handle
-	 `(lambda ()
-	    (let ((inhibit-read-only t))
-	      (delete-region ,(point-min-marker) ,(point-max-marker)))))))))
+	 (let ((beg (point-min-marker))
+	       (end (point-max-marker)))
+	   (lambda ()
+	     (let ((inhibit-read-only t))
+	       (delete-region beg end)))))))))
 
 ;; Shut up byte-compiler.
 (defvar font-lock-mode-hook)
@@ -450,7 +493,7 @@
   "Insert HANDLE inline fontifying with MODE.
 If MODE is not set, try to find mode automatically."
   (let ((charset (mail-content-type-get (mm-handle-type handle) 'charset))
-	text coding-system)
+	text coding-system ovs)
     (unless (eq charset 'gnus-decoded)
       (mm-with-unibyte-buffer
 	(mm-insert-part handle)
@@ -468,38 +511,51 @@ If MODE is not set, try to find mode automatically."
 		       (buffer-string)))
 		    (coding-system
 		     (decode-coding-string text coding-system))
-		    (charset
-		     (mm-decode-string text charset))
-		    (t
-		     text)))
-      (require 'font-lock)
-      ;; I find font-lock a bit too verbose.
-      (let ((font-lock-verbose nil)
-	    (font-lock-support-mode nil)
+                    (t
+                     (mm-decode-string text (or charset 'undecided)))))
+      (let ((font-lock-verbose nil)     ; font-lock is a bit too verbose.
 	    (enable-local-variables nil))
-	;; Disable support modes, e.g., jit-lock, lazy-lock, etc.
-	;; Note: XEmacs people use `font-lock-mode-hook' to run those modes.
-	(set (make-local-variable 'font-lock-mode-hook) nil)
+        ;; We used to set font-lock-mode-hook to nil to avoid enabling
+        ;; support modes, but now that we use font-lock-ensure, support modes
+        ;; aren't a problem any more.  So we could probably get rid of this
+        ;; setting now, but it seems harmless and potentially still useful.
+	(setq-local font-lock-mode-hook nil)
         (setq buffer-file-name (mm-handle-filename handle))
 	(with-demoted-errors
-	  (if mode
-	      (save-window-excursion
-		(switch-to-buffer (current-buffer))
-		(funcall mode))
+	    (if mode
+                (save-window-excursion
+                  ;; According to Katsumi Yamaoka <yamaoka@jpl.org>, org-mode
+                  ;; requires the buffer to be temporarily displayed here, but
+                  ;; I could not reproduce this problem.  Furthermore, if
+                  ;; there's such a problem, we should fix org-mode rather than
+                  ;; use switch-to-buffer which can have undesirable
+                  ;; side-effects!
+                  ;;(switch-to-buffer (current-buffer))
+	          (funcall mode))
 	    (let ((auto-mode-alist
 		   (delq (rassq 'doc-view-mode-maybe auto-mode-alist)
 			 (copy-sequence auto-mode-alist))))
-	      (set-auto-mode)))
-	  ;; The mode function might have already turned on font-lock.
+	      ;; Don't run hooks that might assume buffer-file-name
+	      ;; really associates buffer with a file (bug#39190).
+	      (delay-mode-hooks (set-auto-mode))
+	      (setq mode major-mode)))
 	  ;; Do not fontify if the guess mode is fundamental.
-	  (unless (or font-lock-mode
-		      (eq major-mode 'fundamental-mode))
+	  (when (and (not (eq major-mode 'fundamental-mode))
+		     mm-inline-font-lock)
 	    (font-lock-ensure))))
       (setq text (buffer-string))
+      (when (eq mode 'diff-mode)
+	(setq ovs (mapcar (lambda (ov) (list ov (overlay-start ov)
+	                                        (overlay-end ov)))
+	                  (overlays-in (point-min) (point-max)))))
       ;; Set buffer unmodified to avoid confirmation when killing the
       ;; buffer.
       (set-buffer-modified-p nil))
-    (mm-insert-inline handle text)))
+    (let ((b (- (point) (save-restriction (widen) (point-min)))))
+      (mm-insert-inline handle text)
+      (dolist (ov ovs)
+	(move-overlay (nth 0 ov) (+ (nth 1 ov) b)
+	                         (+ (nth 2 ov) b) (current-buffer))))))
 
 ;; Shouldn't these functions check whether the user even wants to use
 ;; font-lock?  Also, it would be nice to change for the size of the
@@ -523,7 +579,7 @@ If MODE is not set, try to find mode automatically."
   (mm-display-inline-fontify handle 'shell-script-mode))
 
 (defun mm-display-javascript-inline (handle)
-  "Show JavsScript code from HANDLE inline."
+  "Show JavaScript code from HANDLE inline."
   (mm-display-inline-fontify handle 'javascript-mode))
 
 ;;      id-signedData OBJECT IDENTIFIER ::= { iso(1) member-body(2)
@@ -561,31 +617,38 @@ If MODE is not set, try to find mode automatically."
 	   (error "Could not identify PKCS#7 type")))))
 
 (defun mm-view-pkcs7 (handle &optional from)
-  (case (mm-view-pkcs7-get-type handle)
+  (cl-case (mm-view-pkcs7-get-type handle)
     (enveloped (mm-view-pkcs7-decrypt handle from))
     (signed (mm-view-pkcs7-verify handle))
     (otherwise (error "Unknown or unimplemented PKCS#7 type"))))
 
 (defun mm-view-pkcs7-verify (handle)
   (let ((verified nil))
-    (with-temp-buffer
-      (insert "MIME-Version: 1.0\n")
-      (mm-insert-headers "application/pkcs7-mime" "base64" "smime.p7m")
-      (insert-buffer-substring (mm-handle-buffer handle))
-      (setq verified (smime-verify-region (point-min) (point-max))))
-    (goto-char (point-min))
-    (mm-insert-part handle)
-    (if (search-forward "Content-Type: " nil t)
-	(delete-region (point-min) (match-beginning 0)))
-    (goto-char (point-max))
-    (if (re-search-backward "--\r?\n?" nil t)
-	(delete-region (match-end 0) (point-max)))
-    (unless verified
-      (insert-buffer-substring smime-details-buffer)))
-  (goto-char (point-min))
-  (while (search-forward "\r\n" nil t)
-    (replace-match "\n"))
-  t)
+    (if (eq mml-smime-use 'epg)
+	;; Use EPG/gpgsm
+	(insert
+	 (with-temp-buffer
+	   (insert-buffer-substring (mm-handle-buffer handle))
+	   (goto-char (point-min))
+	   (let ((part (base64-decode-string (buffer-string)))
+		 (context (epg-make-context 'CMS)))
+	     (prog1
+		 (epg-verify-string context part)
+	       (let ((result (car (epg-context-result-for context 'verify))))
+		 (mm-sec-status
+		  'gnus-info (epg-signature-status result)
+		  'gnus-details
+		  (format "%s:%s" (epg-signature-validity result)
+			  (epg-signature-key-id result))))))))
+      (with-temp-buffer
+	(insert "MIME-Version: 1.0\n")
+	(mm-insert-headers "application/pkcs7-mime" "base64" "smime.p7m")
+	(insert-buffer-substring (mm-handle-buffer handle))
+	(setq verified (smime-verify-region (point-min) (point-max))))
+      (if verified
+	  (insert verified)
+	(insert-buffer-substring smime-details-buffer)))
+    t))
 
 (autoload 'epg-decrypt-string "epg")
 
