@@ -1,8 +1,8 @@
-;;; nnmairix.el --- Mairix back end for Gnus, the Emacs newsreader
+;;; nnmairix.el --- Mairix back end for Gnus, the Emacs newsreader  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2007-2018 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2021 Free Software Foundation, Inc.
 
-;; Author: David Engster <dengste@eml.cc>
+;; Author: David Engster <deng@randomsample.de>
 ;; Keywords: mail searching
 ;; Old-Version: 0.6
 
@@ -134,8 +134,6 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))       ;For (pop (cdr ogroup)).
-
 (require 'nnoo)
 (require 'gnus-group)
 (require 'gnus-sum)
@@ -195,8 +193,8 @@
   (define-key gnus-summary-mode-map
     (kbd "G G u") 'nnmairix-remove-tick-mark-original-article))
 
-(add-hook 'gnus-group-mode-hook 'nnmairix-group-mode-hook)
-(add-hook 'gnus-summary-mode-hook 'nnmairix-summary-mode-hook)
+(add-hook 'gnus-group-mode-hook #'nnmairix-group-mode-hook)
+(add-hook 'gnus-summary-mode-hook #'nnmairix-summary-mode-hook)
 
 ;; ;;;###autoload
 ;; (defun nnmairix-initialize (&optional force)
@@ -204,8 +202,8 @@
 ;;   (if (not (or (file-readable-p "~/.mairixrc")
 ;; 	       force))
 ;;       (message "No file `~/.mairixrc', skipping nnmairix setup")
-;;     (add-hook 'gnus-group-mode-hook 'nnmairix-group-mode-hook)
-;;     (add-hook 'gnus-summary-mode-hook 'nnmairix-summary-mode-hook)))
+;;     (add-hook 'gnus-group-mode-hook #'nnmairix-group-mode-hook)
+;;     (add-hook 'gnus-summary-mode-hook #'nnmairix-summary-mode-hook)))
 
 ;; Customizable stuff
 
@@ -221,20 +219,17 @@ server will be this prefix plus a random number.  You can delete
 unused nnmairix groups on the back end using
 `nnmairix-purge-old-groups'."
   :version "23.1"
-  :type 'string
-  :group 'nnmairix)
+  :type 'string)
 
 (defcustom nnmairix-mairix-output-buffer "*mairix output*"
   "Buffer used for mairix output."
   :version "23.1"
-  :type 'string
-  :group 'nnmairix)
+  :type 'string)
 
 (defcustom nnmairix-customize-query-buffer "*mairix query*"
   "Name of the buffer for customizing Mairix queries."
   :version "23.1"
-  :type 'string
-  :group 'nnmairix)
+  :type 'string)
 
 (defcustom nnmairix-mairix-update-options '("-F" "-Q")
   "Options when calling mairix for updating the database.
@@ -242,21 +237,18 @@ The default is \"-F\" and \"-Q\" for making updates faster.  You
 should call mairix without these options from time to
 time (e.g. via cron job)."
   :version "23.1"
-  :type '(repeat string)
-  :group 'nnmairix)
+  :type '(repeat string))
 
 (defcustom nnmairix-mairix-search-options '("-Q")
   "Options when calling mairix for searching.
 The default is \"-Q\" for making searching faster."
   :version "23.1"
-  :type '(repeat string)
-  :group 'nnmairix)
+  :type '(repeat string))
 
 (defcustom nnmairix-mairix-synchronous-update nil
   "Set this to t if you want Emacs to wait for mairix updating the database."
   :version "23.1"
-  :type 'boolean
-  :group 'nnmairix)
+  :type 'boolean)
 
 (defcustom nnmairix-rename-files-for-nnml t
   "Rename nnml mail files so that they are consecutively numbered.
@@ -265,8 +257,7 @@ article numbers which will produce wrong article counts by
 Gnus.  This option controls whether nnmairix should rename the
 files consecutively."
   :version "23.1"
-  :type 'boolean
-  :group 'nnmairix)
+  :type 'boolean)
 
 (defcustom nnmairix-widget-fields-list
   '(("from" "f" "From") ("to" "t" "To") ("cc" "c" "Cc")
@@ -290,28 +281,25 @@ nil for disabling this)."
 			  (const :tag "Subject" "subject")
 			  (const :tag "Message ID" "Message-ID"))
 		  (string :tag "Command")
-		  (string :tag "Description")))
-  :group 'nnmairix)
+		  (string :tag "Description"))))
 
 (defcustom nnmairix-widget-select-window-function
   (lambda () (select-window (get-largest-window)))
   "Function for selecting the window for customizing the mairix query.
 The default chooses the largest window in the current frame."
   :version "23.1"
-  :type 'function
-  :group 'nnmairix)
+  :type 'function)
 
 (defcustom nnmairix-propagate-marks-upon-close t
   "Flag if marks should be propagated upon closing a group.
-The default of this variable is t.  If set to 'ask, the
+The default of this variable is t.  If set to `ask', the
 user will be asked if the flags should be propagated when the
 group is closed.  If set to nil, the user will have to manually
 call `nnmairix-propagate-marks'."
   :version "23.1"
   :type '(choice (const :tag "always" t)
 		 (const :tag "ask" ask)
-		 (const :tag "never" nil))
-  :group 'nnmairix)
+		 (const :tag "never" nil)))
 
 (defcustom nnmairix-propagate-marks-to-nnmairix-groups nil
   "Flag if marks from original articles should be seen in nnmairix groups.
@@ -321,8 +309,7 @@ e.g. an IMAP server (which stores the marks in the maildir file
 name).  You may safely set this to t for testing - the worst that
 can happen are wrong marks in nnmairix groups."
   :version "23.1"
-  :type 'boolean
-  :group 'nnmairix)
+  :type 'boolean)
 
 (defcustom nnmairix-only-use-registry nil
   "Use only the registry for determining original group(s).
@@ -332,16 +319,14 @@ propagating marks).  If set to nil, it will also try to determine
 the group from an additional mairix search which might be slow
 when propagating lots of marks."
   :version "23.1"
-  :type 'boolean
-  :group 'nnmairix)
+  :type 'boolean)
 
 (defcustom nnmairix-allowfast-default nil
   "Whether fast entering should be the default for nnmairix groups.
 You may set this to t to make entering the group faster, but note that
 this might lead to problems, especially when used with marks propagation."
   :version "23.1"
-  :type 'boolean
-  :group 'nnmairix)
+  :type 'boolean)
 
 ;; ==== Other variables
 
@@ -383,7 +368,7 @@ wrong count of total articles shown by Gnus.")
 its maildir mail folders (e.g. the Dovecot IMAP server or mutt).")
 
 (defvoo nnmairix-default-group nil
-  "Default search group. This is the group which is used for all
+  "Default search group.  This is the group which is used for all
 temporary searches, e.g. nnmairix-search.")
 
 ;;; === Internal variables
@@ -419,7 +404,7 @@ Other back ends might or might not work.")
   (setq nnmairix-current-server server)
   (nnoo-change-server 'nnmairix server definitions))
 
-(deffoo nnmairix-request-group (group &optional server fast info)
+(deffoo nnmairix-request-group (group &optional server fast _info)
   ;; Call mairix and request group on back end server
   (when server (nnmairix-open-server server))
   (let* ((qualgroup (if server
@@ -432,7 +417,7 @@ Other back ends might or might not work.")
 	 (backendmethod (gnus-server-to-method
 			 (format "%s:%s" (symbol-name nnmairix-backend)
 				 nnmairix-backend-server)))
-	 rval mfolder folderpath args)
+	 rval mfolder folderpath) ;; args
     (cond
      ((not folder)
       ;; No folder parameter -> error
@@ -512,12 +497,12 @@ Other back ends might or might not work.")
 	nil))))))
 
 
-(deffoo nnmairix-request-create-group (group &optional server args)
+(deffoo nnmairix-request-create-group (group &optional server _args)
   (let ((qualgroup (if server (gnus-group-prefixed-name group (list 'nnmairix server))
 		     group))
 	(exist t)
 	(count 0)
-	groupname info)
+	groupname) ;; info
     (when server (nnmairix-open-server server))
     (gnus-group-add-parameter qualgroup '(query . nil))
     (gnus-group-add-parameter qualgroup '(threads . nil))
@@ -576,7 +561,7 @@ Other back ends might or might not work.")
 (deffoo nnmairix-request-list (&optional server)
   (when server (nnmairix-open-server server))
   (if (nnmairix-call-backend "request-list" nnmairix-backend-server)
-      (let (cpoint cur qualgroup folder)
+      (let (cpoint cur qualgroup) ;; folder
 	(with-current-buffer nntp-server-buffer
 	  (goto-char (point-min))
 	  (setq cpoint (point))
@@ -605,7 +590,7 @@ Other back ends might or might not work.")
     (nnmairix-open-server server))
   (let* ((qualgroup (gnus-group-prefixed-name group (list 'nnmairix nnmairix-current-server)))
 	 (propmarks (gnus-group-get-parameter qualgroup 'propmarks))
-	 (propto (gnus-group-get-parameter qualgroup 'propto t))
+	 ;; (propto (gnus-group-get-parameter qualgroup 'propto t))
 	 (corr (nnmairix-get-numcorr group server))
 	 (folder (nnmairix-get-backend-folder group server)))
     (save-excursion
@@ -613,7 +598,7 @@ Other back ends might or might not work.")
 	(let ((type (nth 1 cur))
 	      (cmdmarks (nth 2 cur))
 	      (range (gnus-uncompress-range (nth 0 cur)))
-	      mid ogroup number method temp)
+	      mid ogroup temp) ;; number method
 	  (when (and corr
 		     (not (zerop (cadr corr))))
 	    (setq range (mapcar (lambda (arg)
@@ -676,7 +661,7 @@ Other back ends might or might not work.")
     (nnmairix-open-server server))
   (let* ((qualgroup (gnus-group-prefixed-name group (list 'nnmairix nnmairix-current-server)))
 	 (propmarks (gnus-group-get-parameter qualgroup 'propmarks))
-	 method)
+	 ) ;; method
     (when (and propmarks
 	       nnmairix-marks-cache)
       (when (or (eq nnmairix-propagate-marks-upon-close t)
@@ -691,9 +676,9 @@ Other back ends might or might not work.")
 (autoload 'nnimap-request-update-info-internal "nnimap")
 
 (deffoo nnmairix-request-marks (group info &optional server)
-;; propagate info from underlying IMAP folder to nnmairix group
-;; This is currently experimental and must be explicitly activated
-;; with nnmairix-propagate-marks-to-nnmairix-group
+  ;; propagate info from underlying IMAP folder to nnmairix group
+  ;; This is currently experimental and must be explicitly activated
+  ;; with nnmairix-propagate-marks-to-nnmairix-group
   (when server
     (nnmairix-open-server server))
   (let* ((qualgroup (gnus-group-prefixed-name
@@ -705,7 +690,7 @@ Other back ends might or might not work.")
 	 (corr (nnmairix-get-numcorr group server))
 	 (docorr (and corr (not (zerop (cadr corr)))))
 	 (folderinfo `(,group 1 ((1 . 1))))
-	 readrange marks)
+	 ) ;; readrange marks
       (when (and propmarks
 		 nnmairix-propagate-marks-to-nnmairix-groups)
 	;; these groups are not subscribed, so we have to ask the back end directly
@@ -713,29 +698,29 @@ Other back ends might or might not work.")
 	    (nnimap-request-update-info-internal folder folderinfo nnmairix-backend-server)
 	  (nnmairix-call-backend "request-update-info" folder folderinfo nnmairix-backend-server))
 	;; set range of read articles
-	(gnus-info-set-read
-	 info
-	 (if docorr
-	     (nnmairix-map-range
-	      `(lambda (x) (+ x ,(cadr corr)))
-	      (gnus-info-read folderinfo))
-	   (gnus-info-read folderinfo)))
+	(setf (gnus-info-read info)
+	      (if docorr
+	          (nnmairix-map-range
+	           (let ((off (cadr corr)))
+	             (lambda (x) (+ x off)))
+	           (gnus-info-read folderinfo))
+	        (gnus-info-read folderinfo)))
 	;; set other marks
-	(gnus-info-set-marks
-	 info
-	 (if docorr
-	     (mapcar (lambda (cur)
-			 (cons
-			  (car cur)
-			  (nnmairix-map-range
-			   `(lambda (x) (+ x ,(cadr corr)))
-			   (list (cadr cur)))))
-		     (gnus-info-marks folderinfo))
-	   (gnus-info-marks folderinfo))))
+	(setf (gnus-info-marks info)
+	      (if docorr
+		  (mapcar (lambda (cur)
+			    (cons
+			     (car cur)
+			     (nnmairix-map-range
+			      (let ((off (cadr corr)))
+			        (lambda (x) (+ x off)))
+			      (list (cadr cur)))))
+			  (gnus-info-marks folderinfo))
+		(gnus-info-marks folderinfo))))
       (when (eq readmarks 'unread)
-	(gnus-info-set-read info nil))
+	(setf (gnus-info-read info) nil))
       (when (eq readmarks 'read)
-	(gnus-info-set-read info (gnus-active qualgroup))))
+	(setf (gnus-info-read info) (gnus-active qualgroup))))
   t)
 
 (nnoo-define-skeleton nnmairix)
@@ -759,10 +744,9 @@ called interactively, user will be asked for parameters."
   (when (not (listp query))
     (setq query (list query)))
   (when (and server group query)
-    (save-excursion
-      (let ((groupname (gnus-group-prefixed-name group server))
-	    info)
-	(set-buffer gnus-group-buffer)
+    (let ((groupname (gnus-group-prefixed-name group server))
+          ) ;; info
+      (with-current-buffer gnus-group-buffer
 	(gnus-group-make-group group server)
 	(gnus-group-set-parameter groupname 'query  query)
 	(gnus-group-set-parameter groupname 'threads threads)
@@ -785,7 +769,7 @@ called interactively, user will be asked for parameters."
 	(setq finished (not (y-or-n-p "Add another search query? "))
 	      achar nil))
     (nnmairix-search
-     (mapconcat 'identity query " ")
+     (mapconcat #'identity query " ")
      (car (nnmairix-get-server))
      (y-or-n-p "Include whole threads? "))))
 
@@ -794,7 +778,7 @@ called interactively, user will be asked for parameters."
   (interactive)
   (let ((char-header nnmairix-interactive-query-parameters)
 	(server (nnmairix-backend-to-server gnus-current-select-method))
-	 query achar header finished group threads cq)
+	 query achar header finished group threads) ;; cq
     (when (or (not (gnus-buffer-live-p gnus-article-buffer))
 	      (not (gnus-buffer-live-p gnus-summary-buffer)))
       (error "No article or summary buffer"))
@@ -812,7 +796,8 @@ called interactively, user will be asked for parameters."
 	    (setq achar nil)))
 	(set-buffer gnus-article-buffer)
 	(setq header nil)
-	(when (setq cq (nth 1 (assoc achar char-header)))
+	(when ;; (setq cq
+	    (nth 1 (assoc achar char-header)) ;;)
 	  (setq header
 		(nnmairix-replace-illegal-chars
 		 (gnus-fetch-field (nth 1 (assoc achar char-header))))))
@@ -826,7 +811,7 @@ called interactively, user will be asked for parameters."
     (setq group (read-string "Group name: "))
     (set-buffer gnus-summary-buffer)
     (message "Creating group %s on server %s with query %s." group
-	     (gnus-method-to-server server) (mapconcat 'identity query " "))
+	     (gnus-method-to-server server) (mapconcat #'identity query " "))
     (nnmairix-create-search-group server group query threads)))
 
 (defun nnmairix-create-server-and-default-group ()
@@ -843,7 +828,7 @@ All necessary information will be queried from the user."
 	(hidden (and (string-match "^nn\\(imap\\|maildir\\)$" backend)
 		     (y-or-n-p
 		      "Does the back end server work with maildir++ (i.e. hidden directories)? ")))
-	create)
+	) ;; create
 
     (apply (intern (format "%s-%s" backend "open-server"))
 	   (list servername))
@@ -868,7 +853,7 @@ All necessary information will be queried from the user."
     (if (eq (car method) 'nnmairix)
 	(progn
 	  (when (listp oldquery)
-	    (setq oldquery (mapconcat 'identity oldquery " ")))
+	    (setq oldquery (mapconcat #'identity oldquery " ")))
 	  (setq query (or query
 			  (read-string "New query: " oldquery)))
 	  (when (stringp query)
@@ -1025,7 +1010,7 @@ before deleting a group on the back end.  SERVER specifies nnmairix server."
     (if (nnmairix-open-server (nth 1 server))
 	(when (nnmairix-call-backend
 	       "request-list" nnmairix-backend-server)
-	  (let (cur qualgroup folder)
+	  (let (cur qualgroup) ;; folder
 	    (with-current-buffer nntp-server-buffer
 	      (goto-char (point-min))
 	      (while (re-search-forward nnmairix-group-regexp (point-max) t)
@@ -1070,7 +1055,7 @@ with `nnmairix-mairix-update-options'."
 	    (if (> (length commandsplit) 1)
 		(setq args (append args (cdr commandsplit) nnmairix-mairix-update-options))
 	      (setq args (append args nnmairix-mairix-update-options)))
-	    (apply 'call-process args)
+	    (apply #'call-process args)
 	    (nnheader-message 7 "Updating mairix database for %s... done" cur))
 	(progn
 	  (setq args (append (list cur (get-buffer nnmairix-mairix-output-buffer)
@@ -1078,7 +1063,7 @@ with `nnmairix-mairix-update-options'."
 	  (if (> (length commandsplit) 1)
 	      (setq args (append args (cdr commandsplit) nnmairix-mairix-update-options))
 	    (setq args (append args nnmairix-mairix-update-options)))
-	  (set-process-sentinel (apply 'start-process args)
+	  (set-process-sentinel (apply #'start-process args)
 				'nnmairix-sentinel-mairix-update-finished))))))
 
 (defun nnmairix-group-delete-recreate-this-group ()
@@ -1098,7 +1083,7 @@ show wrong article counts."
 (defun nnmairix-propagate-marks (&optional server)
   "Propagate marks from nnmairix group to original articles.
 Unless SERVER is explicitly specified, will use the last opened
-nnmairix server. Only marks from current session will be set."
+nnmairix server.  Only marks from current session will be set."
   (interactive)
   (if server
       (nnmairix-open-server server)
@@ -1188,7 +1173,7 @@ Marks propagation has to be enabled for this to work."
     (error "Not in a nnmairix group"))
   (save-excursion
     (let ((mid (mail-header-message-id (gnus-summary-article-header)))
-	  groups cur)
+	  groups) ;; cur
       (when mid
 	(setq groups (nnmairix-determine-original-group-from-registry mid))
 	(unless (or groups
@@ -1251,7 +1236,7 @@ Marks propagation has to be enabled for this to work."
 If THREADS is non-nil, enable full threads."
   (let ((args (cons (car command) '(nil t nil))))
     (with-current-buffer
-       (get-buffer-create nnmairix-mairix-output-buffer)
+       (gnus-get-buffer-create nnmairix-mairix-output-buffer)
       (erase-buffer)
       (when (> (length command) 1)
 	(setq args (append args (cdr command))))
@@ -1262,19 +1247,19 @@ If THREADS is non-nil, enable full threads."
 	(setq args (append args '("-c"))))
       (when threads
 	(setq args (append args '("-t"))))
-      (apply 'call-process
+      (apply #'call-process
 	     (append args (list "-o" folder) searchquery)))))
 
 (defun nnmairix-call-mairix-binary-raw (command query)
   "Call mairix binary with COMMAND and QUERY in raw mode."
   (let ((args (cons (car command) '(nil t nil))))
     (with-current-buffer
-       (get-buffer-create nnmairix-mairix-output-buffer)
+       (gnus-get-buffer-create nnmairix-mairix-output-buffer)
       (erase-buffer)
       (when (> (length command) 1)
         (setq args (append args (cdr command))))
       (setq args (append args '("-r")))
-      (apply 'call-process
+      (apply #'call-process
              (append args query)))))
 
 (defun nnmairix-get-server ()
@@ -1315,7 +1300,7 @@ If ALL is t, return also the unopened/failed ones."
   "Return list of valid back end servers for nnmairix groups."
   (let ((alist gnus-opened-servers)
 	(mairixservers (nnmairix-get-nnmairix-servers t))
-	server mserver openedserver occ cur)
+	server mserver openedserver occ) ;; cur
     ;; Get list of all nnmairix backends (i.e. backends which are
     ;; already occupied)
     (dolist (cur mairixservers)
@@ -1384,9 +1369,9 @@ This should correct problems of wrong article counts when using
 nnmairix with nnml backends."
   (let* ((files
 	 (sort
-	  (mapcar 'string-to-number
+	  (mapcar #'string-to-number
 		  (directory-files path nil "[0-9]+" t))
-	  '<))
+	  #'<))
 	 (lastplusone (car files))
 	 (path (file-name-as-directory path)))
     (dolist (cur files)
@@ -1402,14 +1387,14 @@ nnmairix with nnml backends."
   "Replace folder names in Xref header and correct article numbers.
 Do this for all ARTICLES on BACKENDGROUP.  Replace using
 MAIRIXGROUP.  NUMC contains values for article number correction.
-TYPE is either 'nov or 'headers."
+TYPE is either `nov' or `headers'."
   (nnheader-message 7 "nnmairix: Rewriting headers...")
   (cond
    ((eq type 'nov)
-    (let ((buf (get-buffer-create " *nnmairix buffer*"))
+    (let ((buf (gnus-get-buffer-create " *nnmairix buffer*"))
 	  (corr (not (zerop numc)))
 	  (name (buffer-name nntp-server-buffer))
-	  header cur xref)
+	  cur xref) ;; header
       (with-current-buffer buf
 	(erase-buffer)
 	(set-buffer nntp-server-buffer)
@@ -1421,12 +1406,12 @@ TYPE is either 'nov or 'headers."
 	     (setq cur (nnheader-parse-nov))
 	     (when corr
 	       (setq article (+ (mail-header-number cur) numc))
-	       (mail-header-set-number cur article))
+	       (setf (mail-header-number cur) article))
 	     (setq xref (mail-header-xref cur))
 	     (when (and (stringp xref)
 			(string-match (format "[ \t]%s:[0-9]+" backendgroup) xref))
 	       (setq xref (replace-match (format " %s:%d" mairixgroup article) t nil xref))
-	       (mail-header-set-xref cur xref))
+	       (setf (mail-header-xref cur) xref))
 	     (set-buffer buf)
 	     (nnheader-insert-nov cur)
 	     (set-buffer nntp-server-buffer)
@@ -1451,7 +1436,7 @@ TYPE is either 'nov or 'headers."
 (defun nnmairix-backend-to-server (server)
   "Return nnmairix server most probably responsible for back end SERVER.
 User will be asked if this cannot be determined.  Result is saved in
-parameter 'indexed-servers of corresponding default search
+parameter `indexed-servers' of corresponding default search
 group."
   (let ((allservers (nnmairix-get-nnmairix-servers))
 	mairixserver found defaultgroup)
@@ -1550,9 +1535,8 @@ See %s for details" proc nnmairix-mairix-output-buffer)))
 (defun nnmairix-create-message-line-for-search ()
   "Create message line for interactive query in minibuffer."
   (mapconcat
-   (function
-    (lambda (cur)
-      (format "%c=%s" (car cur) (nth 3 cur))))
+   (lambda (cur)
+     (format "%c=%s" (car cur) (nth 3 cur)))
    nnmairix-interactive-query-parameters ","))
 
 (defun nnmairix-replace-illegal-chars (header)
@@ -1603,7 +1587,7 @@ search in raw mode."
   (when (not (gnus-buffer-live-p gnus-article-buffer))
     (error "No article buffer available"))
   (let ((server (nth 1 gnus-current-select-method))
-	mid rval group allgroups)
+	mid group allgroups) ;; rval
     ;; get message id
     (with-current-buffer gnus-article-buffer
       (gnus-summary-toggle-header 1)
@@ -1645,7 +1629,7 @@ SERVER."
   (while (string-match "[<>]" mid)
     (setq mid (replace-match "" t t mid)))
   ;; mairix somehow does not like '$' in message-id
-  (when (string-match "\\$" mid)
+  (when (string-search "$" mid)
     (setq mid (concat mid "=")))
   (while (string-match "\\$" mid)
     (setq mid (replace-match "=," t t mid)))
@@ -1776,8 +1760,8 @@ If VERSION is a string: must be contained in mairix version output."
 	(setq versionstring
 	      (let* ((commandsplit (split-string nnmairix-mairix-command))
 		     (args (append (list (car commandsplit))
-				  `(nil t nil) (cdr commandsplit) '("-V"))))
-	      (apply 'call-process args)
+				   '(nil t nil) (cdr commandsplit) '("-V"))))
+	      (apply #'call-process args)
 	      (goto-char (point-min))
 	      (re-search-forward "mairix.*")
 	      (match-string 0))))
@@ -1813,13 +1797,12 @@ If VERSION is a string: must be contained in mairix version output."
       (gnus-summary-toggle-header 1)
       (set-buffer gnus-article-buffer)
       (mapcar
-       (function
-	(lambda (field)
-	  (list (car (cddr field))
-		(if (car field)
-		    (nnmairix-replace-illegal-chars
-		     (gnus-fetch-field (car field)))
-		  nil))))
+       (lambda (field)
+         (list (car (cddr field))
+               (if (car field)
+                   (nnmairix-replace-illegal-chars
+                    (gnus-fetch-field (car field)))
+                 nil)))
        nnmairix-widget-fields-list))))
 
 
@@ -1835,10 +1818,10 @@ MVALUES may contain values from current article."
   (widget-create 'push-button
 		 :notify
 		 (if mvalues
-		     (lambda (&rest ignore)
+		     (lambda (&rest _ignore)
 		       (nnmairix-widget-send-query nnmairix-widgets
 						   t))
-		   (lambda (&rest ignore)
+		   (lambda (&rest _ignore)
 		     (nnmairix-widget-send-query nnmairix-widgets
 						 nil)))
 		 "Send Query")
@@ -1846,16 +1829,16 @@ MVALUES may contain values from current article."
   (widget-create 'push-button
 		 :notify
 		 (if mvalues
-		     (lambda (&rest ignore)
+		     (lambda (&rest _ignore)
 		       (nnmairix-widget-create-group nnmairix-widgets
 						     t))
-		   (lambda (&rest ignore)
+		   (lambda (&rest _ignore)
 		     (nnmairix-widget-create-group nnmairix-widgets
 						   nil)))
 		 "Create permanent group")
   (widget-insert "   ")
   (widget-create 'push-button
-		 :notify (lambda (&rest ignore)
+		 :notify (lambda (&rest _ignore)
 			   (kill-buffer nnmairix-customize-query-buffer))
 		 "Cancel")
   (use-local-map widget-keymap)
@@ -1913,25 +1896,24 @@ If WITHVALUES is t, query is based on current article."
     (when (member 'flags nnmairix-widget-other)
       (setq flag
 	    (mapconcat
-	     (function
-	      (lambda (flag)
-		(setq temp
-		      (widget-value (cadr (assoc (car flag) nnmairix-widgets))))
-		(if (string= "yes" temp)
-		    (cadr flag)
-		  (if (string= "no" temp)
-		      (concat "-" (cadr flag))))))
+             (lambda (flag)
+               (setq temp
+                     (widget-value (cadr (assoc (car flag) nnmairix-widgets))))
+               (if (string= "yes" temp)
+                   (cadr flag)
+                 (if (string= "no" temp)
+                     (concat "-" (cadr flag)))))
 	     '(("seen" "s") ("replied" "r") ("flagged" "f")) ""))
       (when (not (zerop (length flag)))
 	(push (concat "F:" flag) query)))
     ;; return query string
-    (mapconcat 'identity query " ")))
+    (mapconcat #'identity query " ")))
 
 
 (defun nnmairix-widget-create-query (&optional values)
   "Create widgets for creating mairix queries.
 Fill in VALUES if based on an article."
-  (let (allwidgets)
+  ;;(let (allwidgets)
     (when (get-buffer nnmairix-customize-query-buffer)
       (kill-buffer nnmairix-customize-query-buffer))
     (switch-to-buffer nnmairix-customize-query-buffer)
@@ -1962,7 +1944,7 @@ Fill in VALUES if based on an article."
     (when (member 'threads nnmairix-widget-other)
       (widget-insert "\n")
       (nnmairix-widget-add "Threads" 'checkbox nil))
-      (widget-insert " Show full threads\n\n")))
+      (widget-insert " Show full threads\n\n")) ;; )
 
 (defun nnmairix-widget-build-editable-fields (values)
   "Build editable field widgets in `nnmairix-widget-fields-list'.
@@ -1970,32 +1952,31 @@ VALUES may contain values for editable fields from current article."
   ;; how can this be done less ugly?
   (let ((ret))
     (mapc
-     (function
-      (lambda (field)
-	(setq field (car (cddr field)))
-	(setq ret
-	      (nconc
-	       (list
-		(list
-		 (concat "c" field)
-		 (widget-create 'checkbox
-				:tag field
-				:notify (lambda (widget &rest ignore)
-					  (nnmairix-widget-toggle-activate widget))
-				nil)))
-	       (list
-		(list
-		 (concat "e" field)
-		 (widget-create 'editable-field
-				:size 60
-				:format (concat " " field ":"
-						(make-string (- 11 (length field)) ?\ )
-						"%v")
-				:value (or (cadr (assoc field values)) ""))))
-	       ret))
-	(widget-insert "\n")
-	;; Deactivate editable field
-	(widget-apply (cadr (nth 1 ret)) :deactivate)))
+     (lambda (field)
+       (setq field (car (cddr field)))
+       (setq ret
+             (nconc
+              (list
+               (list
+                (concat "c" field)
+                (widget-create 'checkbox
+                               :tag field
+                               :notify (lambda (widget &rest _ignore)
+                                         (nnmairix-widget-toggle-activate widget))
+                               nil)))
+              (list
+               (list
+                (concat "e" field)
+                (widget-create 'editable-field
+                               :size 60
+                               :format (concat " " field ":"
+                                               (make-string (- 11 (length field)) ?\ )
+                                               "%v")
+                               :value (or (cadr (assoc field values)) ""))))
+              ret))
+       (widget-insert "\n")
+       ;; Deactivate editable field
+       (widget-apply (cadr (nth 1 ret)) :deactivate))
      nnmairix-widget-fields-list)
     ret))
 
@@ -2003,7 +1984,7 @@ VALUES may contain values for editable fields from current article."
   "Add a widget NAME with optional ARGS."
   (push
    (list name
-	 (apply 'widget-create args))
+	 (apply #'widget-create args))
    nnmairix-widgets))
 
 (defun nnmairix-widget-toggle-activate (widget)
