@@ -1,16 +1,27 @@
-;;; ibuffer-loaddefs.el --- automatically extracted autoloads
+;;; ibuffer-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "ibuf-ext" "ibuf-ext.el" "3a405e78141bc4ef84c0bd1d43b584d7")
+;;;### (autoloads nil "ibuf-ext" "ibuf-ext.el" "ae2054a606832a04fbf49f41e6d6cdb0")
 ;;; Generated autoloads from ibuf-ext.el
 
 (autoload 'ibuffer-auto-mode "ibuf-ext" "\
 Toggle use of Ibuffer's auto-update facility (Ibuffer Auto mode).
-With a prefix argument ARG, enable Ibuffer Auto mode if ARG is
-positive, and disable it otherwise.  If called from Lisp, enable
-the mode if ARG is omitted or nil.
+
+This is a minor mode.  If called interactively, toggle the
+`Ibuffer-Auto mode' mode.  If the prefix argument is positive, enable
+the mode, and if it is zero or negative, disable the mode.
+
+If called from Lisp, toggle the mode if ARG is `toggle'.  Enable the
+mode if ARG is nil, omitted, or is a positive number.  Disable the
+mode if ARG is a negative number.
+
+To check whether the minor mode is enabled in the current buffer,
+evaluate `ibuffer-auto-mode'.
+
+The mode's hook is called both when the mode is enabled and when it is
+disabled.
 
 \(fn &optional ARG)" t nil)
 
@@ -30,9 +41,7 @@ Toggle the display status of the filter group chosen with the mouse.
 \(fn EVENT)" t nil)
 
 (autoload 'ibuffer-toggle-filter-group "ibuf-ext" "\
-Toggle the display status of the filter group on this line.
-
-\(fn)" t nil)
+Toggle the display status of the filter group on this line." t nil)
 
 (autoload 'ibuffer-forward-filter-group "ibuf-ext" "\
 Move point forwards by COUNT filtering groups.
@@ -60,7 +69,7 @@ Move point backwards by COUNT filtering groups.
 (autoload 'ibuffer-included-in-filters-p "ibuf-ext" "\
 Return non-nil if BUF passes all FILTERS.
 
-BUF is a lisp buffer object, and FILTERS is a list of filter
+BUF is a Lisp buffer object, and FILTERS is a list of filter
 specifications with the same structure as
 `ibuffer-filtering-qualifiers'.
 
@@ -72,14 +81,10 @@ Make the current filters into a filtering group.
 \(fn NAME)" t nil)
 
 (autoload 'ibuffer-set-filter-groups-by-mode "ibuf-ext" "\
-Set the current filter groups to filter by mode.
-
-\(fn)" t nil)
+Set the current filter groups to filter by mode." t nil)
 
 (autoload 'ibuffer-pop-filter-group "ibuf-ext" "\
-Remove the first filter group.
-
-\(fn)" t nil)
+Remove the first filter group." t nil)
 
 (autoload 'ibuffer-decompose-filter-group "ibuf-ext" "\
 Decompose the filter group GROUP into active filters.
@@ -87,9 +92,7 @@ Decompose the filter group GROUP into active filters.
 \(fn GROUP)" t nil)
 
 (autoload 'ibuffer-clear-filter-groups "ibuf-ext" "\
-Remove all filter groups.
-
-\(fn)" t nil)
+Remove all filter groups." t nil)
 
 (autoload 'ibuffer-jump-to-filter-group "ibuf-ext" "\
 Move point to the filter group whose name is NAME.
@@ -109,9 +112,7 @@ See also `ibuffer-kill-filter-group'.
 \(fn &optional ARG INTERACTIVE-P)" t nil)
 
 (autoload 'ibuffer-yank "ibuf-ext" "\
-Yank the last killed filter group before group at point.
-
-\(fn)" t nil)
+Yank the last killed filter group before group at point." t nil)
 
 (autoload 'ibuffer-yank-filter-group "ibuf-ext" "\
 Yank the last killed filter group before group named NAME.
@@ -145,28 +146,20 @@ group definitions by setting `ibuffer-filter-groups' to nil.
 \(fn &optional DELETE-FILTER-GROUPS)" t nil)
 
 (autoload 'ibuffer-pop-filter "ibuf-ext" "\
-Remove the top filter in this buffer.
-
-\(fn)" t nil)
+Remove the top filter in this buffer." t nil)
 
 (autoload 'ibuffer-decompose-filter "ibuf-ext" "\
 Separate this buffer's top compound filter (AND, OR, NOT, or SAVED).
 
 This means that the topmost filter on the filtering stack, which must
 be a complex filter like (OR [name: foo] [mode: bar-mode]), will be
-turned into separate filters, like [name: foo] and [mode: bar-mode].
-
-\(fn)" t nil)
+turned into separate filters, like [name: foo] and [mode: bar-mode]." t nil)
 
 (autoload 'ibuffer-exchange-filters "ibuf-ext" "\
-Exchange the top two filters on the stack in this buffer.
-
-\(fn)" t nil)
+Exchange the top two filters on the stack in this buffer." t nil)
 
 (autoload 'ibuffer-negate-filter "ibuf-ext" "\
-Negate the sense of the top filter in the current buffer.
-
-\(fn)" t nil)
+Negate the sense of the top filter in the current buffer." t nil)
 
 (autoload 'ibuffer-or-filter "ibuf-ext" "\
 Replace the top two filters in this buffer with their logical OR.
@@ -206,6 +199,7 @@ Set this buffer's filters to filters with NAME from `ibuffer-saved-filters'.
  (autoload 'ibuffer-filter-by-used-mode "ibuf-ext")
  (autoload 'ibuffer-filter-by-derived-mode "ibuf-ext")
  (autoload 'ibuffer-filter-by-name "ibuf-ext")
+ (autoload 'ibuffer-filter-by-process "ibuf-ext")
  (autoload 'ibuffer-filter-by-starred-name "ibuf-ext")
  (autoload 'ibuffer-filter-by-filename "ibuf-ext")
  (autoload 'ibuffer-filter-by-basename "ibuf-ext")
@@ -225,14 +219,10 @@ Default sorting modes are:
  Recency - the last time the buffer was viewed
  Name - the name of the buffer
  Major Mode - the name of the major mode of the buffer
- Size - the size of the buffer
-
-\(fn)" t nil)
+ Size - the size of the buffer" t nil)
 
 (autoload 'ibuffer-invert-sorting "ibuf-ext" "\
-Toggle whether or not sorting is in reverse order.
-
-\(fn)" t nil)
+Toggle whether or not sorting is in reverse order." t nil)
  (autoload 'ibuffer-do-sort-by-major-mode "ibuf-ext")
  (autoload 'ibuffer-do-sort-by-mode-name "ibuf-ext")
  (autoload 'ibuffer-do-sort-by-alphabetic "ibuf-ext")
@@ -240,9 +230,7 @@ Toggle whether or not sorting is in reverse order.
  (autoload 'ibuffer-do-sort-by-filename/process "ibuf-ext")
 
 (autoload 'ibuffer-bs-show "ibuf-ext" "\
-Emulate `bs-show' from the bs.el package.
-
-\(fn)" t nil)
+Emulate `bs-show' from the bs.el package." t nil)
 
 (autoload 'ibuffer-add-to-tmp-hide "ibuf-ext" "\
 Add REGEXP to `ibuffer-tmp-hide-regexps'.
@@ -278,9 +266,7 @@ to move by.  The default is `ibuffer-marked-char'.
 \(fn &optional COUNT MARK)" t nil)
 
 (autoload 'ibuffer-do-kill-lines "ibuf-ext" "\
-Hide all of the currently marked lines.
-
-\(fn)" t nil)
+Hide all of the currently marked lines." t nil)
 
 (autoload 'ibuffer-jump-to-buffer "ibuf-ext" "\
 Move point to the buffer whose name is NAME.
@@ -298,9 +284,7 @@ a prefix argument reverses the meaning of that variable.
 (autoload 'ibuffer-diff-with-file "ibuf-ext" "\
 View the differences between marked buffers and their associated files.
 If no buffers are marked, use buffer at point.
-This requires the external program \"diff\" to be in your `exec-path'.
-
-\(fn)" t nil)
+This requires the external program \"diff\" to be in your `exec-path'." t nil)
 
 (autoload 'ibuffer-copy-filename-as-kill "ibuf-ext" "\
 Copy filenames of marked (or next ARG) buffers into the kill ring.
@@ -330,9 +314,7 @@ Mark all buffers whose name matches REGEXP.
 \(fn REGEXP)" t nil)
 
 (autoload 'ibuffer-mark-by-locked "ibuf-ext" "\
-Mark all locked buffers.
-
-\(fn)" t nil)
+Mark all locked buffers." t nil)
 
 (autoload 'ibuffer-mark-by-mode-regexp "ibuf-ext" "\
 Mark all buffers whose major mode matches REGEXP.
@@ -359,49 +341,31 @@ Mark all buffers whose major mode equals MODE.
 \(fn MODE)" t nil)
 
 (autoload 'ibuffer-mark-modified-buffers "ibuf-ext" "\
-Mark all modified buffers.
-
-\(fn)" t nil)
+Mark all modified buffers." t nil)
 
 (autoload 'ibuffer-mark-unsaved-buffers "ibuf-ext" "\
-Mark all modified buffers that have an associated file.
-
-\(fn)" t nil)
+Mark all modified buffers that have an associated file." t nil)
 
 (autoload 'ibuffer-mark-dissociated-buffers "ibuf-ext" "\
-Mark all buffers whose associated file does not exist.
-
-\(fn)" t nil)
+Mark all buffers whose associated file does not exist." t nil)
 
 (autoload 'ibuffer-mark-help-buffers "ibuf-ext" "\
-Mark buffers whose major mode is in variable `ibuffer-help-buffer-modes'.
-
-\(fn)" t nil)
+Mark buffers whose major mode is in variable `ibuffer-help-buffer-modes'." t nil)
 
 (autoload 'ibuffer-mark-compressed-file-buffers "ibuf-ext" "\
-Mark buffers whose associated file is compressed.
-
-\(fn)" t nil)
+Mark buffers whose associated file is compressed." t nil)
 
 (autoload 'ibuffer-mark-old-buffers "ibuf-ext" "\
-Mark buffers which have not been viewed in `ibuffer-old-time' hours.
-
-\(fn)" t nil)
+Mark buffers which have not been viewed in `ibuffer-old-time' hours." t nil)
 
 (autoload 'ibuffer-mark-special-buffers "ibuf-ext" "\
-Mark all buffers whose name begins and ends with `*'.
-
-\(fn)" t nil)
+Mark all buffers whose name begins and ends with `*'." t nil)
 
 (autoload 'ibuffer-mark-read-only-buffers "ibuf-ext" "\
-Mark all read-only buffers.
-
-\(fn)" t nil)
+Mark all read-only buffers." t nil)
 
 (autoload 'ibuffer-mark-dired-buffers "ibuf-ext" "\
-Mark all `dired' buffers.
-
-\(fn)" t nil)
+Mark all `dired' buffers." t nil)
 
 (autoload 'ibuffer-do-occur "ibuf-ext" "\
 View lines which match REGEXP in all marked buffers.
@@ -409,6 +373,14 @@ Optional argument NLINES says how many lines of context to display: it
 defaults to one.
 
 \(fn REGEXP &optional NLINES)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "ibuf-ext" "ibuf-ext.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from ibuf-ext.el
+
+(register-definition-prefixes "ibuf-ext" '("alphabetic" "basename" "content" "derived-mode" "directory" "eval" "file" "ibuffer-" "major-mode" "mod" "name" "predicate" "print" "process" "query-replace" "rename-uniquely" "replace-regexp" "revert" "shell-command-" "size" "starred-name" "used-mode" "view-and-eval" "visiting-file"))
+
+;;;***
 
 ;;;***
 
