@@ -1,6 +1,6 @@
-;;; vietnamese.el --- support for Vietnamese -*- coding: utf-8; -*-
+;;; vietnamese.el --- support for Vietnamese -*- coding: utf-8; lexical-binding: t -*-
 
-;; Copyright (C) 1998, 2001-2019 Free Software Foundation, Inc.
+;; Copyright (C) 1998, 2001-2021 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -72,9 +72,9 @@
 (define-coding-system-alias 'viqr 'vietnamese-viqr)
 
 (set-language-info-alist
- "Vietnamese" `((charset viscii)
+ "Vietnamese" '((charset viscii)
 		(coding-system vietnamese-viscii vietnamese-vscii
-			        vietnamese-tcvn vietnamese-viqr windows-1258)
+			       vietnamese-tcvn vietnamese-viqr windows-1258)
 		(nonascii-translation . viscii)
 		(coding-priority vietnamese-viscii)
 		(input-method . "vietnamese-viqr")
