@@ -1,19 +1,25 @@
-;;; rmail-loaddefs.el --- automatically extracted autoloads
+;;; rmail-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "rmailedit" "rmailedit.el" "11082b57e892c06603495f1623a25337")
+;;;### (autoloads nil "rmailedit" "rmailedit.el" "4cfc844e39352f61cfc737a7e1009009")
 ;;; Generated autoloads from rmailedit.el
 
 (autoload 'rmail-edit-current-message "rmailedit" "\
-Edit the contents of this message.
+Edit the contents of this message." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "rmailedit" "rmailedit.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from rmailedit.el
 
-\(fn)" t nil)
+(register-definition-prefixes "rmailedit" '("rmail-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "rmailkwd" "rmailkwd.el" "f60d31eee0dffd147f7d1a6195109fd2")
+;;;### (autoloads nil "rmailkwd" "rmailkwd.el" "963d322a55fbb275586c21aeaa53ce70")
 ;;; Generated autoloads from rmailkwd.el
 
 (autoload 'rmail-add-label "rmailkwd" "\
@@ -53,10 +59,18 @@ If LABELS is empty, the last set of labels specified is used.
 With prefix argument N moves forward N messages with these labels.
 
 \(fn N LABELS)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "rmailkwd" "rmailkwd.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from rmailkwd.el
+
+(register-definition-prefixes "rmailkwd" '("rmail-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "rmailmm" "rmailmm.el" "984347a8042724d56ef855eb60444f27")
+;;;### (autoloads nil "rmailmm" "rmailmm.el" "9d2021c92f0488e106c628de19c158c3")
 ;;; Generated autoloads from rmailmm.el
 
 (autoload 'rmail-mime "rmailmm" "\
@@ -72,17 +86,25 @@ STATE forces a particular display state, rather than toggling.
 `raw' forces raw mode, any other non-nil value forces decoded mode.
 
 If `rmail-enable-mime' is nil, this creates a temporary \"*RMAIL*\"
-buffer holding a decoded copy of the message. Inline content-types are
-handled according to `rmail-mime-media-type-handlers-alist'.
+buffer holding a decoded copy of the message.  Inline content-types
+are handled according to `rmail-mime-media-type-handlers-alist'.
 By default, this displays text and multipart messages, and offers to
 download attachments as specified by `rmail-mime-attachment-dirs-alist'.
 The arguments ARG and STATE have no effect in this case.
 
 \(fn &optional ARG STATE)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "rmailmm" "rmailmm.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from rmailmm.el
+
+(register-definition-prefixes "rmailmm" '("rmail-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "rmailmsc" "rmailmsc.el" "12c473a41ca30612b6fc96a596c45829")
+;;;### (autoloads nil "rmailmsc" "rmailmsc.el" "a573b824d28825d9dd1b7ffb841a1d5f")
 ;;; Generated autoloads from rmailmsc.el
 
 (autoload 'set-rmail-inbox-list "rmailmsc" "\
@@ -96,7 +118,7 @@ This applies only to the current session.
 
 ;;;***
 
-;;;### (autoloads nil "rmailsort" "rmailsort.el" "23b2490397a9592dfad6ab30f24790f7")
+;;;### (autoloads nil "rmailsort" "rmailsort.el" "a57357393b3b564b86ea7c92bae868d1")
 ;;; Generated autoloads from rmailsort.el
 
 (autoload 'rmail-sort-by-date "rmailsort" "\
@@ -150,16 +172,22 @@ so on.  Messages that have none of these labels come last.
 If prefix argument REVERSE is non-nil, sorts in reverse order.
 
 \(fn REVERSE LABELS)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "rmailsort" "rmailsort.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from rmailsort.el
+
+(register-definition-prefixes "rmailsort" '("rmail-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "rmailsum" "rmailsum.el" "a0f64254eb04947b6055d094350a9c06")
+;;;### (autoloads nil "rmailsum" "rmailsum.el" "3b1b163b8c7ed4762799f93879ec95ca")
 ;;; Generated autoloads from rmailsum.el
 
 (autoload 'rmail-summary "rmailsum" "\
-Display a summary of all messages, one line per message.
-
-\(fn)" t nil)
+Display a summary of all messages, one line per message." t nil)
 
 (autoload 'rmail-summary-by-labels "rmailsum" "\
 Display a summary of all messages with one or more LABELS.
@@ -194,28 +222,41 @@ SUBJECT is a regular expression.
 
 (autoload 'rmail-summary-by-senders "rmailsum" "\
 Display a summary of all messages whose \"From\" field matches SENDERS.
-SENDERS is a regular expression.
+SENDERS is a regular expression.  The default for SENDERS matches the
+sender of the current message.
 
 \(fn SENDERS)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "rmailsum" "rmailsum.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from rmailsum.el
+
+(register-definition-prefixes "rmailsum" '("rmail-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "undigest" "undigest.el" "76edacd45b0d057718403004a6ed81a6")
+;;;### (autoloads nil "undigest" "undigest.el" "dc9173dc70d0870b6a458dbe92b8d5e9")
 ;;; Generated autoloads from undigest.el
 
 (autoload 'undigestify-rmail-message "undigest" "\
 Break up a digest message into its constituent messages.
-Leaves original message, deleted, before the undigestified messages.
-
-\(fn)" t nil)
+Leaves original message, deleted, before the undigestified messages." t nil)
 
 (autoload 'unforward-rmail-message "undigest" "\
 Extract a forwarded message from the containing message.
 This puts the forwarded message into a separate rmail message following
 the containing message.  This command is only useful when messages are
-forwarded with `rmail-enable-mime-composing' set to nil.
+forwarded with `rmail-enable-mime-composing' set to nil." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "undigest" "undigest.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from undigest.el
 
-\(fn)" t nil)
+(register-definition-prefixes "undigest" '("rmail-"))
+
+;;;***
 
 ;;;***
 
