@@ -1,10 +1,9 @@
-;;; mh-comp.el --- MH-E functions for composing and sending messages
+;;; mh-comp.el --- MH-E functions for composing and sending messages  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1993, 1995, 1997, 2000-2018 Free Software Foundation,
+;; Copyright (C) 1993, 1995, 1997, 2000-2021 Free Software Foundation,
 ;; Inc.
 
 ;; Author: Bill Wohler <wohler@newt.com>
-;; Maintainer: Bill Wohler <wohler@newt.com>
 ;; Keywords: mail
 ;; See: mh-e.el
 
@@ -30,8 +29,6 @@
 ;; that are used to send the mail. Other that those, functions that
 ;; are needed in mh-letter.el should be found there.
 
-;;; Change Log:
-
 ;;; Code:
 
 (require 'mh-e)
@@ -41,6 +38,7 @@
 (require 'sendmail)
 
 (autoload 'easy-menu-add "easymenu")
+(autoload 'mail-header-parse-address "mail-parse")
 (autoload 'mml-insert-tag "mml")
 
 
@@ -77,6 +75,14 @@ Default is \"components\".
 If not an absolute file name, the file is searched for first in the
 user's MH directory, then in the system MH lib directory.")
 
+(defvar mh-dist-formfile "distcomps"
+  "Name of file to be used as a skeleton for redistributing messages.
+
+Default is \"distcomps\".
+
+If not an absolute file name, the file is searched for first in the
+user's MH directory, then in the system MH lib directory.")
+
 (defvar mh-repl-formfile "replcomps"
   "Name of file to be used as a skeleton for replying to messages.
 
@@ -210,7 +216,7 @@ TO, CC, and SUBJECT arguments are used."
 (defvar mh-error-if-no-draft nil)       ;raise error over using old draft
 
 ;;;###autoload
-(defun mh-smail-batch (&optional to subject other-headers &rest ignored)
+(defun mh-smail-batch (&optional to subject _other-headers &rest _ignored)
   "Compose a message with the MH mail system.
 
 This function does not prompt the user for any header fields, and
@@ -232,10 +238,7 @@ applications should use `mh-user-agent-compose'."
   'mh-before-send-letter-hook)
 
 ;;;###autoload
-(defun mh-user-agent-compose (&optional to subject other-headers continue
-                                        switch-function yank-action
-                                        send-actions return-action
-                                        &rest ignored)
+(defun mh-user-agent-compose (&optional to subject other-headers &rest _ignored)
   "Set up mail composition draft with the MH mail system.
 This is the `mail-user-agent' entry point to MH-E. This function
 conforms to the contract specified by `define-mail-user-agent'
@@ -249,8 +252,7 @@ OTHER-HEADERS is an alist specifying additional header fields.
 Elements look like (HEADER . VALUE) where both HEADER and VALUE
 are strings.
 
-CONTINUE, SWITCH-FUNCTION, YANK-ACTION, SEND-ACTIONS, and
-RETURN-ACTION and any additional arguments are IGNORED."
+Any additional arguments are IGNORED."
   (mh-find-path)
   (let ((mh-error-if-no-draft t))
     (mh-send to "" subject)
@@ -259,9 +261,7 @@ RETURN-ACTION and any additional arguments are IGNORED."
                         (cdr (car other-headers)))
       (setq other-headers (cdr other-headers)))))
 
-;; Shush compiler.
-(mh-do-in-xemacs
-  (defvar sendmail-coding-system))
+(defvar sendmail-coding-system)
 
 ;;;###autoload
 (defun mh-send-letter (&optional arg)
@@ -304,25 +304,28 @@ message and scan line."
   (let ((draft-buffer (current-buffer))
         (file-name buffer-file-name)
         (config mh-previous-window-config)
+        ;; FIXME this is subtly different to select-message-coding-system.
         (coding-system-for-write
-         (if (and (local-variable-p 'buffer-file-coding-system
-                                    (current-buffer)) ;XEmacs needs two args
-                  ;; We're not sure why, but buffer-file-coding-system
-                  ;; tends to get set to undecided-unix.
-                  (not (memq buffer-file-coding-system
-                             '(undecided undecided-unix undecided-dos))))
-             buffer-file-coding-system
-           (or (and (boundp 'sendmail-coding-system) sendmail-coding-system)
-               (and (default-boundp 'buffer-file-coding-system)
-                    (default-value 'buffer-file-coding-system))
-               'iso-latin-1))))
+         (if (fboundp 'select-message-coding-system)
+             (select-message-coding-system) ; Emacs has this since at least 21.1
+           (if (and (local-variable-p 'buffer-file-coding-system
+                                      (current-buffer)) ;XEmacs needs two args
+                    ;; We're not sure why, but buffer-file-coding-system
+                    ;; tends to get set to undecided-unix.
+                    (not (memq buffer-file-coding-system
+                               '(undecided undecided-unix undecided-dos))))
+               buffer-file-coding-system
+             (or (and (boundp 'sendmail-coding-system) sendmail-coding-system)
+                 (and (default-boundp 'buffer-file-coding-system)
+                      (default-value 'buffer-file-coding-system))
+                 'utf-8)))))
     ;; Older versions of spost do not support -msgid and -mime.
     (unless mh-send-uses-spost-flag
       ;; Adding a Message-ID field looks good, makes it easier to search for
       ;; message in your +outbox, and best of all doesn't break threading for
       ;; the recipient if you reply to a message in your +outbox.
       (setq mh-send-args (concat "-msgid " mh-send-args))
-      ;; The default BCC encapsulation will make a MIME message unreadable.
+      ;; The default Bcc encapsulation will make a MIME message unreadable.
       ;; With nmh use the -mime arg to prevent this.
       (if (and (mh-variant-p 'nmh)
                (mh-goto-header-field "Bcc:")
@@ -411,7 +414,7 @@ See also `mh-send'."
   (interactive (list (mh-get-msg-num t)))
   (let* ((from-folder mh-current-folder)
          (config (current-window-configuration))
-         (components-file (mh-bare-components))
+         (components-file (mh-bare-components mh-comp-formfile))
          (draft
           (cond ((and mh-draft-folder (equal from-folder mh-draft-folder))
                  (pop-to-buffer (find-file-noselect (mh-msg-filename message))
@@ -431,43 +434,42 @@ See also `mh-send'."
     (mh-insert-header-separator)
     ;; Merge in components
     (mh-mapc
-     (function
-      (lambda (header-field)
-        (let ((field (car header-field))
-              (value (cdr header-field))
-              (case-fold-search t))
-          (cond
-           ;; Address field
-           ((string-match field "^To$\\|^Cc$\\|^From$")
-            (cond
-             ((not (mh-goto-header-field (concat field ":")))
-              ;; Header field does not exist, add it
-              (mh-goto-header-end 0)
-              (insert field ": " value "\n"))
-             ((string-equal value "")
-              ;; Header field already exists and no value
-              )
-             (t
-              ;; Header field exists and we have a value
-              (let (address mailbox (alias (mh-alias-expand value)))
-                (and alias
-                     (setq address (ietf-drums-parse-address alias))
-                     (setq mailbox (car address)))
-                ;; XXX - Need to parse all addresses out of field
-                (if (and
-                     (not (mh-regexp-in-field-p
-                           (concat "\\b" (regexp-quote value) "\\b") field))
-                     mailbox
-                     (not (mh-regexp-in-field-p
-                           (concat "\\b" (regexp-quote mailbox) "\\b") field)))
-                    (insert " " value ","))
-                ))))
-           ((string-match field "^Fcc$")
-            ;; Folder reference
-            (mh-modify-header-field field value))
-           ;; Text field, that's an easy case
-           (t
-            (mh-modify-header-field field value))))))
+     (lambda (header-field)
+       (let ((field (car header-field))
+             (value (cdr header-field))
+             (case-fold-search t))
+         (cond
+          ;; Address field
+          ((string-match field "^To$\\|^Cc$\\|^From$")
+           (cond
+            ((not (mh-goto-header-field (concat field ":")))
+             ;; Header field does not exist, add it
+             (mh-goto-header-end 0)
+             (insert field ": " value "\n"))
+            ((string-equal value "")
+             ;; Header field already exists and no value
+             )
+            (t
+             ;; Header field exists and we have a value
+             (let (address mailbox (alias (mh-alias-expand value)))
+               (and alias
+                    (setq address (mail-header-parse-address alias))
+                    (setq mailbox (car address)))
+               ;; XXX - Need to parse all addresses out of field
+               (if (and
+                    (not (mh-regexp-in-field-p
+                          (concat "\\b" (regexp-quote value) "\\b") field))
+                    mailbox
+                    (not (mh-regexp-in-field-p
+                          (concat "\\b" (regexp-quote mailbox) "\\b") field)))
+                   (insert " " value ","))
+               ))))
+          ((string-match field "^Fcc$")
+           ;; Folder reference
+           (mh-modify-header-field field value))
+          ;; Text field, that's an easy case
+          (t
+           (mh-modify-header-field field value)))))
      (mh-components-to-list components-file))
     (delete-file components-file)
     (goto-char (point-min))
@@ -602,7 +604,7 @@ See also `mh-compose-forward-as-mime-flag',
               (goto-char (mh-mail-header-end))
               (while
                   (re-search-forward
-                   "^#forw \\[\\([^]]+\\)\\] \\(+\\S-+\\) \\(.*\\)$"
+                   "^#forw \\[\\([^]]+\\)\\] \\(\\+\\S-+\\) \\(.*\\)$"
                    (point-max) t)
                 (let ((description (if (equal (match-string 1)
                                               "forwarded messages")
@@ -636,8 +638,10 @@ See also `mh-compose-forward-as-mime-flag',
 (defun mh-forwarded-letter-subject (from subject)
   "Return a Subject suitable for a forwarded message.
 Original message has headers FROM and SUBJECT."
-  (let ((addr-start (string-match "<" from))
-        (comment (string-match "(" from)))
+  ;; Join continued lines.
+  (setq from (replace-regexp-in-string "\\s *\n\\s +" " " from))
+  (let ((addr-start (string-search "<" from))
+        (comment (string-search "(" from)))
     (cond ((and addr-start (> addr-start 0))
            ;; Full Name <luser@host>
            (setq from (substring from 0 (1- addr-start))))
@@ -647,15 +651,16 @@ Original message has headers FROM and SUBJECT."
   (format mh-forward-subject-format from subject))
 
 ;;;###mh-autoload
-(defun mh-redistribute (to cc &optional message)
+(defun mh-redistribute (to cc identity &optional message)
   "Redistribute a message.
 
 This command is similar in function to forwarding mail, but it
 does not allow you to edit the message, nor does it add your name
 to the \"From\" header field. It appears to the recipient as if
 the message had come from the original sender. When you run this
-command, you are prompted for the TO and CC recipients. The
-default MESSAGE is the current message.
+command, you are prompted for the TO and CC recipients. You are
+also prompted for the sending IDENTITY to use. The default
+MESSAGE is the current message.
 
 Also investigate the command \\[mh-edit-again] for another way to
 redistribute messages.
@@ -666,6 +671,9 @@ The hook `mh-annotate-msg-hook' is run after annotating the
 message and scan line."
   (interactive (list (mh-read-address "Redist-To: ")
                      (mh-read-address "Redist-Cc: ")
+                     (if mh-identity-list
+                         (mh-select-identity mh-identity-default)
+                       nil)
                      (mh-get-msg-num t)))
   (or message
       (setq message (mh-get-msg-num t)))
@@ -675,14 +683,52 @@ message and scan line."
                                 (if mh-redist-full-contents-flag
                                     (mh-msg-filename message)
                                   nil)
-                                nil)))
-      (mh-goto-header-end 0)
-      (insert "Resent-To: " to "\n")
-      (if (not (equal cc "")) (insert "Resent-cc: " cc "\n"))
-      (mh-clean-msg-header
-       (point-min)
-       "^Message-Id:\\|^Received:\\|^Return-Path:\\|^Sender:\\|^Date:\\|^From:"
-       nil)
+                                nil))
+          (from (mh-identity-field identity "From"))
+          (fcc  (mh-identity-field identity "Fcc"))
+          (bcc  (mh-identity-field identity "Bcc"))
+          comp-fcc comp-to comp-cc comp-bcc)
+      (if mh-redist-full-contents-flag
+          (mh-clean-msg-header
+           (point-min)
+           "^Message-Id:\\|^Received:\\|^Return-Path:\\|^Date:\\|^Resent-.*:"
+           nil))
+      ;; Read fields from the distcomps file and put them in our
+      ;; draft. For "To", "Cc", "Bcc", and "Fcc", multiple headers are
+      ;; combined into a single header with comma-separated entries.
+      ;; For "From", the first value wins, with the identity's "From"
+      ;; trumping anything in the distcomps file.
+      (let ((components-file (mh-bare-components mh-dist-formfile)))
+        (mh-mapc
+         (lambda (header-field)
+           (let ((field (car header-field))
+                 (value (cdr header-field))
+                 (case-fold-search t))
+             (cond
+              ((string-match field "^Resent-Fcc$")
+               (setq comp-fcc value))
+              ((string-match field "^Resent-From$")
+               (or from
+                   (setq from value)))
+              ((string-match field "^Resent-To$")
+               (setq comp-to value))
+              ((string-match field "^Resent-Cc$")
+               (setq comp-cc value))
+              ((string-match field "^Resent-Bcc$")
+               (setq comp-bcc value))
+              ((string-match field "^Resent-.*$")
+               (mh-insert-fields field value)))))
+         (mh-components-to-list components-file))
+        (delete-file components-file))
+      (mh-insert-fields "Resent-To:" (mapconcat #'identity (list to comp-to)
+                                                ", ")
+                        "Resent-Cc:" (mapconcat #'identity (list cc comp-cc)
+                                                ", ")
+                        "Resent-Fcc:" (mapconcat #'identity (list fcc comp-fcc)
+                                                 ", ")
+                        "Resent-Bcc:" (mapconcat #'identity (list bcc comp-bcc)
+                                                 ", ")
+                        "Resent-From:" from)
       (save-buffer)
       (message "Redistributing...")
       (let ((env "mhdist=1"))
@@ -700,7 +746,8 @@ message and scan line."
         ;; Annotate...
         (mh-annotate-msg message folder mh-note-dist
                          "-component" "Resent:"
-                         "-text" (format "\"%s %s\"" to cc)))
+                         "-text" (format "\"To: %s Cc: %s From: %s\""
+                                         to cc from)))
       (kill-buffer draft)
       (message "Redistributing...done"))))
 
@@ -896,7 +943,7 @@ CONFIG is the window configuration before sending mail."
     (message "Composing a message...")
     (let ((draft (mh-read-draft
                   "message"
-                  (mh-bare-components)
+                  (mh-bare-components mh-comp-formfile)
                   t)))
       (mh-insert-fields "To:" to "Subject:" subject "Cc:" cc)
       (goto-char (point-max))
@@ -906,23 +953,25 @@ CONFIG is the window configuration before sending mail."
       (mh-letter-mode-message)
       (mh-letter-adjust-point))))
 
-(defun mh-bare-components ()
-  "Generate a temporary, clean components file and return its path."
-  ;; Let comp(1) create the skeleton for us.  This is particularly
+(defun mh-bare-components (formfile)
+  "Generate a temporary, clean components file from FORMFILE.
+Return the path to the temporary file."
+  ;; Let comp(1) create the skeleton for us. This is particularly
   ;; important with nmh-1.5, because its default "components" needs
-  ;; some processing before it can be used.  Unfortunately, comp(1)
-  ;; doesn't have a -build option.  So, to avoid the possibility of
-  ;; clobbering an existing draft, create a temporary directory and
-  ;; use it as the drafts folder.  Then copy the skeleton to a regular
-  ;; temp file, and return the regular temp file.
+  ;; some processing before it can be used. Unfortunately, comp(1)
+  ;; didn't have a -build option until later versions of nmh. So, to
+  ;; avoid the possibility of clobbering an existing draft, create
+  ;; a temporary directory and use it as the drafts folder. Then
+  ;; copy the skeleton to a regular temp file, and return the
+  ;; regular temp file.
   (let (new
         (temp-folder (make-temp-file
                       (concat mh-user-path "draftfolder.") t)))
     (mh-exec-cmd "comp" "-nowhatnowproc"
                  "-draftfolder" (format "+%s"
                                         (file-name-nondirectory temp-folder))
-                 (if (stringp mh-comp-formfile)
-                     (list "-form" mh-comp-formfile)))
+                 (if (stringp formfile)
+                     (list "-form" formfile)))
     (setq new (make-temp-file "comp."))
     (rename-file (concat temp-folder "/" "1") new t)
     ;; The temp folder could contain various metadata files.  Rather
@@ -1050,12 +1099,13 @@ letter."
   (setq mode-line-buffer-identification (list "    {%b}"))
   (mh-logo-display)
   (mh-make-local-hook 'kill-buffer-hook)
-  (add-hook 'kill-buffer-hook 'mh-tidy-draft-buffer nil t)
+  (add-hook 'kill-buffer-hook #'mh-tidy-draft-buffer nil t)
   (run-hook-with-args 'mh-compose-letter-function to subject cc))
 
 (defun mh-insert-x-mailer ()
   "Append an X-Mailer field to the header.
 The versions of MH-E, Emacs, and MH are shown."
+  (or mh-variant-in-use (mh-variant-set mh-variant))
   ;; Lazily initialize mh-x-mailer-string.
   (when (and mh-insert-x-mailer-flag (null mh-x-mailer-string))
     (setq mh-x-mailer-string
@@ -1118,7 +1168,7 @@ This should be the last function called when composing the draft."
 MSG can be a message number, a list of message numbers, or a sequence.
 The hook `mh-annotate-msg-hook' is run after annotating; see its
 documentation for variables it can use."
-  (apply 'mh-exec-cmd "anno" folder
+  (apply #'mh-exec-cmd "anno" folder
          (if (listp msg) (append msg args) (cons msg args)))
   (save-excursion
     (cond ((get-buffer folder)          ; Buffer may be deleted
@@ -1243,10 +1293,10 @@ discarded."
   "Check if current buffer is entirely composed of ASCII.
 The function doesn't work for XEmacs since `find-charset-region'
 doesn't exist there."
-  (loop for charset in (mh-funcall-if-exists
-                        find-charset-region (point-min) (point-max))
-        unless (eq charset 'ascii) return nil
-        finally return t))
+  (cl-loop for charset in (mh-funcall-if-exists
+                           find-charset-region (point-min) (point-max))
+           unless (eq charset 'ascii) return nil
+           finally return t))
 
 (provide 'mh-comp)
 
