@@ -1,6 +1,6 @@
-;;; mh-print.el --- MH-E printing support
+;;; mh-print.el --- MH-E printing support  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2003-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2003-2021 Free Software Foundation, Inc.
 
 ;; Author: Jeffrey C Honig <jch@honig.net>
 ;; Maintainer: Bill Wohler <wohler@newt.com>
@@ -24,8 +24,6 @@
 
 ;;; Commentary:
 
-;;; Change Log:
-
 ;;; Code:
 
 (require 'mh-e)
@@ -207,8 +205,9 @@ Consider using \\[mh-ps-print-msg] instead."
     ;; Print scan listing if we have more than one message.
     (if (> (length msgs) 1)
         (let* ((msgs-string
-                (mapconcat 'identity (mh-list-to-string
-                                      (mh-coalesce-msg-list msgs)) " "))
+                (mapconcat #'identity (mh-list-to-string
+                                       (mh-coalesce-msg-list msgs))
+                           " "))
                (lpr-command
                 (format mh-lpr-command-format
                         (cond ((listp range)
