@@ -1,9 +1,8 @@
-;;; quickurl.el --- insert a URL based on text at point in buffer
+;;; quickurl.el --- insert a URL based on text at point in buffer  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1999-2018 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
 ;; Author: Dave Pearson <davep@davep.org>
-;; Maintainer: Dave Pearson <davep@davep.org>
 ;; Created: 1999-05-28
 ;; Keywords: hypermedia
 
@@ -25,19 +24,19 @@
 ;;; Commentary:
 ;;
 ;; This package provides a simple method of inserting a URL based on the
-;; text at point in the current buffer. This is part of an on-going effort
+;; text at point in the current buffer.  This is part of an on-going effort
 ;; to increase the information I provide people while reducing the amount
-;; of typing I need to do. No-doubt there are undiscovered Emacs packages
+;; of typing I need to do.  No-doubt there are undiscovered Emacs packages
 ;; out there that do all of this and do it better, feel free to point me to
 ;; them, in the mean time I'm having fun playing with Emacs Lisp.
 ;;
 ;; The URLs are stored in an external file as a list of either cons cells,
-;; or lists. A cons cell entry looks like this:
+;; or lists.  A cons cell entry looks like this:
 ;;
 ;;    (<Lookup> . <URL>)
 ;;
 ;; where <Lookup> is a string that acts as the keyword lookup and <URL> is
-;; the URL associated with it. An example might be:
+;; the URL associated with it.  An example might be:
 ;;
 ;;    ("GNU" . "https://www.gnu.org/")
 ;;
@@ -46,8 +45,8 @@
 ;;    (<Lookup> <URL> <Comment>)
 ;;
 ;; where <Lookup> and <URL> are the same as with the cons cell and <Comment>
-;; is any text you like that describes the URL. This description will be
-;; used when presenting a list of URLS using `quickurl-list'. An example
+;; is any text you like that describes the URL.  This description will be
+;; used when presenting a list of URLS using `quickurl-list'.  An example
 ;; might be:
 ;;
 ;;    ("FSF" "https://www.fsf.org/" "The Free Software Foundation")
@@ -98,23 +97,19 @@
   (locate-user-emacs-file "quickurls" ".quickurls")
   "File that contains the URL list."
   :version "24.4"                       ; added locate-user-emacs-file
-  :type  'file
-  :group 'quickurl)
+  :type  'file)
 
 (defcustom quickurl-format-function #'quickurl-format-url
   "Function to format the URL before insertion into the current buffer."
-  :type  'function
-  :group 'quickurl)
+  :type  'function)
 
 (defcustom quickurl-sort-function #'quickurl-sort-urls
   "Function to sort the URL list."
-  :type  'function
-  :group 'quickurl)
+  :type  'function)
 
 (defcustom quickurl-grab-lookup-function #'current-word
   "Function to grab the thing to lookup."
-  :type  'function
-  :group 'quickurl)
+  :type  'function)
 
 (defun quickurl--assoc-function (key alist)
   "Default function for `quickurl-assoc-function'."
@@ -123,31 +118,26 @@
 (defcustom quickurl-assoc-function #'quickurl--assoc-function
   "Function to use for alist lookup into `quickurl-urls'."
   :version "26.1"                 ; was the obsolete assoc-ignore-case
-  :type  'function
-  :group 'quickurl)
+  :type  'function)
 
 (defcustom quickurl-completion-ignore-case t
   "Should `quickurl-ask' ignore case when doing the input lookup?"
-  :type  'boolean
-  :group 'quickurl)
+  :type  'boolean)
 
 (defcustom quickurl-prefix ";; -*- lisp -*-\n\n"
   "Text to write to `quickurl-url-file' before writing the URL list."
-  :type  'string
-  :group 'quickurl)
+  :type  'string)
 
 (defcustom quickurl-postfix ""
   "Text to write to `quickurl-url-file' after writing the URL list.
 
 See the constant `quickurl-reread-hook-postfix' for some example text that
 could be used here."
-  :type  'string
-  :group 'quickurl)
+  :type  'string)
 
 (defcustom quickurl-list-mode-hook nil
   "Hooks for `quickurl-list-mode'."
-  :type  'hook
-  :group 'quickurl)
+  :type  'hook)
 
 ;; Constants.
 
@@ -155,7 +145,7 @@ could be used here."
 (defconst quickurl-reread-hook-postfix
     "
 ;; Local Variables:
-;; eval: (progn (require 'quickurl) (add-hook 'local-write-file-hooks (lambda () (quickurl-read) nil)))
+;; eval: (progn (require 'quickurl) (add-hook 'write-file-functions (lambda () (quickurl-read) nil) nil t))
 ;; End:
 "
   "Example `quickurl-postfix' text that adds a local variable to the
@@ -225,8 +215,8 @@ Note that this function is a setfable place."
 (defun quickurl-url-comment (url)
   "Get the comment from a URL.
 
-If the URL has no comment an empty string is returned. Also note that this
-function is a setfable place."
+If the URL has no comment an empty string is returned.  Also note
+that this function is a setfable place."
   (declare
    (gv-setter (lambda (store)
                 `(if (quickurl-url-commented-p ,url)
@@ -294,7 +284,7 @@ It also restores point after the `read'."
   "Return URL associated with key LOOKUP.
 
 The lookup is done by looking in the alist `quickurl-urls' and the `cons'
-for the URL is returned. The actual method used to look into the alist
+for the URL is returned.  The actual method used to look into the alist
 depends on the setting of the variable `quickurl-assoc-function'."
   (funcall quickurl-assoc-function lookup quickurl-urls))
 
@@ -504,15 +494,15 @@ TYPE dictates what will be inserted, options are:
         (with-current-buffer quickurl-list-last-buffer
           (insert
            (pcase type
-             (`url         (funcall quickurl-format-function url))
-             (`naked-url   (quickurl-url-url url))
-             (`with-lookup (format "%s <URL:%s>"
+             ('url         (funcall quickurl-format-function url))
+             ('naked-url   (quickurl-url-url url))
+             ('with-lookup (format "%s <URL:%s>"
                                    (quickurl-url-keyword url)
                                    (quickurl-url-url url)))
-             (`with-desc   (format "%S <URL:%s>"
+             ('with-desc   (format "%S <URL:%s>"
                                    (quickurl-url-description url)
                                    (quickurl-url-url url)))
-             (`lookup      (quickurl-url-keyword url)))))
+             ('lookup      (quickurl-url-keyword url)))))
       (error "No URL details on that line"))
     url))
 
