@@ -1,6 +1,6 @@
-;;; pgg-gpg.el --- GnuPG support for PGG.
+;;; pgg-gpg.el --- GnuPG support for PGG.  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1999-2000, 2002-2019 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2000, 2002-2021 Free Software Foundation, Inc.
 
 ;; Author: Daiki Ueno <ueno@unixuser.org>
 ;; Symmetric encryption and gpg-agent support added by:
@@ -27,8 +27,7 @@
 
 ;;; Code:
 
-(eval-when-compile
-  (require 'cl))
+(eval-when-compile (require 'cl-lib))
 
 (require 'pgg)
 
@@ -38,23 +37,19 @@
 
 (defcustom pgg-gpg-program "gpg"
   "The GnuPG executable."
-  :group 'pgg-gpg
   :type 'string)
 
 (defcustom pgg-gpg-extra-args nil
   "Extra arguments for every GnuPG invocation."
-  :group 'pgg-gpg
   :type '(repeat (string :tag "Argument")))
 
 (defcustom pgg-gpg-recipient-argument "--recipient"
   "GnuPG option to specify recipient."
-  :group 'pgg-gpg
   :type '(choice (const :tag "New `--recipient' option" "--recipient")
 		 (const :tag "Old `--remote-user' option" "--remote-user")))
 
 (defcustom pgg-gpg-use-agent t
   "Whether to use gnupg agent for key caching."
-  :group 'pgg-gpg
   :type 'boolean)
 
 (defvar pgg-gpg-user-id nil
@@ -98,7 +93,7 @@
 			 passphrase-with-newline
 			 (coding-system-change-eol-conversion
 			  pgg-passphrase-coding-system 'unix)))
-		  (pgg-clear-string passphrase-with-newline))
+		  (clear-string passphrase-with-newline))
 	      (setq encoded-passphrase-with-new-line passphrase-with-newline
 		    passphrase-with-newline nil))
 	    (process-send-string process encoded-passphrase-with-new-line))
@@ -126,9 +121,9 @@
 	    (if (= 127 exit-status)
 		(error "%s could not be found" program))))
       (if passphrase-with-newline
-	  (pgg-clear-string passphrase-with-newline))
+	  (clear-string passphrase-with-newline))
       (if encoded-passphrase-with-new-line
-	  (pgg-clear-string encoded-passphrase-with-new-line))
+	  (clear-string encoded-passphrase-with-new-line))
       (if (and process (eq 'run (process-status process)))
 	  (interrupt-process process))
       (if (file-exists-p output-file-name)
@@ -303,7 +298,7 @@ passphrase cache or user."
 
 (defun pgg-gpg-select-matching-key (message-keys secret-keys)
   "Choose a key from MESSAGE-KEYS that matches one of the keys in SECRET-KEYS."
-  (loop for message-key in message-keys
+  (cl-loop for message-key in message-keys
 	for message-key-id = (and (equal (car message-key) 1)
 				  (cdr (assq 'key-identifier
 					     (cdr message-key))))
