@@ -1,6 +1,6 @@
 ;;; ws-mode.el --- WordStar emulation mode for GNU Emacs -*- lexical-binding: t -*-
 
-;; Copyright (C) 1991, 2001-2020 Free Software Foundation, Inc.
+;; Copyright (C) 1991, 2001-2021 Free Software Foundation, Inc.
 
 ;; Author: Juergen Nickelsen <nickel@cs.tu-berlin.de>
 ;; Version: 0.7
@@ -41,144 +41,144 @@
 (defvar wordstar-C-k-map
   (let ((map (make-keymap)))
     (define-key map " " ())
-    (define-key map "0" 'ws-set-marker-0)
-    (define-key map "1" 'ws-set-marker-1)
-    (define-key map "2" 'ws-set-marker-2)
-    (define-key map "3" 'ws-set-marker-3)
-    (define-key map "4" 'ws-set-marker-4)
-    (define-key map "5" 'ws-set-marker-5)
-    (define-key map "6" 'ws-set-marker-6)
-    (define-key map "7" 'ws-set-marker-7)
-    (define-key map "8" 'ws-set-marker-8)
-    (define-key map "9" 'ws-set-marker-9)
-    (define-key map "b" 'ws-begin-block)
-    (define-key map "\C-b" 'ws-begin-block)
-    (define-key map "c" 'ws-copy-block)
-    (define-key map "\C-c" 'ws-copy-block)
-    (define-key map "d" 'save-buffers-kill-emacs)
-    (define-key map "\C-d" 'save-buffers-kill-emacs)
-    (define-key map "f" 'find-file)
-    (define-key map "\C-f" 'find-file)
-    (define-key map "h" 'ws-show-markers)
-    (define-key map "\C-h" 'ws-show-markers)
-    (define-key map "i" 'ws-indent-block)
-    (define-key map "\C-i" 'ws-indent-block)
-    (define-key map "k" 'ws-end-block)
-    (define-key map "\C-k" 'ws-end-block)
-    (define-key map "p" 'ws-print-block)
-    (define-key map "\C-p" 'ws-print-block)
-    (define-key map "q" 'kill-emacs)
-    (define-key map "\C-q" 'kill-emacs)
-    (define-key map "r" 'insert-file)
-    (define-key map "\C-r" 'insert-file)
-    (define-key map "s" 'save-some-buffers)
-    (define-key map "\C-s" 'save-some-buffers)
-    (define-key map "t" 'ws-mark-word)
-    (define-key map "\C-t" 'ws-mark-word)
-    (define-key map "u" 'ws-exdent-block)
-    (define-key map "\C-u" 'keyboard-quit)
-    (define-key map "v" 'ws-move-block)
-    (define-key map "\C-v" 'ws-move-block)
-    (define-key map "w" 'ws-write-block)
-    (define-key map "\C-w" 'ws-write-block)
-    (define-key map "x" 'save-buffers-kill-emacs)
-    (define-key map "\C-x" 'save-buffers-kill-emacs)
-    (define-key map "y" 'ws-delete-block)
-    (define-key map "\C-y" 'ws-delete-block)
+    (define-key map "0" #'ws-set-marker-0)
+    (define-key map "1" #'ws-set-marker-1)
+    (define-key map "2" #'ws-set-marker-2)
+    (define-key map "3" #'ws-set-marker-3)
+    (define-key map "4" #'ws-set-marker-4)
+    (define-key map "5" #'ws-set-marker-5)
+    (define-key map "6" #'ws-set-marker-6)
+    (define-key map "7" #'ws-set-marker-7)
+    (define-key map "8" #'ws-set-marker-8)
+    (define-key map "9" #'ws-set-marker-9)
+    (define-key map "b" #'ws-begin-block)
+    (define-key map "\C-b" #'ws-begin-block)
+    (define-key map "c" #'ws-copy-block)
+    (define-key map "\C-c" #'ws-copy-block)
+    (define-key map "d" #'save-buffers-kill-emacs)
+    (define-key map "\C-d" #'save-buffers-kill-emacs)
+    (define-key map "f" #'find-file)
+    (define-key map "\C-f" #'find-file)
+    (define-key map "h" #'ws-show-markers)
+    (define-key map "\C-h" #'ws-show-markers)
+    (define-key map "i" #'ws-indent-block)
+    (define-key map "\C-i" #'ws-indent-block)
+    (define-key map "k" #'ws-end-block)
+    (define-key map "\C-k" #'ws-end-block)
+    (define-key map "p" #'ws-print-block)
+    (define-key map "\C-p" #'ws-print-block)
+    (define-key map "q" #'kill-emacs)
+    (define-key map "\C-q" #'kill-emacs)
+    (define-key map "r" #'insert-file)
+    (define-key map "\C-r" #'insert-file)
+    (define-key map "s" #'save-some-buffers)
+    (define-key map "\C-s" #'save-some-buffers)
+    (define-key map "t" #'ws-mark-word)
+    (define-key map "\C-t" #'ws-mark-word)
+    (define-key map "u" #'ws-exdent-block)
+    (define-key map "\C-u" #'keyboard-quit)
+    (define-key map "v" #'ws-move-block)
+    (define-key map "\C-v" #'ws-move-block)
+    (define-key map "w" #'ws-write-block)
+    (define-key map "\C-w" #'ws-write-block)
+    (define-key map "x" #'save-buffers-kill-emacs)
+    (define-key map "\C-x" #'save-buffers-kill-emacs)
+    (define-key map "y" #'ws-delete-block)
+    (define-key map "\C-y" #'ws-delete-block)
     map))
 
 (defvar wordstar-C-o-map
   (let ((map (make-keymap)))
     (define-key map " " ())
-    (define-key map "c" 'wordstar-center-line)
-    (define-key map "\C-c" 'wordstar-center-line)
-    (define-key map "b" 'switch-to-buffer)
-    (define-key map "\C-b" 'switch-to-buffer)
-    (define-key map "j" 'justify-current-line)
-    (define-key map "\C-j" 'justify-current-line)
-    (define-key map "k" 'kill-buffer)
-    (define-key map "\C-k" 'kill-buffer)
-    (define-key map "l" 'list-buffers)
-    (define-key map "\C-l" 'list-buffers)
-    (define-key map "m" 'auto-fill-mode)
-    (define-key map "\C-m" 'auto-fill-mode)
-    (define-key map "r" 'set-fill-column)
-    (define-key map "\C-r" 'set-fill-column)
-    (define-key map "\C-u" 'keyboard-quit)
-    (define-key map "wd" 'delete-other-windows)
-    (define-key map "wh" 'split-window-right)
-    (define-key map "wo" 'other-window)
-    (define-key map "wv" 'split-window-below)
+    (define-key map "c" #'wordstar-center-line)
+    (define-key map "\C-c" #'wordstar-center-line)
+    (define-key map "b" #'switch-to-buffer)
+    (define-key map "\C-b" #'switch-to-buffer)
+    (define-key map "j" #'justify-current-line)
+    (define-key map "\C-j" #'justify-current-line)
+    (define-key map "k" #'kill-buffer)
+    (define-key map "\C-k" #'kill-buffer)
+    (define-key map "l" #'list-buffers)
+    (define-key map "\C-l" #'list-buffers)
+    (define-key map "m" #'auto-fill-mode)
+    (define-key map "\C-m" #'auto-fill-mode)
+    (define-key map "r" #'set-fill-column)
+    (define-key map "\C-r" #'set-fill-column)
+    (define-key map "\C-u" #'keyboard-quit)
+    (define-key map "wd" #'delete-other-windows)
+    (define-key map "wh" #'split-window-right)
+    (define-key map "wo" #'other-window)
+    (define-key map "wv" #'split-window-below)
     map))
 
 (defvar wordstar-C-q-map
   (let ((map (make-keymap)))
     (define-key map " " ())
-    (define-key map "0" 'ws-find-marker-0)
-    (define-key map "1" 'ws-find-marker-1)
-    (define-key map "2" 'ws-find-marker-2)
-    (define-key map "3" 'ws-find-marker-3)
-    (define-key map "4" 'ws-find-marker-4)
-    (define-key map "5" 'ws-find-marker-5)
-    (define-key map "6" 'ws-find-marker-6)
-    (define-key map "7" 'ws-find-marker-7)
-    (define-key map "8" 'ws-find-marker-8)
-    (define-key map "9" 'ws-find-marker-9)
-    (define-key map "a" 'ws-query-replace)
-    (define-key map "\C-a" 'ws-query-replace)
-    (define-key map "b" 'ws-goto-block-begin)
-    (define-key map "\C-b" 'ws-goto-block-begin)
-    (define-key map "c" 'end-of-buffer)
-    (define-key map "\C-c" 'end-of-buffer)
-    (define-key map "d" 'end-of-line)
-    (define-key map "\C-d" 'end-of-line)
-    (define-key map "f" 'ws-search)
-    (define-key map "\C-f" 'ws-search)
-    (define-key map "k" 'ws-goto-block-end)
-    (define-key map "\C-k" 'ws-goto-block-end)
-    (define-key map "l" 'ws-undo)
-    (define-key map "\C-l" 'ws-undo)
-    (define-key map "p" 'ws-last-cursorp)
-    (define-key map "\C-p" 'ws-last-cursorp)
-    (define-key map "r" 'beginning-of-buffer)
-    (define-key map "\C-r" 'beginning-of-buffer)
-    (define-key map "s" 'beginning-of-line)
-    (define-key map "\C-s" 'beginning-of-line)
-    (define-key map "\C-u" 'keyboard-quit)
-    (define-key map "w" 'ws-last-error)
-    (define-key map "\C-w" 'ws-last-error)
-    (define-key map "y" 'ws-kill-eol)
-    (define-key map "\C-y" 'ws-kill-eol)
-    (define-key map "\177" 'ws-kill-bol)
+    (define-key map "0" #'ws-find-marker-0)
+    (define-key map "1" #'ws-find-marker-1)
+    (define-key map "2" #'ws-find-marker-2)
+    (define-key map "3" #'ws-find-marker-3)
+    (define-key map "4" #'ws-find-marker-4)
+    (define-key map "5" #'ws-find-marker-5)
+    (define-key map "6" #'ws-find-marker-6)
+    (define-key map "7" #'ws-find-marker-7)
+    (define-key map "8" #'ws-find-marker-8)
+    (define-key map "9" #'ws-find-marker-9)
+    (define-key map "a" #'ws-query-replace)
+    (define-key map "\C-a" #'ws-query-replace)
+    (define-key map "b" #'ws-goto-block-begin)
+    (define-key map "\C-b" #'ws-goto-block-begin)
+    (define-key map "c" #'end-of-buffer)
+    (define-key map "\C-c" #'end-of-buffer)
+    (define-key map "d" #'end-of-line)
+    (define-key map "\C-d" #'end-of-line)
+    (define-key map "f" #'ws-search)
+    (define-key map "\C-f" #'ws-search)
+    (define-key map "k" #'ws-goto-block-end)
+    (define-key map "\C-k" #'ws-goto-block-end)
+    (define-key map "l" #'ws-undo)
+    (define-key map "\C-l" #'ws-undo)
+    ;; (define-key map "p" #'ws-last-cursorp)
+    ;; (define-key map "\C-p" #'ws-last-cursorp)
+    (define-key map "r" #'beginning-of-buffer)
+    (define-key map "\C-r" #'beginning-of-buffer)
+    (define-key map "s" #'beginning-of-line)
+    (define-key map "\C-s" #'beginning-of-line)
+    (define-key map "\C-u" #'keyboard-quit)
+    (define-key map "w" #'ws-last-error)
+    (define-key map "\C-w" #'ws-last-error)
+    (define-key map "y" #'ws-kill-eol)
+    (define-key map "\C-y" #'ws-kill-eol)
+    (define-key map "\177" #'ws-kill-bol)
     map))
 
 (defvar wordstar-mode-map
   (let ((map (make-keymap)))
-    (define-key map "\C-a" 'backward-word)
-    (define-key map "\C-b" 'fill-paragraph)
-    (define-key map "\C-c" 'scroll-up-command)
-    (define-key map "\C-d" 'forward-char)
-    (define-key map "\C-e" 'previous-line)
-    (define-key map "\C-f" 'forward-word)
-    (define-key map "\C-g" 'delete-char)
-    (define-key map "\C-h" 'backward-char)
-    (define-key map "\C-i" 'indent-for-tab-command)
-    (define-key map "\C-j" 'help-for-help)
+    (define-key map "\C-a" #'backward-word)
+    (define-key map "\C-b" #'fill-paragraph)
+    (define-key map "\C-c" #'scroll-up-command)
+    (define-key map "\C-d" #'forward-char)
+    (define-key map "\C-e" #'previous-line)
+    (define-key map "\C-f" #'forward-word)
+    (define-key map "\C-g" #'delete-char)
+    (define-key map "\C-h" #'backward-char)
+    (define-key map "\C-i" #'indent-for-tab-command)
+    (define-key map "\C-j" #'help-for-help)
     (define-key map "\C-k" wordstar-C-k-map)
-    (define-key map "\C-l" 'ws-repeat-search)
-    (define-key map "\C-n" 'open-line)
+    (define-key map "\C-l" #'ws-repeat-search)
+    (define-key map "\C-n" #'open-line)
     (define-key map "\C-o" wordstar-C-o-map)
-    (define-key map "\C-p" 'quoted-insert)
+    (define-key map "\C-p" #'quoted-insert)
     (define-key map "\C-q" wordstar-C-q-map)
-    (define-key map "\C-r" 'scroll-down-command)
-    (define-key map "\C-s" 'backward-char)
-    (define-key map "\C-t" 'kill-word)
-    (define-key map "\C-u" 'keyboard-quit)
-    (define-key map "\C-v" 'overwrite-mode)
-    (define-key map "\C-w" 'scroll-down-line)
-    (define-key map "\C-x" 'next-line)
-    (define-key map "\C-y" 'kill-complete-line)
-    (define-key map "\C-z" 'scroll-up-line)
+    (define-key map "\C-r" #'scroll-down-command)
+    (define-key map "\C-s" #'backward-char)
+    (define-key map "\C-t" #'kill-word)
+    (define-key map "\C-u" #'keyboard-quit)
+    (define-key map "\C-v" #'overwrite-mode)
+    (define-key map "\C-w" #'scroll-down-line)
+    (define-key map "\C-x" #'next-line)
+    (define-key map "\C-y" #'kill-complete-line)
+    (define-key map "\C-z" #'scroll-up-line)
     map))
 
 ;; wordstar-C-j-map not yet implemented
