@@ -1,9 +1,9 @@
-;;; ps-print-loaddefs.el --- automatically extracted autoloads
+;;; ps-print-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "ps-mule" "ps-mule.el" "eaeb18a25ffba31b2495851ca093b7a4")
+;;;### (autoloads nil "ps-mule" "ps-mule.el" "8a30a78d7b8f4d1bed34f87a21628f6e")
 ;;; Generated autoloads from ps-mule.el
 
 (defvar ps-multibyte-buffer nil "\
@@ -52,9 +52,7 @@ Any other value is treated as nil.")
 (custom-autoload 'ps-multibyte-buffer "ps-mule" t)
 
 (autoload 'ps-mule-initialize "ps-mule" "\
-Initialize global data for printing multi-byte characters.
-
-\(fn)" nil nil)
+Initialize global data for printing multi-byte characters." nil nil)
 
 (autoload 'ps-mule-begin-job "ps-mule" "\
 Start printing job for multi-byte chars between FROM and TO.
@@ -63,9 +61,15 @@ It checks if all multi-byte characters in the region are printable or not.
 \(fn FROM TO)" nil nil)
 
 (autoload 'ps-mule-end-job "ps-mule" "\
-Finish printing job for multi-byte chars.
+Finish printing job for multi-byte chars." nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "ps-mule" "ps-mule.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from ps-mule.el
 
-\(fn)" nil nil)
+(register-definition-prefixes "ps-mule" '("ps-mule-"))
+
+;;;***
 
 ;;;***
 
