@@ -1,6 +1,6 @@
 ;;; css-mode.el --- Major mode to edit CSS files  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2006-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2006-2021 Free Software Foundation, Inc.
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 ;; Maintainer: Simen Heggestøyl <simenheg@gmail.com>
@@ -57,7 +57,7 @@
   "Identifiers for pseudo-classes.")
 
 (defconst css-pseudo-element-ids
-  '("after" "before" "first-letter" "first-line")
+  '("after" "before" "first-letter" "first-line" "selection")
   "Identifiers for pseudo-elements.")
 
 (defconst css-at-ids
@@ -67,13 +67,12 @@
 
 (defconst scss-at-ids
   '("at-root" "content" "debug" "each" "else" "else if" "error" "extend"
-    "for" "function" "if" "import" "include" "mixin" "return" "warn"
+    "for" "function" "if" "import" "include" "mixin" "return" "use" "warn"
     "while")
   "Additional identifiers that appear in the form @foo in SCSS.")
 
-(defvar css--at-ids css-at-ids
+(defvar-local css--at-ids css-at-ids
   "List of at-rules for the current mode.")
-(make-variable-buffer-local 'css--at-ids)
 
 (defconst css-bang-ids
   '("important")
@@ -83,9 +82,8 @@
   '("default" "global" "optional")
   "Additional identifiers that appear in the form !foo in SCSS.")
 
-(defvar css--bang-ids css-bang-ids
+(defvar-local css--bang-ids css-bang-ids
   "List of bang-rules for the current mode.")
-(make-variable-buffer-local 'css--bang-ids)
 
 (defconst css-descriptor-ids
   '("ascent" "baseline" "bbox" "cap-height" "centerline" "definition-src"
@@ -100,7 +98,7 @@
   "Identifiers for types of media.")
 
 (defconst css-property-alist
-  ;; CSS 2.1 properties (http://www.w3.org/TR/CSS21/propidx.html).
+  ;; CSS 2.1 properties (https://www.w3.org/TR/CSS21/propidx.html).
   ;;
   ;; Properties duplicated by any of the CSS3 modules below have been
   ;; removed.
@@ -119,7 +117,6 @@
     ("cue" cue-before cue-after)
     ("cue-after" uri "none")
     ("cue-before" uri "none")
-    ("direction" "ltr" "rtl")
     ("display" "inline" "block" "list-item" "inline-block" "table"
      "inline-table" "table-row-group" "table-header-group"
      "table-footer-group" "table-row" "table-column-group"
@@ -180,7 +177,6 @@
     ("stress" number)
     ("table-layout" "auto" "fixed")
     ("top" length percentage "auto")
-    ("unicode-bidi" "normal" "embed" "bidi-override")
     ("vertical-align" "baseline" "sub" "super" "top" "text-top"
      "middle" "bottom" "text-bottom" percentage length)
     ("visibility" "visible" "hidden" "collapse")
@@ -192,7 +188,7 @@
     ("z-index" "auto" integer)
 
     ;; CSS Animations
-    ;; (http://www.w3.org/TR/css3-animations/#property-index)
+    ;; (https://www.w3.org/TR/css3-animations/#property-index)
     ("animation" single-animation-name time single-timing-function
      single-animation-iteration-count single-animation-direction
      single-animation-fill-mode single-animation-play-state)
@@ -206,7 +202,7 @@
     ("animation-timing-function" single-timing-function)
 
     ;; CSS Backgrounds and Borders Module Level 3
-    ;; (http://www.w3.org/TR/css3-background/#property-index)
+    ;; (https://www.w3.org/TR/css3-background/#property-index)
     ("background" bg-layer final-bg-layer)
     ("background-attachment" attachment)
     ("background-clip" box)
@@ -251,7 +247,7 @@
     ("box-shadow" "none" shadow)
 
     ;; CSS Basic User Interface Module Level 3 (CSS3 UI)
-    ;; (http://www.w3.org/TR/css3-ui/#property-index)
+    ;; (https://www.w3.org/TR/css3-ui/#property-index)
     ("box-sizing" "content-box" "border-box")
     ("caret-color" "auto" color)
     ("cursor" uri x y "auto" "default" "none" "context-menu" "help"
@@ -274,12 +270,17 @@
     ("text-overflow" "clip" "ellipsis" string)
 
     ;; CSS Color Module Level 3
-    ;; (http://www.w3.org/TR/css3-color/#property)
+    ;; (https://www.w3.org/TR/css3-color/#property)
     ("color" color)
     ("opacity" alphavalue)
 
-    ;; CSS Grid Layout Module Level 1
-    ;; (https://www.w3.org/TR/css-grid-1/#property-index)
+    ;; CSS Containment Module Level 2
+    ;; (https://www.w3.org/TR/css-contain-2/#property-index)
+    ("contain" "none" "strict" "content" "size" "layout" "style" "paint")
+    ("content-visibility" "visible" "auto" "hidden")
+
+    ;; CSS Grid Layout Module Level 2
+    ;; (https://www.w3.org/TR/css-grid-2/#property-index)
     ("grid" grid-template grid-template-rows "auto-flow" "dense"
      grid-auto-columns grid-auto-rows grid-template-columns)
     ("grid-area" grid-line)
@@ -298,17 +299,32 @@
     ("grid-template" "none" grid-template-rows grid-template-columns
      line-names string track-size line-names explicit-track-list)
     ("grid-template-areas" "none" string)
-    ("grid-template-columns" "none" track-list auto-track-list)
-    ("grid-template-rows" "none" track-list auto-track-list)
+    ("grid-template-columns" "none" track-list auto-track-list "subgrid")
+    ("grid-template-rows" "none" track-list auto-track-list "subgrid")
 
-    ;; CSS Flexible Box Layout Module Level 1
-    ;; (http://www.w3.org/TR/css-flexbox-1/#property-index)
-    ("align-content" "flex-start" "flex-end" "center" "space-between"
-     "space-around" "stretch")
-    ("align-items" "flex-start" "flex-end" "center" "baseline"
-     "stretch")
-    ("align-self" "auto" "flex-start" "flex-end" "center" "baseline"
-     "stretch")
+    ;; CSS Box Alignment Module Level 3
+    ;; (https://www.w3.org/TR/css-align-3/#property-index)
+    ("align-content"
+     baseline-position content-distribution overflow-position content-position)
+    ("align-items"
+     "normal" "stretch" baseline-position overflow-position self-position)
+    ("align-self"
+     "auto" "normal" "stretch"
+     baseline-position overflow-position self-position)
+    ("justify-content" "normal"
+     content-distribution overflow-position content-position "left" "right")
+    ("justify-items"
+     "normal" "stretch" baseline-position overflow-position self-position
+     "left" "right" "legacy")
+    ("justify-self"
+     "auto" "normal" "stretch" baseline-position overflow-position self-position
+     "left" "right")
+    ("place-content" align-content justify-content)
+    ("place-items" align-items justify-items)
+    ("place-self" justify-self align-self)
+
+    ;; CSS Flexible Box Layout Module Level 2
+    ;; (https://www.w3.org/TR/css-flexbox-2/#property-index)
     ("flex" "none" flex-grow flex-shrink flex-basis)
     ("flex-basis" "auto" "content" width)
     ("flex-direction" "row" "row-reverse" "column" "column-reverse")
@@ -316,12 +332,10 @@
     ("flex-grow" number)
     ("flex-shrink" number)
     ("flex-wrap" "nowrap" "wrap" "wrap-reverse")
-    ("justify-content" "flex-start" "flex-end" "center"
-     "space-between" "space-around")
     ("order" integer)
 
     ;; CSS Fonts Module Level 3
-    ;; (http://www.w3.org/TR/css3-fonts/#property-index)
+    ;; (https://www.w3.org/TR/css3-fonts/#property-index)
     ("font" font-style font-variant-css21 font-weight font-stretch
      font-size line-height font-family "caption" "icon" "menu"
      "message-box" "small-caption" "status-bar")
@@ -417,7 +431,7 @@
     ("columns" column-width column-count)
 
     ;; CSS Overflow Module Level 3
-    ;; (http://www.w3.org/TR/css-overflow-3/#property-index)
+    ;; (https://www.w3.org/TR/css-overflow-3/#property-index)
     ("max-lines" "none" integer)
     ("overflow" "visible" "hidden" "scroll" "auto" "paged-x" "paged-y"
      "paged-x-controls" "paged-y-controls" "fragments")
@@ -427,7 +441,7 @@
      "paged-y" "paged-x-controls" "paged-y-controls" "fragments")
 
     ;; CSS Text Decoration Module Level 3
-    ;; (http://dev.w3.org/csswg/css-text-decor-3/#property-index)
+    ;; (https://dev.w3.org/csswg/css-text-decor-3/#property-index)
     ("text-decoration" text-decoration-line text-decoration-style
      text-decoration-color)
     ("text-decoration-color" color)
@@ -446,7 +460,7 @@
     ("text-underline-position" "auto" "under" "left" "right")
 
     ;; CSS Text Module Level 3
-    ;; (http://www.w3.org/TR/css3-text/#property-index)
+    ;; (https://www.w3.org/TR/css3-text/#property-index)
     ("hanging-punctuation" "none" "first" "force-end" "allow-end"
      "last")
     ("hyphens" "none" "manual" "auto")
@@ -468,7 +482,7 @@
     ("word-wrap" "normal" "break-word")
 
     ;; CSS Transforms Module Level 1
-    ;; (http://www.w3.org/TR/css3-2d-transforms/#property-index)
+    ;; (https://www.w3.org/TR/css3-2d-transforms/#property-index)
     ("backface-visibility" "visible" "hidden")
     ("perspective" "none" length)
     ("perspective-origin" "left" "center" "right" "top" "bottom"
@@ -479,7 +493,7 @@
     ("transform-style" "flat" "preserve-3d")
 
     ;; CSS Transitions
-    ;; (http://www.w3.org/TR/css3-transitions/#property-index)
+    ;; (https://www.w3.org/TR/css3-transitions/#property-index)
     ("transition" single-transition)
     ("transition-delay" time)
     ("transition-duration" time)
@@ -490,8 +504,18 @@
     ;; (https://www.w3.org/TR/css-will-change-1/#property-index)
     ("will-change" "auto" animateable-feature)
 
+    ;; CSS Writing Modes Level 3
+    ;; (https://www.w3.org/TR/css-writing-modes-3/#property-index)
+    ;; "glyph-orientation-vertical" is obsolete and left out.
+    ("direction" "ltr" "rtl")
+    ("text-combine-upright" "none" "all")
+    ("text-orientation" "mixed" "upright" "sideways")
+    ("unicode-bidi" "normal" "embed" "isolate" "bidi-override"
+     "isolate-override" "plaintext")
+    ("writing-mode" "horizontal-tb" "vertical-rl" "vertical-lr")
+
     ;; Filter Effects Module Level 1
-    ;; (http://www.w3.org/TR/filter-effects/#property-index)
+    ;; (https://www.w3.org/TR/filter-effects/#property-index)
     ("color-interpolation-filters" "auto" "sRGB" "linearRGB")
     ("filter" "none" filter-function-list)
     ("flood-color" color)
@@ -747,6 +771,13 @@ further value candidates, since that list would be infinite.")
     (padding-width length percentage)
     (position
      "left" "center" "right" "top" "bottom" percentage length)
+    (baseline-position "left" "right" "baseline")
+    (content-distribution
+     "space-between" "space-around" "space-evenly" "stretch")
+    (overflow-position "unsafe" "safe")
+    (content-position "center" "start" "end" "flex-start" "flex-end")
+    (self-position
+     "center" "start" "end" "self-start" "self-end" "flex-start" "flex-end")
     (radial-gradient "radial-gradient()")
     (relative-size "larger" "smaller")
     (repeat-style
@@ -874,7 +905,7 @@ cannot be completed sensibly: `custom-ident',
 
 (defconst css-escapes-re
   "\\\\\\(?:[^\000-\037\177]\\|[[:xdigit:]]+[ \n\t\r\f]?\\)")
-(defconst css-nmchar-re (concat "\\(?:[-[:alnum:]]\\|" css-escapes-re "\\)"))
+(defconst css-nmchar-re (concat "\\(?:[-_[:alnum:]]\\|" css-escapes-re "\\)"))
 (defconst css-nmstart-re (concat "\\(?:[[:alpha:]]\\|" css-escapes-re "\\)"))
 (defconst css-ident-re ;; (concat css-nmstart-re css-nmchar-re "*")
   ;; Apparently, "at rules" names can start with a dash, e.g. @-moz-keyframes.
@@ -1125,7 +1156,7 @@ by `css--colors-regexp'.  START-POINT is the start of the color,
 and MATCH is the string matched by the regexp.
 
 This function will either return the color, as a hex RGB string;
-or `nil' if no color could be recognized.  When this function
+or nil if no color could be recognized.  When this function
 returns, point will be at the end of the recognized color."
   (cond
    ((eq (aref match 0) ?#)
@@ -1137,20 +1168,9 @@ returns, point will be at the end of the recognized color."
    ;; Evaluate to the color if the name is found.
    ((css--named-color start-point match))))
 
-(defun css--contrasty-color (name)
-  "Return a color that contrasts with NAME.
-NAME is of any form accepted by `color-distance'.
-The returned color will be usable by Emacs and will contrast
-with NAME; in particular so that if NAME is used as a background
-color, the returned color can be used as the foreground and still
-be readable."
-  ;; See bug#25525 for a discussion of this.
-  (if (> (color-distance name "black") 292485)
-      "black" "white"))
-
 (defcustom css-fontify-colors t
   "Whether CSS colors should be fontified using the color as the background.
-When non-`nil', a text representing CSS color will be fontified
+When non-nil, a text representing CSS color will be fontified
 such that its background is the color itself.  E.g., #ff0000 will
 be fontified with a red background."
   :version "26.1"
@@ -1187,7 +1207,8 @@ START and END are buffer positions."
 		    (add-text-properties
 		     start (point)
 		     (list 'face (list :background color
-				       :foreground (css--contrasty-color color)
+				       :foreground (readable-foreground-color
+                                                    color)
 				       :box '(:line-width -1))))))))))))
     extended-region))
 
@@ -1307,10 +1328,14 @@ for determining whether point is within a selector."
     (let ((pos (point)))
       (skip-chars-backward "-[:alnum:]")
       (when (eq (char-before) ?\:)
-        (list (point) pos
-              (if (eq (char-before (- (point) 1)) ?\:)
-                  css-pseudo-element-ids
-                css-pseudo-class-ids))))))
+        (let ((double-colon (eq (char-before (- (point) 1)) ?\:)))
+          (list (- (point) (if double-colon 2 1))
+                pos
+                (nconc
+                 (unless double-colon
+                   (mapcar (lambda (id) (concat ":" id)) css-pseudo-class-ids))
+                 (mapcar (lambda (id) (concat "::" id)) css-pseudo-element-ids))
+                :company-kind (lambda (_) 'function)))))))
 
 (defun css--complete-at-rule ()
   "Complete at-rule (statement beginning with `@') at point."
@@ -1318,7 +1343,8 @@ for determining whether point is within a selector."
     (let ((pos (point)))
       (skip-chars-backward "-[:alnum:]")
       (when (eq (char-before) ?\@)
-        (list (point) pos css--at-ids)))))
+        (list (point) pos css--at-ids
+              :company-kind (lambda (_) 'keyword))))))
 
 (defvar css--property-value-cache
   (make-hash-table :test 'equal :size (length css-property-alist))
@@ -1354,29 +1380,27 @@ the string PROPERTY."
 
 (defun css--complete-property-value ()
   "Complete property value at point."
-  (let ((property
-         (save-excursion
-           (re-search-backward ":[^/]" (line-beginning-position) t)
-           (when (eq (char-after) ?:)
-             (let ((property-end (point)))
-               (skip-chars-backward "-[:alnum:]")
-               (let ((prop (buffer-substring (point) property-end)))
-                 (car (member prop css-property-ids))))))))
+  (let ((property (and (looking-back "\\([[:alnum:]-]+\\):[^/][^;]*"
+                                     (or (ppss-innermost-start (syntax-ppss))
+                                         (point))
+                                     t)
+                       (member (match-string-no-properties 1)
+                               css-property-ids))))
     (when property
       (let ((end (point)))
         (save-excursion
           (skip-chars-backward "[:graph:]")
           (list (point) end
                 (append '("inherit" "initial" "unset")
-                        (css--property-values property))))))))
+                        (css--property-values (car property)))
+                :company-kind (lambda (_) 'value)))))))
 
 (defvar css--html-tags (mapcar #'car html-tag-alist)
   "List of HTML tags.
 Used to provide completion of HTML tags in selectors.")
 
-(defvar css--nested-selectors-allowed nil
+(defvar-local css--nested-selectors-allowed nil
   "Non-nil if nested selectors are allowed in the current mode.")
-(make-variable-buffer-local 'css--nested-selectors-allowed)
 
 (defvar css-class-list-function #'ignore
   "Called to provide completions of class names.
@@ -1438,6 +1462,8 @@ tags, classes and IDs."
                     (list prop-beg prop-end)
                   (list sel-beg sel-end))
               ,(completion-table-merge prop-table sel-table)
+              :company-kind
+              ,(lambda (s) (if (test-completion s prop-table) 'property 'keyword))
               :exit-function
               ,(lambda (string status)
                  (and (eq status 'finished)
@@ -1881,12 +1907,9 @@ on what is seen near point."
    (list
     (let* ((sym (css--mdn-find-symbol))
 	   (enable-recursive-minibuffers t)
-	   (value (completing-read
-		   (if sym
-		       (format "Describe CSS symbol (default %s): " sym)
-		     "Describe CSS symbol: ")
-		   css--mdn-completion-list nil nil nil
-		   'css--mdn-lookup-history sym)))
+	   (value (completing-read (format-prompt "Describe CSS symbol" sym)
+		                   css--mdn-completion-list nil nil nil
+		                   'css--mdn-lookup-history sym)))
       (if (equal value "") sym value))))
   (when symbol
     ;; If we see a single-colon pseudo-element like ":after", turn it
