@@ -1,9 +1,9 @@
-;;; reftex-loaddefs.el --- automatically extracted autoloads
+;;; reftex-loaddefs.el --- automatically extracted autoloads  -*- lexical-binding: t -*-
 ;;
 ;;; Code:
 
 
-;;;### (autoloads nil "reftex-auc" "reftex-auc.el" "e9509ac2bb3d8be96f0c031ca9c54b53")
+;;;### (autoloads nil "reftex-auc" "reftex-auc.el" "71816fc226d522acf93a79f3e4140b9e")
 ;;; Generated autoloads from reftex-auc.el
 
 (autoload 'reftex-arg-label "reftex-auc" "\
@@ -31,15 +31,10 @@ argument identify one of multiple indices.
 
 \(fn OPTIONAL &optional PROMPT &rest ARGS)" nil nil)
 
-(autoload 'reftex-plug-into-AUCTeX "reftex-auc" "\
-
-
-\(fn)" nil nil)
+(autoload 'reftex-plug-into-AUCTeX "reftex-auc" nil nil nil)
 
 (autoload 'reftex-toggle-plug-into-AUCTeX "reftex-auc" "\
-Toggle Interface between AUCTeX and RefTeX on and off.
-
-\(fn)" t nil)
+Toggle Interface between AUCTeX and RefTeX on and off." t nil)
 
 (autoload 'reftex-add-label-environments "reftex-auc" "\
 Add label environment descriptions to `reftex-label-alist-style'.
@@ -54,7 +49,7 @@ the label information is recompiled on next use.
 
 \(fn ENTRY-LIST)" nil nil)
 
-(defalias 'reftex-add-to-label-alist 'reftex-add-label-environments)
+(defalias 'reftex-add-to-label-alist #'reftex-add-label-environments)
 
 (autoload 'reftex-add-section-levels "reftex-auc" "\
 Add entries to the value of `reftex-section-levels'.
@@ -64,36 +59,35 @@ of ENTRY-LIST is a list of cons cells (\"MACRONAME\" . LEVEL).  See
 
 \(fn ENTRY-LIST)" nil nil)
 
-(autoload 'reftex-notice-new-section "reftex-auc" "\
+(autoload 'reftex-notice-new-section "reftex-auc" nil nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-auc" "reftex-auc.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from reftex-auc.el
 
+(register-definition-prefixes "reftex-auc" '("reftex-"))
 
-\(fn)" nil nil)
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-cite" "reftex-cite.el" "e27fae8c6f46ed7deae1e9be85bfcbc8")
+;;;### (autoloads nil "reftex-cite" "reftex-cite.el" "b66d23aea2a7b8b386e0988dfe8c47b8")
 ;;; Generated autoloads from reftex-cite.el
 
 (autoload 'reftex-default-bibliography "reftex-cite" "\
 Return the expanded value of variable `reftex-default-bibliography'.
-The expanded value is cached.
-
-\(fn)" nil nil)
+The expanded value is cached." nil nil)
 
 (autoload 'reftex-bib-or-thebib "reftex-cite" "\
 Test if BibTeX or \\begin{thebibliography} should be used for the citation.
-Find the bof of the current file
-
-\(fn)" nil nil)
+Find the bof of the current file." nil nil)
 
 (autoload 'reftex-get-bibfile-list "reftex-cite" "\
 Return list of bibfiles for current document.
 When using the chapterbib or bibunits package you should either
 use the same database files everywhere, or separate parts using
 different databases into different files (included into the mater file).
-Then this function will return the applicable database files.
-
-\(fn)" nil nil)
+Then this function will return the applicable database files." nil nil)
 
 (autoload 'reftex-pop-to-bibtex-entry "reftex-cite" "\
 Find BibTeX KEY in any file in FILE-LIST in another window.
@@ -127,7 +121,7 @@ If NO-INSERT is non-nil, nothing is inserted, only the selected key returned.
 
 FORMAT-KEY can be used to pre-select a citation format.
 
-When called with a `C-u' prefix, prompt for optional arguments in
+When called with a \\[universal-argument] prefix, prompt for optional arguments in
 cite macros.  When called with a numeric prefix, make that many
 citations.  When called with point inside the braces of a `\\cite'
 command, it will add another key, ignoring the value of
@@ -141,14 +135,10 @@ While entering the regexp, completion on knows citation keys is possible.
 \(fn &optional NO-INSERT FORMAT-KEY)" t nil)
 
 (autoload 'reftex-citep "reftex-cite" "\
-Call `reftex-citation' with a format selector `?p'.
-
-\(fn)" t nil)
+Call `reftex-citation' with a format selector `?p'." t nil)
 
 (autoload 'reftex-citet "reftex-cite" "\
-Call `reftex-citation' with a format selector `?t'.
-
-\(fn)" t nil)
+Call `reftex-citation' with a format selector `?t'." t nil)
 
 (autoload 'reftex-make-cite-echo-string "reftex-cite" "\
 Format a bibtex ENTRY for the echo area and cache the result.
@@ -170,24 +160,35 @@ created files in the variables `reftex-create-bibtex-header' or
 `reftex-create-bibtex-footer' respectively.
 
 \(fn BIBFILE)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-cite"
+;;;;;;  "reftex-cite.el" (0 0 0 0))
+;;; Generated autoloads from reftex-cite.el
+
+(register-definition-prefixes "reftex-cite" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-dcr" "reftex-dcr.el" "049d96c86d8bb4046f003da71201d8d3")
+;;;### (autoloads nil "reftex-dcr" "reftex-dcr.el" "e6a8a9cf5f9727e6fcd10d581653fc6c")
 ;;; Generated autoloads from reftex-dcr.el
 
 (autoload 'reftex-view-crossref "reftex-dcr" "\
-View cross reference of macro at point.  Point must be on the KEY
-argument.  When at a `\\ref' macro, show corresponding `\\label'
-definition, also in external documents (`xr').  When on a label, show
-a locations where KEY is referenced.  Subsequent calls find additional
-locations.  When on a `\\cite', show the associated `\\bibitem' macro or
-the BibTeX database entry.  When on a `\\bibitem', show a `\\cite' macro
-which uses this KEY. When on an `\\index', show other locations marked
-by the same index entry.
+View cross reference of macro at point.
+
+Point must be on the KEY argument.  When at a `\\ref' macro, show
+corresponding `\\label' definition, also in external
+documents (`xr').  When on a label, show a locations where KEY is
+referenced.  Subsequent calls find additional locations.  When on
+a `\\cite', show the associated `\\bibitem' macro or the BibTeX
+database entry.  When on a `\\bibitem', show a `\\cite' macro
+which uses this KEY.  When on an `\\index', show other locations
+marked by the same index entry.
+
 To define additional cross referencing items, use the option
 `reftex-view-crossref-extra'.  See also `reftex-view-crossref-from-bibtex'.
-With one or two C-u prefixes, enforce rescanning of the document.
+With one or two \\[universal-argument] prefixes, enforce rescanning of the document.
 With argument 2, select the window showing the cross reference.
 AUTO-HOW is only for the automatic crossref display and is handed through
 to the functions `reftex-view-cr-cite' and `reftex-view-cr-ref'.
@@ -206,9 +207,7 @@ With argument, actually select the window showing the cross reference.
 (autoload 'reftex-toggle-auto-view-crossref "reftex-dcr" "\
 Toggle the automatic display of crossref information in the echo area.
 When active, leaving point idle in the argument of a \\ref or \\cite macro
-will display info in the echo area.
-
-\(fn)" t nil)
+will display info in the echo area." t nil)
 
 (autoload 'reftex-view-crossref-from-bibtex "reftex-dcr" "\
 View location in a LaTeX document which cites the BibTeX entry at point.
@@ -218,17 +217,23 @@ link to a document, call the function with a prefix arg.
 Calling this function several times find successive citation locations.
 
 \(fn &optional ARG)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-dcr" "reftex-dcr.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from reftex-dcr.el
+
+(register-definition-prefixes "reftex-dcr" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-global" "reftex-global.el" "84c1d2d615d63b1b3849391ebb4c7a1c")
+;;;### (autoloads nil "reftex-global" "reftex-global.el" "f53a5da243ff4e38db09eafb3cb2d889")
 ;;; Generated autoloads from reftex-global.el
 
 (autoload 'reftex-create-tags-file "reftex-global" "\
 Create TAGS file by running `etags' on the current document.
-The TAGS file is also immediately visited with `visit-tags-table'.
-
-\(fn)" t nil)
+The TAGS file is also immediately visited with `visit-tags-table'." t nil)
 
 (autoload 'reftex-grep-document "reftex-global" "\
 Run grep query through all files related to this document.
@@ -255,9 +260,7 @@ No active TAGS table is required.
 \(fn &optional FROM TO DELIMITED)" t nil)
 
 (autoload 'reftex-find-duplicate-labels "reftex-global" "\
-Produce a list of all duplicate labels in the document.
-
-\(fn)" t nil)
+Produce a list of all duplicate labels in the document." t nil)
 
 (autoload 'reftex-change-label "reftex-global" "\
 Run `query-replace-regexp' of FROM with TO in all macro arguments.
@@ -277,16 +280,12 @@ the labels will be changed as well.  For this, RefTeX looks at the
 arguments of any macros which either start or end in the string `ref'.
 This command should be used with care, in particular in multifile
 documents.  You should not use it if another document refers to this
-one with the `xr' package.
-
-\(fn)" t nil)
+one with the `xr' package." t nil)
 
 (autoload 'reftex-save-all-document-buffers "reftex-global" "\
 Save all documents associated with the current document.
 The function is useful after a global action like replacing or renumbering
-labels.
-
-\(fn)" t nil)
+labels." t nil)
 
 (autoload 'reftex-isearch-minor-mode "reftex-global" "\
 When on, isearch searches the whole document, not only the current file.
@@ -298,18 +297,26 @@ With no argument, this command toggles
 `reftex-isearch-minor-mode' on if ARG is positive, otherwise turn it off.
 
 \(fn &optional ARG)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-global"
+;;;;;;  "reftex-global.el" (0 0 0 0))
+;;; Generated autoloads from reftex-global.el
+
+(register-definition-prefixes "reftex-global" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-index" "reftex-index.el" "44d4e7e86f04da3173a602734ebad660")
+;;;### (autoloads nil "reftex-index" "reftex-index.el" "6cb8d2501dcf169558255d584606258e")
 ;;; Generated autoloads from reftex-index.el
 
 (autoload 'reftex-index-selection-or-word "reftex-index" "\
 Put selection or the word near point into the default index macro.
 This uses the information in `reftex-index-default-macro' to make an index
 entry.  The phrase indexed is the current selection or the word near point.
-When called with one `C-u' prefix, let the user have a chance to edit the
-index entry.  When called with 2 `C-u' as prefix, also ask for the index
+When called with one \\[universal-argument] prefix, let the user have a chance to edit the
+index entry.  When called with 2 \\[universal-argument] as prefix, also ask for the index
 macro and other stuff.
 When called inside TeX math mode as determined by the `texmathp.el' library
 which is part of AUCTeX, the string is first processed with the
@@ -331,10 +338,7 @@ will prompt for other arguments.
 
 \(fn &optional ITAG OPT-ARGS)" nil nil)
 
-(autoload 'reftex-index-select-tag "reftex-index" "\
-
-
-\(fn)" nil nil)
+(autoload 'reftex-index-select-tag "reftex-index" nil nil nil)
 
 (autoload 'reftex-index-complete-key "reftex-index" "\
 
@@ -350,7 +354,7 @@ will prompt for other arguments.
 Display a buffer with an index compiled from the current document.
 When the document has multiple indices, first prompts for the correct one.
 When index support is turned off, offer to turn it on.
-With one or two `C-u' prefixes, rescan document first.
+With one or two \\[universal-argument] prefixes, rescan document first.
 With prefix 2, restrict index to current document section.
 With prefix 3, restrict index to region.
 
@@ -366,9 +370,7 @@ You get a chance to edit the entry in the phrases buffer - finish with
 \(fn ARG)" t nil)
 
 (autoload 'reftex-index-visit-phrases-buffer "reftex-index" "\
-Switch to the phrases buffer, initialize if empty.
-
-\(fn)" t nil)
+Switch to the phrases buffer, initialize if empty." t nil)
 
 (autoload 'reftex-index-phrases-mode "reftex-index" "\
 Major mode for managing the Index phrases of a LaTeX document.
@@ -396,21 +398,25 @@ Here are all local bindings.
 \\{reftex-index-phrases-mode-map}
 
 \(fn)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-index"
+;;;;;;  "reftex-index.el" (0 0 0 0))
+;;; Generated autoloads from reftex-index.el
+
+(register-definition-prefixes "reftex-index" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-parse" "reftex-parse.el" "bb0eee4736c10bd8edad72a3aec3d347")
+;;;### (autoloads nil "reftex-parse" "reftex-parse.el" "85373955924c7f1e7ac020e478cb7e38")
 ;;; Generated autoloads from reftex-parse.el
 
 (autoload 'reftex-parse-one "reftex-parse" "\
-Re-parse this file.
-
-\(fn)" t nil)
+Re-parse this file." t nil)
 
 (autoload 'reftex-parse-all "reftex-parse" "\
-Re-parse entire document.
-
-\(fn)" t nil)
+Re-parse entire document." t nil)
 
 (autoload 'reftex-do-parse "reftex-parse" "\
 Do a document rescan.
@@ -418,10 +424,7 @@ When allowed, do only a partial scan from FILE.
 
 \(fn RESCAN &optional FILE)" nil nil)
 
-(autoload 'reftex-everything-regexp "reftex-parse" "\
-
-
-\(fn)" nil nil)
+(autoload 'reftex-everything-regexp "reftex-parse" nil nil nil)
 
 (autoload 'reftex-all-document-files "reftex-parse" "\
 Return a list of all files belonging to the current document.
@@ -437,7 +440,8 @@ Scan buffer for bibliography macros and return file list.
 
 (autoload 'reftex-section-info "reftex-parse" "\
 Return a section entry for the current match.
-Careful: This function expects the match-data to be still in place!
+Careful: This function expects the `match-data' to still be in
+place!
 
 \(fn FILE)" nil nil)
 
@@ -455,7 +459,8 @@ will rescan the entire document.
 
 (autoload 'reftex-index-info "reftex-parse" "\
 Return an index entry for the current match.
-Careful: This function expects the match-data to be still in place!
+Careful: This function expects the `match-data' to still be in
+place!
 
 \(fn FILE)" nil nil)
 
@@ -467,9 +472,7 @@ Get about one line of useful context for the label definition at point.
 (autoload 'reftex-where-am-I "reftex-parse" "\
 Return the docstruct entry above point.
 Actually returns a cons cell in which the cdr is a flag indicating
-if the information is exact (t) or approximate (nil).
-
-\(fn)" nil nil)
+if the information is exact (t) or approximate (nil)." nil nil)
 
 (autoload 'reftex-notice-new "reftex-parse" "\
 Hook to handshake with RefTeX after something new has been inserted.
@@ -539,13 +542,10 @@ OPT-ARGS is a list of argument numbers which are optional.
 
 \(fn N &optional OPT-ARGS)" nil nil)
 
-(autoload 'reftex-move-over-touching-args "reftex-parse" "\
-
-
-\(fn)" nil nil)
+(autoload 'reftex-move-over-touching-args "reftex-parse" nil nil nil)
 
 (autoload 'reftex-init-section-numbers "reftex-parse" "\
-Initialize the section numbers with zeros or with what is found in the TOC-ENTRY.
+Initialize section numbers with zeros or with what is found in the TOC-ENTRY.
 
 \(fn &optional TOC-ENTRY APPENDIX)" nil nil)
 
@@ -554,10 +554,18 @@ Return a string with the current section number.
 When LEVEL is non-nil, increase section numbers on that level.
 
 \(fn &optional LEVEL STAR)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-parse"
+;;;;;;  "reftex-parse.el" (0 0 0 0))
+;;; Generated autoloads from reftex-parse.el
+
+(register-definition-prefixes "reftex-parse" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-ref" "reftex-ref.el" "d00bce646455f31050ffa7c72e03bd40")
+;;;### (autoloads nil "reftex-ref" "reftex-ref.el" "e2dd102dbb852e0e98034a40956efc7f")
 ;;; Generated autoloads from reftex-ref.el
 
 (autoload 'reftex-label-location "reftex-ref" "\
@@ -596,14 +604,11 @@ reftex-label.  Rescanning of the buffer can also be requested from the
 label selection menu.
 The function returns the selected label or nil.
 If NO-INSERT is non-nil, do not insert \\ref command, just return label.
-When called with 2 C-u prefix args, disable magic word recognition.
+When called with 2 \\[universal-argument] prefix args, disable magic word recognition.
 
 \(fn &optional TYPE NO-INSERT CUT)" t nil)
 
-(autoload 'reftex-query-label-type "reftex-ref" "\
-
-
-\(fn)" nil nil)
+(autoload 'reftex-query-label-type "reftex-ref" nil nil nil)
 
 (autoload 'reftex-show-label-location "reftex-ref" "\
 
@@ -615,10 +620,18 @@ Prompt for a label (with completion) and jump to the location of this label.
 Optional prefix argument OTHER-WINDOW goes to the label in another window.
 
 \(fn &optional OTHER-WINDOW)" t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-ref" "reftex-ref.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from reftex-ref.el
+
+(register-definition-prefixes "reftex-ref" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-sel" "reftex-sel.el" "b7356613e0fd509778ab4d1c13f94ff1")
+;;;### (autoloads nil "reftex-sel" "reftex-sel.el" "e039a6f1b790cc064bb4a11cc577ea6c")
 ;;; Generated autoloads from reftex-sel.el
 
 (autoload 'reftex-select-label-mode "reftex-sel" "\
@@ -667,16 +680,24 @@ During a selection process, these are the local bindings.
 (autoload 'reftex-select-item "reftex-sel" "\
 
 
-\(fn REFTEX-SELECT-PROMPT HELP-STRING KEYMAP &optional OFFSET CALL-BACK CB-FLAG)" nil nil)
+\(fn PROMPT HELP-STRING KEYMAP &optional OFFSET CALL-BACK CB-FLAG)" nil nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-sel" "reftex-sel.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from reftex-sel.el
+
+(register-definition-prefixes "reftex-sel" '("reftex-"))
+
+;;;***
 
 ;;;***
 
-;;;### (autoloads nil "reftex-toc" "reftex-toc.el" "57c999f7b6085d021699c77af85b7225")
+;;;### (autoloads nil "reftex-toc" "reftex-toc.el" "f28a987009cd9c06c2ad6b1496e5b36f")
 ;;; Generated autoloads from reftex-toc.el
 
 (autoload 'reftex-toc "reftex-toc" "\
 Show the table of contents for the current document.
-When called with a raw C-u prefix, rescan the document first.
+When called with a raw \\[universal-argument] prefix, rescan the document first.
 
 \(fn &optional REBUILD REUSE)" t nil)
 
@@ -688,9 +709,15 @@ Display the TOC window and highlight line corresponding to current position.
 (autoload 'reftex-toggle-auto-toc-recenter "reftex-toc" "\
 Toggle the automatic recentering of the TOC window.
 When active, leaving point idle will make the TOC window jump to the correct
-section.
+section." t nil)
+
+;;;### (autoloads "actual autoloads are elsewhere" "reftex-toc" "reftex-toc.el"
+;;;;;;  (0 0 0 0))
+;;; Generated autoloads from reftex-toc.el
 
-\(fn)" t nil)
+(register-definition-prefixes "reftex-toc" '("reftex-"))
+
+;;;***
 
 ;;;***
 
