@@ -1,4 +1,4 @@
-;;; tooltip.el --- show tooltip windows
+;;; tooltip.el --- show tooltip windows  -*- lexical-binding:t -*-
 
 ;; Copyright (C) 1997, 1999-2021 Free Software Foundation, Inc.
 
@@ -70,24 +70,20 @@ echo area, instead of making a pop-up window."
 
 (defcustom tooltip-delay 0.7
   "Seconds to wait before displaying a tooltip the first time."
-  :type 'number
-  :group 'tooltip)
+  :type 'number)
 
 (defcustom tooltip-short-delay 0.1
   "Seconds to wait between subsequent tooltips on different items."
-  :type 'number
-  :group 'tooltip)
+  :type 'number)
 
 (defcustom tooltip-recent-seconds 1
   "Display tooltips if changing tip items within this many seconds.
 Do so after `tooltip-short-delay'."
-  :type 'number
-  :group 'tooltip)
+  :type 'number)
 
 (defcustom tooltip-hide-delay 10
   "Hide tooltips automatically after this many seconds."
-  :type 'number
-  :group 'tooltip)
+  :type 'number)
 
 (defcustom tooltip-x-offset 5
   "X offset, in pixels, for the display of tooltips.
@@ -98,8 +94,7 @@ interfere with clicking where you wish.
 
 If `tooltip-frame-parameters' includes the `left' parameter,
 the value of `tooltip-x-offset' is ignored."
-  :type 'integer
-  :group 'tooltip)
+  :type 'integer)
 
 (defcustom tooltip-y-offset +20
   "Y offset, in pixels, for the display of tooltips.
@@ -110,8 +105,7 @@ interfere with clicking where you wish.
 
 If `tooltip-frame-parameters' includes the `top' parameter,
 the value of `tooltip-y-offset' is ignored."
-  :type 'integer
-  :group 'tooltip)
+  :type 'integer)
 
 (defcustom tooltip-frame-parameters
   '((name . "tooltip")
@@ -127,8 +121,7 @@ Note that font and color parameters are ignored, and the attributes
 of the `tooltip' face are used instead."
   :type '(repeat (cons :format "%v"
 		       (symbol :tag "Parameter")
-		       (sexp :tag "Value")))
-  :group 'tooltip
+                       (sexp :tag "Value")))
   :version "26.1")
 
 (defface tooltip
@@ -149,8 +142,7 @@ When using the GTK toolkit, this face will only be used if
   "Use the echo area instead of tooltip frames for help and GUD tooltips.
 This variable is obsolete; instead of setting it to t, disable
 `tooltip-mode' (which has a similar effect)."
-  :type 'boolean
-  :group 'tooltip)
+  :type 'boolean)
 
 (make-obsolete-variable 'tooltip-use-echo-area
 			"disable Tooltip mode instead" "24.1" 'set)
@@ -164,14 +156,11 @@ the echo area is resized as needed to accommodate the full text
 of the tooltip.
 This variable has effect only on GUI frames."
   :type 'boolean
-  :group 'tooltip
   :version "27.1")
 
 
 ;;; Variables that are not customizable.
 
-(define-obsolete-variable-alias 'tooltip-hook 'tooltip-functions "23.1")
-
 (defvar tooltip-functions nil
   "Functions to call to display tooltips.
 Each function is called with one argument EVENT which is a copy
@@ -263,7 +252,12 @@ in echo area."
 	    (setf (alist-get 'border-color params) fg))
 	  (when (stringp bg)
 	    (setf (alist-get 'background-color params) bg))
-	  (x-show-tip (propertize text 'face 'tooltip)
+          ;; Use non-nil APPEND argument below to avoid overriding any
+          ;; faces used in our TEXT.  Among other things, this allows
+          ;; tooltips to use the `help-key-binding' face used in
+          ;; `substitute-command-keys' substitutions.
+          (add-face-text-property 0 (length text) 'tooltip t text)
+          (x-show-tip text
 		      (selected-frame)
 		      params
 		      tooltip-hide-delay
@@ -352,7 +346,7 @@ It is also called if Tooltip mode is on, for text-only displays."
              (not cursor-in-echo-area))  ;Don't overwrite a prompt.
     (cond
      ((stringp help)
-      (setq help (replace-regexp-in-string "\n" ", " help))
+      (setq help (string-replace "\n" ", " help))
       (unless (or tooltip-previous-message
 		  (equal-including-properties help (current-message))
 		  (and (stringp tooltip-help-message)
