@@ -1,6 +1,6 @@
-;;; url-dav.el --- WebDAV support
+;;; url-dav.el --- WebDAV support  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2001, 2004-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2001, 2004-2021 Free Software Foundation, Inc.
 
 ;; Author: Bill Perry <wmperry@gnu.org>
 ;; Maintainer: emacs-devel@gnu.org
@@ -33,6 +33,7 @@
 (require 'url-util)
 (require 'url-handlers)
 (require 'url-http)
+(require 'parse-time)
 
 (defvar url-dav-supported-protocols '(1 2)
   "List of supported DAV versions.")
@@ -42,22 +43,11 @@
 (defvar url-http-response-status)
 (defvar url-http-end-of-headers)
 
-(defun url-intersection (l1 l2)
-  "Return a list of the elements occurring in both of the lists L1 and L2."
-  (if (null l2)
-      l2
-    (let (result)
-      (while l1
-	(if (member (car l1) l2)
-	    (setq result (cons (pop l1) result))
-	  (pop l1)))
-      (nreverse result))))
-
 ;;;###autoload
 (defun url-dav-supported-p (url)
   "Return WebDAV protocol version supported by URL.
 Returns nil if WebDAV is not supported."
-  (url-intersection url-dav-supported-protocols
+  (seq-intersection url-dav-supported-protocols
 		    (plist-get (url-http-options url) 'dav)))
 
 (defun url-dav-node-text (node)
@@ -83,72 +73,14 @@ Returns nil if WebDAV is not supported."
 (defun url-dav-process-number-property (node)
   (string-to-number (url-dav-node-text node)))
 
-(defconst url-dav-iso8601-regexp
-  (let* ((dash "-?")
-	 (colon ":?")
-	 (4digit "\\([0-9][0-9][0-9][0-9]\\)")
-	 (2digit "\\([0-9][0-9]\\)")
-	 (date-fullyear 4digit)
-	 (date-month 2digit)
-	 (date-mday 2digit)
-	 (time-hour 2digit)
-	 (time-minute 2digit)
-	 (time-second 2digit)
-	 (time-secfrac "\\(\\.[0-9]+\\)?")
-	 (time-numoffset (concat "[-+]\\(" time-hour "\\):" time-minute))
-	 (time-offset (concat "Z" time-numoffset))
-	 (partial-time (concat time-hour colon time-minute colon time-second
-			       time-secfrac))
-	 (full-date (concat date-fullyear dash date-month dash date-mday))
-	 (full-time (concat partial-time time-offset))
-	 (date-time (concat full-date "T" full-time)))
-    (list (concat "^" full-date)
-	  (concat "T" partial-time)
-	  (concat "Z" time-numoffset)))
-  "List of regular expressions matching ISO 8601 dates.
-1st regular expression matches the date.
-2nd regular expression matches the time.
-3rd regular expression matches the (optional) timezone specification.")
-
 (defun url-dav-process-date-property (node)
-  (require 'parse-time)
-  (let* ((date-re (nth 0 url-dav-iso8601-regexp))
-	 (time-re (nth 1 url-dav-iso8601-regexp))
-	 (tz-re (nth 2 url-dav-iso8601-regexp))
-	 (date-string (url-dav-node-text node))
-	 re-start
-	 time seconds minute hour fractional-seconds
-	 day month year day-of-week dst tz)
-    ;; We need to populate 'time' with
-    ;; (SEC MIN HOUR DAY MON YEAR DOW DST TZ)
-
-    ;; Nobody else handles iso8601 correctly, let's do it ourselves.
-    (when (string-match date-re date-string re-start)
-      (setq year (string-to-number (match-string 1 date-string))
-	    month (string-to-number (match-string 2 date-string))
-	    day (string-to-number (match-string 3 date-string))
-	    re-start (match-end 0))
-      (when (string-match time-re date-string re-start)
-	(setq hour (string-to-number (match-string 1 date-string))
-	      minute (string-to-number (match-string 2 date-string))
-	      seconds (string-to-number (match-string 3 date-string))
-	      fractional-seconds (string-to-number (or
-                                                    (match-string 4 date-string)
-                                                    "0"))
-	      re-start (match-end 0))
-	(when (string-match tz-re date-string re-start)
-	  (setq tz (match-string 1 date-string)))
-	(url-debug 'dav "Parsed iso8601%s date" (if tz "tz" ""))
-	(setq time (list seconds minute hour day month year day-of-week dst tz))))
-
-    ;; Fall back to having Gnus do fancy things for us.
-    (when (not time)
-      (setq time (parse-time-string date-string)))
-
+  (let* ((date-string (url-dav-node-text node))
+         (time (parse-iso8601-time-string date-string)))
     (if time
-	(setq time (apply 'encode-time time))
+	(setq time (encode-time time))
       (url-debug 'dav "Unable to decode date (%S) (%s)"
-		 (xml-node-name node) date-string))
+		 (xml-node-name node)
+                 date-string))
     time))
 
 (defun url-dav-process-boolean-property (node)
@@ -190,7 +122,8 @@ Returns nil if WebDAV is not supported."
 	(node-type nil)
 	(props nil)
 	(value nil)
-	(handler-func nil))
+	;; (handler-func nil)
+	)
     (when (not children)
       (error "No child nodes in DAV:prop"))
 
@@ -204,22 +137,22 @@ Returns nil if WebDAV is not supported."
 	    value nil)
 
       (pcase node-type
-	((or `dateTime.iso8601tz
-             `dateTime.iso8601
-             `dateTime.tz
-             `dateTime.rfc1123
-             `dateTime
-             `date)                     ; date is our 'special' one...
+	((or 'dateTime.iso8601tz
+             'dateTime.iso8601
+             'dateTime.tz
+             'dateTime.rfc1123
+             'dateTime
+             'date)                     ; date is our 'special' one...
 	 ;; Some type of date/time string.
 	 (setq value (url-dav-process-date-property node)))
-	(`int
+	('int
 	 ;; Integer type...
 	 (setq value (url-dav-process-integer-property node)))
-	((or `number `float)
+	((or 'number 'float)
 	 (setq value (url-dav-process-number-property node)))
-	(`boolean
+	('boolean
 	 (setq value (url-dav-process-boolean-property node)))
-	(`uri
+	('uri
 	 (setq value (url-dav-process-uri-property node)))
 	(_
 	 (if (not (eq node-type 'unknown))
@@ -510,7 +443,7 @@ FAILURE-RESULTS is a list of (URL STATUS)."
 	   "  </DAV:owner>\n"))
 	 (response nil)		  ; Responses to the LOCK request
 	 (result nil)		  ; For walking thru the response list
-	 (child-url nil)
+	 ;; (child-url nil)
 	 (child-status nil)
 	 (failures nil)		  ; List of failure cases (URL . STATUS)
 	 (successes nil))	  ; List of success cases (URL . STATUS)
@@ -525,7 +458,7 @@ FAILURE-RESULTS is a list of (URL STATUS)."
     ;; status code.
     (while response
       (setq result (pop response)
-	    child-url (url-expand-file-name (pop result) url)
+	    ;; child-url (url-expand-file-name (pop result) url)
 	    child-status (or (plist-get result 'DAV:status) 500))
       (if (url-dav-http-success-p child-status)
 	  (push (list url child-status "huh") successes)
@@ -535,7 +468,7 @@ FAILURE-RESULTS is a list of (URL STATUS)."
 (defun url-dav-active-locks (url &optional depth)
   "Return an assoc list of all active locks on URL."
   (let ((response (url-dav-get-properties url '(DAV:lockdiscovery) depth))
-	(properties nil)
+	;; (properties nil)
 	(child nil)
 	(child-url nil)
 	(child-results nil)
@@ -611,11 +544,11 @@ Returns t if the lock was successfully released."
       (setq lock (car supported-locks)
 	    supported-locks (cdr supported-locks))
       (pcase (car lock)
-	(`DAV:write
+	('DAV:write
 	 (pcase (cdr lock)
-	   (`DAV:shared			; group permissions (possibly world)
+	   ('DAV:shared			; group permissions (possibly world)
 	    (aset modes 5 ?w))
-	   (`DAV:exclusive
+	   ('DAV:exclusive
 	    (aset modes 2 ?w))		; owner permissions?
 	   (_
 	    (url-debug 'dav "Unrecognized DAV:lockscope (%S)" (cdr lock)))))
@@ -733,7 +666,6 @@ Use with care, and even then think three times."
 If optional second argument RECURSIVE is non-nil, then delete all
 files in the collection as well."
   (let ((status nil)
-	(props nil)
 	(props nil))
     (setq props (url-dav-delete-something
 		 url lock-token
@@ -826,7 +758,7 @@ If NOSORT is non-nil, the list is not sorted--its order is unpredictable.
     (when (member 'DAV:collection (plist-get properties 'DAV:resourcetype))
       t)))
 
-(defun url-dav-make-directory (url &optional parents)
+(defun url-dav-make-directory (url &optional _parents)
   "Create the directory DIR and any nonexistent parent dirs."
   (let* ((url-request-extra-headers nil)
 	 (url-request-method "MKCOL")
@@ -906,7 +838,9 @@ that start with FILE.
 If there is only one and FILE matches it exactly, returns t.
 Returns nil if URL contains no name starting with FILE."
   (let ((matches (url-dav-file-name-all-completions file url))
-	(result nil))
+	;; (result nil)
+	)
+    ;; FIXME: Use `try-completion'!
     (cond
      ((null matches)
       ;; No matches
@@ -965,7 +899,9 @@ Returns nil if URL contains no name starting with FILE."
 	t)))
 
 
-;;; Miscellaneous stuff.
+;;; Obsolete.
+
+(define-obsolete-function-alias 'url-intersection #'seq-intersection "28.1")
 
 (provide 'url-dav)
 
