@@ -1,6 +1,6 @@
-;;; ruby-mode-tests.el --- Test suite for ruby-mode
+;;; ruby-mode-tests.el --- Test suite for ruby-mode  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2012-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2012-2021 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -22,6 +22,7 @@
 ;;; Code:
 
 (require 'ert)
+(require 'ert-x)
 (require 'ruby-mode)
 
 (defmacro ruby-with-temp-buffer (contents &rest body)
@@ -31,6 +32,13 @@
      (ruby-mode)
      ,@body))
 
+(defmacro ruby-with-temp-file (contents &rest body)
+  `(ruby-with-temp-buffer ,contents
+     (set-visited-file-name "ruby-mode-tests")
+     ,@body
+     (set-buffer-modified-p nil)
+     (delete-file buffer-file-name)))
+
 (defun ruby-should-indent (content column)
   "Assert indentation COLUMN on the last line of CONTENT."
   (ruby-with-temp-buffer content
@@ -349,7 +357,7 @@ VALUES-PLIST is a list with alternating index and value elements."
   (let ((ruby-align-chained-calls t))
     (ruby-should-indent-buffer
      "one.two.three
-     |       .four
+     |   .four
      |
      |my_array.select { |str| str.size > 5 }
      |        .map    { |str| str.downcase }"
@@ -496,7 +504,8 @@ VALUES-PLIST is a list with alternating index and value elements."
 (ert-deftest ruby-add-log-current-method-examples ()
   (let ((pairs '(("foo" . "#foo")
                  ("C.foo" . ".foo")
-                 ("self.foo" . ".foo"))))
+                 ("self.foo" . ".foo")
+                 ("<<" . "#<<"))))
     (dolist (pair pairs)
       (let ((name  (car pair))
             (value (cdr pair)))
@@ -711,7 +720,7 @@ VALUES-PLIST is a list with alternating index and value elements."
   (ruby-with-temp-buffer ruby-sexp-test-example
     (goto-char (point-min))
     (forward-line 1)
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (= 8 (line-number-at-pos)))))
 
 (ert-deftest ruby-backward-sexp-skips-method-calls-with-keyword-names ()
@@ -719,7 +728,7 @@ VALUES-PLIST is a list with alternating index and value elements."
     (goto-char (point-min))
     (forward-line 7)
     (end-of-line)
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (= 2 (line-number-at-pos)))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-no-args ()
@@ -728,7 +737,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do
      |end")
     (search-backward "do\n")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-no-args ()
@@ -737,7 +746,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do$"))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-empty-args ()
@@ -746,7 +755,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do ||
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-empty-args ()
@@ -755,7 +764,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do ||
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-args ()
@@ -764,7 +773,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,b|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-args ()
@@ -773,7 +782,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,b|
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-any-args ()
@@ -782,7 +791,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |*|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-expanded-one-arg ()
@@ -791,7 +800,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-one-and-any-args ()
@@ -800,7 +809,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,*|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-one-and-any-args ()
@@ -809,7 +818,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,*|
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-toggle-string-quotes-quotes-correctly ()
@@ -842,6 +851,89 @@ VALUES-PLIST is a list with alternating index and value elements."
       (ruby--insert-coding-comment "utf-8")
       (should (string= "# encoding: utf-8\n\n" (buffer-string))))))
 
+(ert-deftest ruby--set-encoding-when-ascii ()
+  (ruby-with-temp-file "ascii"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'us-ascii)
+      (ruby-mode-set-encoding)
+      (should (string= "ascii" (buffer-string))))))
+
+(ert-deftest ruby--set-encoding-when-utf8 ()
+  (ruby-with-temp-file "💎"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'utf-8)
+      (ruby-mode-set-encoding)
+      (should (string= "💎" (buffer-string))))))
+
+(ert-deftest ruby--set-encoding-when-latin-15 ()
+  (ruby-with-temp-file "Ⓡ"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'iso-8859-15)
+      (ruby-mode-set-encoding)
+      (should (string= "# coding: iso-8859-15\nⓇ" (buffer-string))))))
+
+(ert-deftest ruby-imenu-with-private-modifier ()
+  (ruby-with-temp-buffer
+      (ruby-test-string
+       "class Blub
+       |  def hi
+       |    'Hi!'
+       |  end
+       |
+       |  def bye
+       |    'Bye!'
+       |  end
+       |
+       |  private def hiding
+       |    'You can't see me'
+       |  end
+       |end")
+    (should (equal (mapcar #'car (ruby-imenu-create-index))
+                   '("Blub"
+                     "Blub#hi"
+                     "Blub#bye"
+                     "Blub#hiding")))))
+
+(ert-deftest ruby--indent/converted-from-manual-test ()
+  :tags '(:expensive-test)
+  ;; Converted from manual test.
+  (let ((buf (find-file-noselect (ert-resource-file "ruby.rb"))))
+    (unwind-protect
+        (with-current-buffer buf
+          (let ((orig (buffer-string)))
+            (indent-region (point-min) (point-max))
+            (should (equal (buffer-string) orig))))
+      (kill-buffer buf))))
+
+(ert-deftest ruby--test-chained-indentation ()
+  (with-temp-buffer
+    (ruby-mode)
+    (setq-local ruby-align-chained-calls t)
+    (insert "some_variable.where
+.not(x: nil)
+.where(y: 2)
+")
+    (indent-region (point-min) (point-max))
+    (should (equal (buffer-string)
+                   "some_variable.where
+             .not(x: nil)
+             .where(y: 2)
+")))
+
+  (with-temp-buffer
+    (ruby-mode)
+    (setq-local ruby-align-chained-calls t)
+    (insert "some_variable.where.not(x: nil)
+.where(y: 2)
+")
+    (indent-region (point-min) (point-max))
+    (should (equal (buffer-string)
+                   "some_variable.where.not(x: nil)
+             .where(y: 2)
+"))))
 
 (provide 'ruby-mode-tests)
 
