@@ -1,4 +1,4 @@
-;;; decompress-tests.el --- Test suite for decompress.
+;;; decompress-tests.el --- Test suite for decompress. -*- lexical-binding: t -*-
 
 ;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
@@ -29,17 +29,17 @@
 
 (ert-deftest zlib--decompress ()
   "Test decompressing a gzipped file."
-  (when (and (fboundp 'zlib-available-p)
-	     (zlib-available-p))
-    (should (string=
-	     (with-temp-buffer
-	       (set-buffer-multibyte nil)
-	       (insert-file-contents-literally
-		(expand-file-name "foo.gz" zlib-tests-data-directory))
-	       (zlib-decompress-region (point-min) (point-max))
-	       (buffer-string))
-	     "foo\n"))))
+  (skip-unless (and (fboundp 'zlib-available-p)
+                    (zlib-available-p)))
+  (should (string=
+           (with-temp-buffer
+             (set-buffer-multibyte nil)
+             (insert-file-contents-literally
+              (expand-file-name "foo.gz" zlib-tests-data-directory))
+             (zlib-decompress-region (point-min) (point-max))
+             (buffer-string))
+           "foo\n")))
 
 (provide 'decompress-tests)
 
-;;; decompress-tests.el ends here.
+;;; decompress-tests.el ends here
